/**
 * Copyright (c) 2000-2009 Liferay, Inc. All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package com.liferay.mail.service;

import com.liferay.portal.PortalException;
import com.liferay.portal.SystemException;
import com.liferay.portal.kernel.annotation.Isolation;
import com.liferay.portal.kernel.annotation.Propagation;
import com.liferay.portal.kernel.annotation.Transactional;

/**
 * <a href="AccountEntryLocalService.java.html"><b><i>View Source</i></b></a>
 *
 * @author Scott Lee
 */
@Transactional(isolation = Isolation.PORTAL, rollbackFor =  {
	PortalException.class, SystemException.class})
public interface AccountEntryLocalService {
	public com.liferay.mail.model.AccountEntry addAccountEntry(
		com.liferay.mail.model.AccountEntry accountEntry)
		throws com.liferay.portal.SystemException;

	public com.liferay.mail.model.AccountEntry createAccountEntry(
		long accountEntryId);

	public void deleteAccountEntry(long accountEntryId)
		throws com.liferay.portal.SystemException,
			com.liferay.portal.PortalException;

	public void deleteAccountEntry(
		com.liferay.mail.model.AccountEntry accountEntry)
		throws com.liferay.portal.SystemException;

	public java.util.List<Object> dynamicQuery(
		com.liferay.portal.kernel.dao.orm.DynamicQuery dynamicQuery)
		throws com.liferay.portal.SystemException;

	public java.util.List<Object> dynamicQuery(
		com.liferay.portal.kernel.dao.orm.DynamicQuery dynamicQuery, int start,
		int end) throws com.liferay.portal.SystemException;

	@Transactional(propagation = Propagation.SUPPORTS, readOnly = true)
	public com.liferay.mail.model.AccountEntry getAccountEntry(
		long accountEntryId)
		throws com.liferay.portal.SystemException,
			com.liferay.portal.PortalException;

	@Transactional(propagation = Propagation.SUPPORTS, readOnly = true)
	public java.util.List<com.liferay.mail.model.AccountEntry> getAccountEntries(
		int start, int end) throws com.liferay.portal.SystemException;

	@Transactional(propagation = Propagation.SUPPORTS, readOnly = true)
	public int getAccountEntriesCount()
		throws com.liferay.portal.SystemException;

	public com.liferay.mail.model.AccountEntry updateAccountEntry(
		com.liferay.mail.model.AccountEntry accountEntry)
		throws com.liferay.portal.SystemException;

	public com.liferay.mail.model.AccountEntry updateAccountEntry(
		com.liferay.mail.model.AccountEntry accountEntry, boolean merge)
		throws com.liferay.portal.SystemException;

	public com.liferay.mail.model.AccountEntry addAccountEntry(long userId,
		java.lang.String type, java.lang.String messageAddress,
		java.lang.String username, java.lang.String password,
		boolean passwordSaved, java.lang.String senderName,
		java.lang.String signature, boolean useSignature)
		throws com.liferay.portal.PortalException,
			com.liferay.portal.SystemException;

	public com.liferay.mail.model.AccountEntry createAccountEntry(long userId,
		java.lang.String type, java.lang.String messageAddress,
		java.lang.String username, java.lang.String password,
		boolean passwordSaved, java.lang.String senderName,
		java.lang.String signature, boolean useSignature)
		throws com.liferay.portal.PortalException,
			com.liferay.portal.SystemException;

	public com.liferay.mail.model.AccountEntry createMockAccountEntry(
		long userId, java.lang.String type, java.lang.String messageAddress,
		java.lang.String username, java.lang.String password,
		java.lang.String senderName, java.lang.String signature,
		boolean useSignature)
		throws com.liferay.portal.PortalException,
			com.liferay.portal.SystemException;

	@Transactional(propagation = Propagation.SUPPORTS, readOnly = true)
	public java.util.List<com.liferay.mail.model.AccountEntry> getCompanyAccountEntries(
		long companyId) throws com.liferay.portal.SystemException;

	@Transactional(propagation = Propagation.SUPPORTS, readOnly = true)
	public com.liferay.mail.model.AccountEntry getCompanyAccountEntry(
		long companyId, java.lang.String type, java.lang.String messageAddress)
		throws com.liferay.mail.NoSuchAccountEntryException,
			com.liferay.portal.SystemException;

	@Transactional(propagation = Propagation.SUPPORTS, readOnly = true)
	public java.util.List<com.liferay.mail.model.AccountEntry> getUserAccountEntries(
		long userId) throws com.liferay.portal.SystemException;

	@Transactional(propagation = Propagation.SUPPORTS, readOnly = true)
	public java.util.List<com.liferay.mail.model.AccountEntry> getUserAccountEntries(
		long userId, java.lang.String type)
		throws com.liferay.portal.SystemException;

	@Transactional(propagation = Propagation.SUPPORTS, readOnly = true)
	public java.util.List<com.liferay.mail.model.AccountEntry> getUserAccountEntries(
		long userId, java.lang.String type, boolean initialized)
		throws com.liferay.portal.SystemException;

	@Transactional(propagation = Propagation.SUPPORTS, readOnly = true)
	public com.liferay.mail.model.AccountEntry getUserAccountEntry(
		long userId, java.lang.String type, java.lang.String messageAddress)
		throws com.liferay.mail.NoSuchAccountEntryException,
			com.liferay.portal.SystemException;

	public com.liferay.mail.model.AccountEntry updateAccountEntry(
		long accountEntryId, java.lang.String messageAddress,
		java.lang.String username, java.lang.String password,
		boolean passwordSaved, java.lang.String senderName,
		java.lang.String signature, boolean useSignature)
		throws com.liferay.portal.PortalException,
			com.liferay.portal.SystemException;
}