/**
 * Copyright (c) 2000-2009 Liferay, Inc. All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package com.liferay.mail;

/**
 * <a href="Recipient.java.html"><b><i>View Source</i></b></a>
 *
 * @author Michael C. Han
 * @author Scott Lee
 *
 */
public class Recipient {

	public enum RecipientType {
		BCC, CC, TO
	}

	public Recipient(RecipientType type, String messageAddress, String name) {
		_type = type;
		_messageAddress = messageAddress;
		_name = name;
	}

	public Recipient(
		RecipientType type, String messageAddress, String name,
		long contactId) {

		_type = type;
		_messageAddress = messageAddress;
		_name = name;
		_contactId = contactId;
	}

	public boolean equals(final Object o) {
		if (this == o) {
			return true;
		}

		if (o == null || getClass() != o.getClass()) {
			return false;
		}

		final Recipient recipient = (Recipient) o;

		if (_messageAddress == null) {
			return false;
		}

		if (!_messageAddress.equals(recipient._messageAddress)) {
			return false;
		}

		return true;
	}

	public long getContactId() {
		return _contactId;
	}

	public String getMessageAddress() {
		return _messageAddress;
	}

	public String getName() {
		return _name;
	}

	public RecipientType getType() {
		return _type;
	}

	public int hashCode() {
		int result;
		result = (_messageAddress != null ? _messageAddress.hashCode() : 0);
		result = 31 * result + (_type != null ? _type.hashCode() : 0);
		return result;
	}

	private String _messageAddress;
	private String _name;
	private RecipientType _type;
	private long _contactId;

}