/**
 * Copyright (c) 2000-2009 Liferay, Inc. All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package com.liferay.mail.action;

import com.liferay.portal.kernel.json.JSONArray;
import com.liferay.portal.kernel.json.JSONException;
import com.liferay.portal.kernel.json.JSONFactoryUtil;
import com.liferay.portal.kernel.json.JSONObject;
import com.liferay.portal.kernel.portlet.ConfigurationAction;
import com.liferay.portal.kernel.util.Constants;
import com.liferay.portal.kernel.util.ParamUtil;
import com.liferay.portlet.PortletPreferencesFactoryUtil;
import com.liferay.util.portlet.PortletProps;

import javax.portlet.ActionRequest;
import javax.portlet.ActionResponse;
import javax.portlet.PortletConfig;
import javax.portlet.PortletPreferences;
import javax.portlet.RenderRequest;
import javax.portlet.RenderResponse;

/**
 * <a href="ConfigurationActionImpl.java.html"><b><i>View Source</i></b></a>
 *
 * @author Scott Lee
 *
 */
public class ConfigurationActionImpl implements ConfigurationAction {

	public void deletePreconfiguredAccount(
			ActionRequest actionRequest, PortletPreferences preferences)
		throws Exception {

		String titleLanguageKey = ParamUtil.getString(
			actionRequest, "titleLanguageKey");

		JSONArray accountEntries = getPreconfiguredAccountEntries(preferences);
		JSONArray updatedAccountEntries = JSONFactoryUtil.createJSONArray();

		for (int i = 0; i < accountEntries.length(); i++) {
			JSONObject tempAccountEntry = accountEntries.getJSONObject(i);

			if (!tempAccountEntry.getString("titleLanguageKey").
					equals(titleLanguageKey)) {

				updatedAccountEntries.put(tempAccountEntry);
			}
		}

		savePreconfiguredAccountEntries(preferences, updatedAccountEntries);
	}

	public void processAction(
			PortletConfig portletConfig, ActionRequest actionRequest,
			ActionResponse actionResponse)
		throws Exception {

		String cmd = ParamUtil.getString(actionRequest, Constants.CMD);

		String portletResource = ParamUtil.getString(
			actionRequest, "portletResource");

		PortletPreferences preferences =
			PortletPreferencesFactoryUtil.getPortletSetup(
				actionRequest, portletResource);

		if (cmd.equals("updateSettings")) {
			updateSettings(actionRequest, preferences);
		}
		else if (cmd.equals("updatePreconfiguredAccount")) {
			updatePreconfiguredAccount(actionRequest, preferences);
		}
		else if (cmd.equals("deletePreconfiguredAccount")) {
			deletePreconfiguredAccount(actionRequest, preferences);
		}
	}

	public String render(
		PortletConfig portletConfig, RenderRequest renderRequest,
		RenderResponse renderResponse) {

		return "/configuration.jsp";
	}
	public void updatePreconfiguredAccount(
			ActionRequest actionRequest, PortletPreferences preferences)
		throws Exception {

		String oldTitleLanguageKey = ParamUtil.getString(actionRequest, "oldTitleLanguageKey");
		String titleLanguageKey = ParamUtil.getString(actionRequest, "titleLanguageKey");
		String descriptionLanguageKey = ParamUtil.getString(actionRequest, "descriptionLanguageKey");
		String messageAddress = ParamUtil.getString(
			actionRequest, "messageAddress");
		boolean useLocalPartOfEmailAddressAsUsername = ParamUtil.getBoolean(
			actionRequest, "useLocalPartOfEmailAddressAsUsername");
		boolean hideSettings = ParamUtil.getBoolean(
			actionRequest, "hideSettings");
		String mailInHostName = ParamUtil.getString(
			actionRequest, "mailInHostName");
		int mailInPort = ParamUtil.getInteger(actionRequest, "mailInPort");
		boolean mailInSecure = ParamUtil.getBoolean(
			actionRequest, "mailInSecure");
		String mailOutHostName = ParamUtil.getString(
			actionRequest, "mailOutHostName");
		int mailOutPort = ParamUtil.getInteger(actionRequest, "mailOutPort");
		boolean mailOutSecure = ParamUtil.getBoolean(
			actionRequest, "mailOutSecure");
		String defaultFolderPrefix = ParamUtil.getString(
			actionRequest, "defaultFolderPrefix");

		JSONArray accountEntries = getPreconfiguredAccountEntries(preferences);
		JSONArray updatedAccountEntries = JSONFactoryUtil.createJSONArray();

		boolean accountEntryUpdated = false;

		JSONObject newAccountEntry = JSONFactoryUtil.createJSONObject();

		newAccountEntry.put("titleLanguageKey", titleLanguageKey);
		newAccountEntry.put("descriptionLanguageKey", descriptionLanguageKey);
		newAccountEntry.put("messageAddress", messageAddress);
		newAccountEntry.put(
			"useLocalPartOfEmailAddressAsUsername",
			useLocalPartOfEmailAddressAsUsername);
		newAccountEntry.put("hideSettings", hideSettings);
		newAccountEntry.put("mailInHostName", mailInHostName);
		newAccountEntry.put("mailInPort", mailInPort);
		newAccountEntry.put("mailInSecure", mailInSecure);
		newAccountEntry.put("mailOutHostName", mailOutHostName);
		newAccountEntry.put("mailOutPort", mailOutPort);
		newAccountEntry.put("mailOutSecure", mailOutSecure);
		newAccountEntry.put("defaultFolderPrefix", defaultFolderPrefix);

		for (int i = 0; i < accountEntries.length(); i++) {
			JSONObject tempAccountEntry = accountEntries.getJSONObject(i);

			if (tempAccountEntry.getString("titleLanguageKey").equals(oldTitleLanguageKey)) {
				updatedAccountEntries.put(newAccountEntry);

				accountEntryUpdated = true;
			}
			else {
				updatedAccountEntries.put(tempAccountEntry);
			}
		}

		if (!accountEntryUpdated) {
			updatedAccountEntries.put(newAccountEntry);
		}

		savePreconfiguredAccountEntries(preferences, updatedAccountEntries);
	}

	public void updateSettings(
			ActionRequest actionRequest, PortletPreferences preferences)
		throws Exception {

		boolean enableMailAccounts = ParamUtil.getBoolean(
			actionRequest, "enableMailAccountEntries");
		boolean enablePrivateAccount = ParamUtil.getBoolean(
			actionRequest, "enablePrivateAccountEntry");

		preferences.setValue(
			"enableMailAccountEntries", String.valueOf(enableMailAccounts));
		preferences.setValue(
			"enablePrivateAccountEntry", String.valueOf(enablePrivateAccount));

		preferences.store();
	}

	protected JSONArray getPreconfiguredAccountEntries(
			PortletPreferences preferences)
		throws JSONException {

		String preconfiguredAccounts = preferences.getValue(
			"preconfiguredMailAccounts",
			PortletProps.get("preconfigured.mail.accounts"));

		JSONObject jsonObj = JSONFactoryUtil.createJSONObject(
			preconfiguredAccounts);

		return jsonObj.getJSONArray("accountEntries");
	}

	protected void savePreconfiguredAccountEntries(
			PortletPreferences preferences, JSONArray accountEntries)
		throws Exception {

		JSONObject jsonObj = JSONFactoryUtil.createJSONObject();

		jsonObj.put("accountEntries", accountEntries);

		preferences.setValue(
			"preconfiguredMailAccounts", jsonObj.toString());

		preferences.store();
	}

}