/**
 * Copyright (c) 2000-2009 Liferay, Inc. All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package com.liferay.mail.hook.events;

import com.liferay.mail.MailboxConstants;
import com.liferay.mail.MessageManager;
import com.liferay.mail.mailbox.Mailbox;
import com.liferay.mail.mailbox.MailboxFactory;
import com.liferay.mail.mailbox.PasswordRetrieverUtil;
import com.liferay.mail.model.AccountEntry;
import com.liferay.mail.store.MailStore;
import com.liferay.mail.store.MailStoreFactory;
import com.liferay.portal.PortalException;
import com.liferay.portal.SystemException;
import com.liferay.portal.kernel.events.Action;
import com.liferay.portal.kernel.events.ActionException;
import com.liferay.portal.model.User;
import com.liferay.portal.service.UserLocalServiceUtil;
import com.liferay.portal.util.PortalUtil;

import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

/**
 * <a href="LoginPostAction.java.html"><b><i>View Source</i></b></a>
 *
 * @author Scott Lee
 *
 */
public class LoginPostAction extends Action {

	public void run(HttpServletRequest request, HttpServletResponse response)
		throws ActionException {

		try {
			verifyPrivateAccountEntryExists(request);
			initiateSynchronization(request);
		}
		catch (Exception e) {
			throw new ActionException(e);
		}
	}

	protected void initiateSynchronization(HttpServletRequest request)
		throws PortalException, SystemException {

		MessageManager messageManager = MessageManager.getInstance(request);

		List<AccountEntry> accountEntries = messageManager.getAccountEntries(
			true, false);

		for (AccountEntry accountEntry : accountEntries) {
			if (accountEntry.isPasswordSaved()) {
				messageManager.sendUpdateMessage(
					accountEntry.getAccountEntryId());
			}
		}
	}

	protected void verifyPrivateAccountEntryExists(HttpServletRequest request)
		throws PortalException, SystemException {

		long userId = PortalUtil.getUserId(request);

		User user = UserLocalServiceUtil.getUser(userId);

		MailStore mailStore = MailStoreFactory.createMailStore(user);

		Mailbox mailbox = MailboxFactory.createMailbox(
			user, null, MailboxConstants.ACCOUNT_TYPE_PRIVATE, mailStore,
			PasswordRetrieverUtil.clone(request));

		mailbox.verifyAccountEntryExists();
	}

}