/**
 * Copyright (c) 2000-2009 Liferay, Inc. All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package com.liferay.mail.imap;

import com.liferay.mail.MailException;
import com.liferay.mail.model.AccountEntry;
import com.liferay.mail.store.MailStore;
import com.liferay.mail.util.MessageConverter;
import com.liferay.mail.util.MessageConverterImpl;
import com.liferay.portal.PortalException;
import com.liferay.portal.kernel.log.Log;
import com.liferay.portal.kernel.log.LogFactoryUtil;
import com.liferay.portal.kernel.util.Validator;
import com.liferay.portal.model.User;

import com.sun.mail.imap.IMAPFolder;

import javax.mail.Message;
import javax.mail.MessagingException;
import javax.mail.event.MessageCountEvent;
import javax.mail.event.MessageCountListener;

/**
 * <a href="MailMessageCountListener.java.html"><b><i>View Source</i></b></a>
 *
 * @author Scott Lee
 *
 */
public class MailMessageCountListener implements MessageCountListener {

	public MailMessageCountListener(
		MailStore mailStore, User user, AccountEntry accountEntry) {

		_user = user;
		_accountEntry = accountEntry;
		_messageConverter = new MessageConverterImpl(mailStore);
	}

	public void messagesAdded(MessageCountEvent event) {
		Message[] messages = event.getMessages();

		IMAPFolder folder = null;

		try {
			for (Message message : messages) {
				folder = openFolder((IMAPFolder)message.getFolder());

				try {
					_messageConverter.convertJavaxMessage(
						_user.getUserId(), _accountEntry.getAccountEntryId(),
						message);
				}
				catch (Exception e) {
					_log.error("Unable to store message added to folder", e);
				}
			}
		}
		catch (Exception e) {
			_log.error("Unable to open folder", e);
		}
		finally {
			closeFolder(folder, false);
		}
	}

	public void messagesRemoved(MessageCountEvent event) {
	}

	protected void closeFolder(IMAPFolder folder, boolean expunge) {
		try {
			if (Validator.isNotNull(folder) && folder.isOpen()) {
				if (expunge) {
					folder.expunge();
				}

				folder.close(expunge);
			}
		}
		catch (MessagingException me) {
			_log.error("Unable to close folder", me);
		}
	}

	protected long getMessageUid(Message message) throws PortalException {
		IMAPFolder folder = null;

		try {
			folder = openFolder((IMAPFolder)message.getFolder());

			return folder.getUID(message);
		}
		catch (MessagingException me) {
			throw new MailException(me);
		}
		finally {
			closeFolder(folder, false);
		}
	}

	protected IMAPFolder openFolder(IMAPFolder folder) throws PortalException {
		try {
			if (folder.isOpen()) {
				return folder;
			}

			folder.open(IMAPFolder.READ_WRITE);

			return folder;
		}
		catch (MessagingException me) {
			throw new MailException(me);
		}
	}

	private static Log _log =
		LogFactoryUtil.getLog(MailMessageCountListener.class);

	private AccountEntry _accountEntry;
	private MessageConverter _messageConverter;
	private User _user;

}