/**
 * Copyright (c) 2000-2009 Liferay, Inc. All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package com.liferay.mail.mailbox;

import com.liferay.mail.MessageManager;
import com.liferay.mail.model.AccountEntry;
import com.liferay.mail.service.AccountEntryLocalServiceUtil;
import com.liferay.portal.PortalException;
import com.liferay.portal.SystemException;
import com.liferay.portal.kernel.util.Validator;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;

/**
 * <a href="DefaultPasswordRetriever.java.html"><b><i>View Source</i></b></a>
 *
 * @author Mike Han
 *
 */
public class DefaultPasswordRetriever implements PasswordRetriever {

	public DefaultPasswordRetriever() {
	}

	public PasswordRetriever clone(
		HttpServletRequest request, ExternalPasswordVault passwordVault) {

		return new DefaultPasswordRetriever(request, passwordVault);
	}

	public String getPassword(long accountEntryId)
		throws PortalException, SystemException {

		if ((_request == null) || (_externalPasswordVault == null)) {
			throw new IllegalStateException("Uninitialized retriever");
		}

		MessageManager messageManager = MessageManager.getInstance(_request);

		if (Validator.isNotNull(messageManager) &&
				messageManager.isPasswordSaved(accountEntryId)) {

			if (MessageManager.STORE_PASSWORDS_REMOTELY) {
				return _externalPasswordVault.getRemotePassword(
						_request, accountEntryId);
			}
			else {
				AccountEntry accountEntry =
					AccountEntryLocalServiceUtil.getAccountEntry(
							accountEntryId);

				return accountEntry.getPasswordDecrypted();
			}
		}
		else {
			HttpSession session = _request.getSession();

			return (String)session.getAttribute(getPasswordKey(accountEntryId));
		}
	}

	public String getPasswordKey(long accountEntryId) {
		return "mail_p_key_" + accountEntryId;
	}

	private DefaultPasswordRetriever(
			HttpServletRequest request,
			ExternalPasswordVault passwordVault) {
		_request = request;
		_externalPasswordVault = passwordVault;
	}

	public ExternalPasswordVault getExternalPasswordVault() {
		return _externalPasswordVault;
	}

	private ExternalPasswordVault _externalPasswordVault;
	private HttpServletRequest _request;

}