/**
 * Copyright (c) 2000-2009 Liferay, Inc. All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package com.liferay.mail.mailbox;

import com.liferay.mail.MailFile;
import com.liferay.mail.Recipient;
import com.liferay.mail.model.AccountEntry;
import com.liferay.mail.model.Contact;
import com.liferay.mail.model.Folder;
import com.liferay.mail.model.Message;
import com.liferay.mail.store.MailStore;
import com.liferay.portal.PortalException;
import com.liferay.portal.SystemException;
import com.liferay.portal.model.User;
import com.liferay.util.portlet.PortletProps;

import java.io.InputStream;
import java.io.UnsupportedEncodingException;

import java.util.List;

/**
 * <a href="Mailbox.java.html"><b><i>View Source</i></b></a>
 *
 * @author Scott Lee
 *
 */
public interface Mailbox {

	public static final int FLAG_ANSWERED = 1;

	public static final int FLAG_DELETED = 2;

	public static final int FLAG_DRAFT = 3;

	public static final int FLAG_FLAGGED = 4;

	public static final int FLAG_RECENT = 5;

	public static final int FLAG_SEEN = 6;

	public static final int FLAG_USER = 7;

	public static final String FOLDER_DRAFTS = "Drafts";

	public static final String FOLDER_INBOX = "Inbox";

	public static final String FOLDER_SENT_MESSAGES = "Sent Messages";

	public static final String FOLDER_TRASH = "Trash";

	public static final String ORDER_BY_ADDRESS = "address";

	public static final String ORDER_BY_DATE = "date";

	public static final String ORDER_BY_SIZE = "size";

	public static final String ORDER_BY_SUBJECT = "subject";

	public static final String RECIPIENTS_INCLUDED_IN_AUTOCOMPLETE =
		PortletProps.get("recipients.included.in.autocomplete");

	public AccountEntry addOrUpdateAccount(
			String messageAddress, String mailInHostName, int mailInPort,
			boolean mailInSecure, String mailOutHostName, int mailOutPort,
			boolean mailOutSecure, String defaultFolderPrefix, String password,
			boolean passwordSaved, String username, String senderName, 
			String signature,boolean useSignature)
		throws PortalException, SystemException;

	public Contact addOrUpdateContact(
			long userId, String name, String messageAddress)
		throws PortalException, SystemException;

	public Mailbox clone(
		User user, AccountEntry accountEntry, MailStore store,
		PasswordRetriever retriever);

	public void createFolder(String name)
		throws PortalException, SystemException;

	public void deleteAccountEntry() throws PortalException, SystemException;

	public void deleteAttachment(long messageId, long attachmentId)
		throws PortalException, SystemException;

	public void deleteContact(long contactId)
		throws PortalException, SystemException;

	public void deleteFolder(long folderId)
		throws PortalException, SystemException;

	public void deleteMessagesByIds(
			long folderId, long[] messageIds, boolean useTrashFolder)
		throws PortalException, SystemException;

	public List<Recipient> getContacts()
		throws PortalException, SystemException;

	public InputStream getAttachment(long attachmentId)
		throws PortalException, SystemException;

	public List<Folder> getFolders() throws SystemException;

	public Message getMessageId(long messageId)
		throws PortalException, SystemException;

	public Message getMessageNumber(
			long folderId, int messageNumber, int offset,
			String orderByField, String orderByType, String keywords)
		throws PortalException, SystemException;

	public int getMessages(
			List<Message> messages, long folderId, int pageNumber,
			int messagesPerPage, String orderByField, String orderByType,
			String keywords)
		throws PortalException, SystemException;

	public int getMessagesStoredCount(long folderId)
		throws PortalException, SystemException;

	public long getUnreadMessageCountByAccountEntryId()
		throws PortalException, SystemException;

	public void moveMessages(
			long sourceFolderId, long desintationFolderId, long[] messageIds)
		throws PortalException, SystemException;

	public List<Recipient> parseRecipients(String recipients)
		throws PortalException;

	public Message saveDraft(
			String to, String cc, String bcc, String subject, String body,
			List<MailFile> mailFiles, long messageId)
		throws PortalException, SystemException;

	public void sendMessage(long sendersAccountEntryId, long messageId)
		throws PortalException, SystemException, UnsupportedEncodingException;

	public void sendUpdateMessage()
		throws PortalException, SystemException;

	public void sendUpdateMessage(long folderId)
		throws PortalException, SystemException;

	public void testAccount(
			String messageAddress, String username, String password,
			String senderName,String signature, boolean useSignature,
			String mailInHostName, int mailInPort, boolean mailInSecure,
			String mailOutHostName, int mailOutPort, boolean mailOutSecure)
		throws PortalException, SystemException;

	public Contact updateContact(
			long contactId, String name, String messageAddress)
		throws PortalException, SystemException;

	public void updateFolder(long folderId, String newFolderName)
	throws PortalException, SystemException;

	public void updateMessageFlags(
		long folderId, long[] messageIds, int flag, boolean value)
	throws PortalException, SystemException;

	public void verifyAccountEntryExists()
		throws PortalException, SystemException;

}