/**
 * Copyright (c) 2000-2009 Liferay, Inc. All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package com.liferay.mail.mailbox;

import com.liferay.mail.model.AccountEntry;
import com.liferay.mail.service.AccountEntryLocalServiceUtil;
import com.liferay.mail.store.MailStore;
import com.liferay.portal.PortalException;
import com.liferay.portal.SystemException;
import com.liferay.portal.model.User;

import java.util.Map;

/**
 * <a href="MailboxFactory.java.html"><b><i>View Source</i></b></a>
 *
 * @author Scott Lee
 * @author Michael C. Han
 *
 */
public class MailboxFactory {

	public static Mailbox createMailbox(
		User user, AccountEntry accountEntry, String type, MailStore store,
		PasswordRetriever retriever) {

		Mailbox mailbox = _mailboxes.get(type);

		if (mailbox == null) {
			throw new IllegalArgumentException(
				"Unable to locate mailbox of type: " + type);
		}

		return mailbox.clone(user, accountEntry, store, retriever);
	}

	public static Mailbox createPasswordlessMailbox(
		User user, AccountEntry accountEntry, String type, MailStore store) {

		Mailbox mailbox = _mailboxes.get(type);

		if (mailbox == null) {
			throw new IllegalArgumentException(
				"Unable to locate mailbox of type: " + type);
		}

		return mailbox.clone(user, accountEntry, store, null);
	}

	public static Mailbox createMailbox(
			User user, long accountEntryId, MailStore store,
			PasswordRetriever retriever)
		throws PortalException, SystemException {

		AccountEntry accountEntry =
			AccountEntryLocalServiceUtil.getAccountEntry(accountEntryId);

		Mailbox mailbox = _mailboxes.get(accountEntry.getType());

		if (mailbox == null) {
			throw new IllegalArgumentException(
				"Unable to locate mailbox of type: " + accountEntry.getType());
		}

		return mailbox.clone(user, accountEntry, store, retriever);
	}

	public void setMailboxes(Map<String, Mailbox> mailboxes) {
		_mailboxes = mailboxes;
	}

	private static Map<String, Mailbox> _mailboxes;

}