/**
 * Copyright (c) 2000-2009 Liferay, Inc. All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package com.liferay.mail.mailbox;

import com.liferay.mail.AddressException;
import com.liferay.mail.FolderException;
import com.liferay.mail.MailFile;
import com.liferay.mail.MailboxConstants;
import com.liferay.mail.Recipient.RecipientType;
import com.liferay.mail.Recipient;
import com.liferay.mail.model.AccountEntry;
import com.liferay.mail.model.Folder;
import com.liferay.mail.model.Message;
import com.liferay.mail.service.AccountEntryLocalServiceUtil;
import com.liferay.mail.service.FolderLocalServiceUtil;
import com.liferay.mail.service.MessageLocalServiceUtil;
import com.liferay.mail.store.MailStore;
import com.liferay.mail.util.MessageAddressConverter;
import com.liferay.mail.util.MessageAddressConverterImpl;
import com.liferay.mail.util.MessageNotificationUtil;
import com.liferay.portal.NoSuchUserException;
import com.liferay.portal.PortalException;
import com.liferay.portal.SystemException;
import com.liferay.portal.kernel.dao.orm.QueryUtil;
import com.liferay.portal.kernel.log.Log;
import com.liferay.portal.kernel.log.LogFactoryUtil;
import com.liferay.portal.kernel.util.StringPool;
import com.liferay.portal.model.User;
import com.liferay.portal.service.UserLocalServiceUtil;
import com.liferay.portal.util.comparator.UserScreenNameComparator;
import com.liferay.portlet.social.model.SocialRelationConstants;

import java.io.InputStream;

import java.util.ArrayList;
import java.util.List;

/**
 * <a href="PrivateMailbox.java.html"><b><i>View Source</i></b></a>
 *
 * @author Scott Lee
 *
 */
public class PrivateMailbox extends GenericMailbox {

	public AccountEntry addOrUpdateAccount(
		String messageAddress, String mailInHostName, int mailInPort,
		boolean mailInSecure, String mailOutHostName, int mailOutPort,
		boolean mailOutSecure, String defaultFolderPrefix, String password,
		boolean passwordSaved, String username, String senderName, 
		String signature, boolean useSignature) {

		throw new UnsupportedOperationException();
	}

	public Mailbox clone(
		User user, AccountEntry accountEntry, MailStore store,
		PasswordRetriever passwordRetriever) {

		PrivateMailbox mailbox = new PrivateMailbox();

		mailbox.setAccountEntry(accountEntry);
		mailbox.setMailStore(store);
		mailbox.setUser(user);

		return mailbox;
	}

	public void createFolder(String name)
		throws PortalException, SystemException {

		name = name.trim();

		if (name.equals(StringPool.BLANK)) {
			throw new FolderException(FolderException.INVALID_NAME);
		}

		_mailStore.addFolder(_accountEntry.getAccountEntryId(), name, name);
	}

	public void deleteAttachment(long messageId, long attachmentId)
		throws PortalException, SystemException {

		_mailStore.deleteAttachment(messageId, attachmentId);
	}

	public void deleteFolder(long folderId)
		throws PortalException, SystemException {

		if (isRequiredFolder(folderId)) {
			throw new FolderException(FolderException.REQUIRED_FOLDER);
		}

		_mailStore.deleteFolder(_accountEntry.getAccountEntryId(), folderId);
	}

	public void deleteMessagesByIds(
			long folderId, long[] messageIds, boolean useTrashFolder)
		throws PortalException, SystemException {

		for (long messageId : messageIds) {
			_mailStore.deleteMessage(messageId, useTrashFolder);
		}
	}

	public InputStream getAttachment(long attachmentId)
		throws PortalException, SystemException {

		return _mailStore.getAttachmentFromStoreAsInputStream(
			_accountEntry.getAccountEntryId(), attachmentId);
	}

	public List<Recipient> getContacts()
		throws PortalException, SystemException {

		List<Recipient> recipients = new ArrayList<Recipient>();

		List<User> users = null;

		if (RECIPIENTS_INCLUDED_IN_AUTOCOMPLETE.equals("friends")) {
			users = UserLocalServiceUtil.getSocialUsers(
				_user.getUserId(), SocialRelationConstants.TYPE_BI_FRIEND,
				QueryUtil.ALL_POS, QueryUtil.ALL_POS,
				new UserScreenNameComparator(true));
		}
		else {
			users = UserLocalServiceUtil.getUsers(
				QueryUtil.ALL_POS, QueryUtil.ALL_POS);
		}

		for (User user : users) {
			if (user.isDefaultUser()) {
				continue;
			}

			AccountEntry accountEntry =
				AccountEntryLocalServiceUtil.getUserAccountEntry(
					user.getUserId(), MailboxConstants.ACCOUNT_TYPE_PRIVATE,
					user.getScreenName());

			recipients.add(
				new Recipient(
					RecipientType.TO, accountEntry.getMessageAddress(),
					user.getFullName()));
		}

		return recipients;
	}

	public Message getMessageId(long messageId)
		throws PortalException, SystemException {

		return _mailStore.getMessage(messageId);
	}

	public int getMessages(
			List<Message> messages, long folderId, int pageNumber,
			int messagesPerPage, String orderByField, String orderByType,
			String keywords)
		throws PortalException, SystemException {

		return _mailStore.getMessages(
			messages, _accountEntry.getAccountEntryId(), folderId, pageNumber,
			messagesPerPage, orderByField, orderByType, keywords);
	}

	public void moveMessages(
			long sourceFolderId, long desintationFolderId, long[] messageIds)
		throws PortalException, SystemException {

		_mailStore.moveMessage(messageIds, desintationFolderId, false);
	}

	public List<Recipient> parseRecipients(String recipients) {
		throw new UnsupportedOperationException();
	}

	public Message saveDraft(
			String to, String cc, String bcc, String subject, String body,
			List<MailFile> mailFiles, long messageId)
		throws PortalException, SystemException {

		MessageAddressConverter messageConverter =
			new MessageAddressConverterImpl();

		// Verify recipients

		Recipient[] recipientsTo =
			messageConverter.convertToRecipients(
				to, Recipient.RecipientType.TO, false);

		for (Recipient recipient : recipientsTo) {
			try {
				UserLocalServiceUtil.getUserByScreenName(
					_user.getCompanyId(), recipient.getMessageAddress());
			}
			catch (NoSuchUserException nsue) {
				throw new AddressException(nsue, recipient.getMessageAddress());
			}
		}

		return _mailStore.saveDraft(
			_accountEntry.getAccountEntryId(), recipientsTo, new Recipient[] {},
			new Recipient[] {}, subject, body, mailFiles, messageId);
	}

	public void sendMessage(long sendersAccountEntryId, long draftMessageId)
		throws PortalException, SystemException {

		Message draftMessage = MessageLocalServiceUtil.getMessage(
			draftMessageId);

		_mailStore.sendMessage(
			sendersAccountEntryId, draftMessage.getMessageId());

		try {
			MessageNotificationUtil.sendNotifications(
				sendersAccountEntryId,
				_mailStore.getAllRecipients(draftMessage.getMessageId()),
				draftMessage.getSubject(), draftMessage.getBody());
		}
		catch (AddressException ae) {
			_log.error(
				"Failed to send private message notifications because of an " +
					"invalid address " + ae.getValue());
		}
	}

	public void sendUpdateMessage() throws PortalException, SystemException {
		throw new UnsupportedOperationException();
	}

	public void sendUpdateMessage(long folderId)
		throws PortalException, SystemException {

		throw new UnsupportedOperationException();
	}

	public void testAccount(
		String messageAddress, String username, String password,
		String senderName, String signature, boolean useSignature,
		String mailInHostName, int mailInPort, boolean mailInSecure,
		String mailOutHostName, int mailOutPort, boolean mailOutSecure) {

		throw new UnsupportedOperationException();
	}

	public void updateFolder(long folderId, String newFolderName)
		throws PortalException, SystemException {

		if (isRequiredFolder(folderId)) {
			throw new FolderException(FolderException.REQUIRED_FOLDER);
		}

		newFolderName = newFolderName.trim();

		if (newFolderName.equals(StringPool.BLANK)) {
			throw new FolderException(FolderException.INVALID_NAME);
		}

		_mailStore.renameFolder(folderId, newFolderName, newFolderName);
	}

	public void updateMessageFlags(
			long folderId, long[] messageIds, int flag, boolean value)
		throws PortalException, SystemException {

		_mailStore.updateMessageFlags(messageIds, flag, value);
	}

	public void verifyAccountEntryExists()
		throws PortalException, SystemException {

		List<AccountEntry> accountEntries =
			AccountEntryLocalServiceUtil.getUserAccountEntries(
				_user.getUserId(), MailboxConstants.ACCOUNT_TYPE_PRIVATE);

		if (accountEntries.isEmpty()) {
			addPrivateMessageAccount();
		}
	}

	protected PrivateMailbox() {
	}

	protected void addPrivateMessageAccount()
		throws PortalException, SystemException {

		AccountEntry accountEntry =
			AccountEntryLocalServiceUtil.createAccountEntry(
				_user.getUserId(), MailboxConstants.ACCOUNT_TYPE_PRIVATE,
				_user.getScreenName(), StringPool.BLANK, StringPool.BLANK,
				true,StringPool.BLANK,StringPool.BLANK, true);

		accountEntry.setInitialized(true);
		accountEntry.setType(MailboxConstants.ACCOUNT_TYPE_PRIVATE);
		accountEntry.setPasswordSaved(true);

		// Folders

		Folder draftFolder = FolderLocalServiceUtil.addFolder(
			_user.getUserId(), accountEntry.getAccountEntryId(), FOLDER_DRAFTS,
			FOLDER_DRAFTS, 0);
		Folder inboxFolder = FolderLocalServiceUtil.addFolder(
			_user.getUserId(), accountEntry.getAccountEntryId(), FOLDER_INBOX,
			FOLDER_INBOX, 0);
		Folder sentFolder = FolderLocalServiceUtil.addFolder(
			_user.getUserId(), accountEntry.getAccountEntryId(),
			FOLDER_SENT_MESSAGES, FOLDER_SENT_MESSAGES, 0);
		Folder trashFolder = FolderLocalServiceUtil.addFolder(
			_user.getUserId(), accountEntry.getAccountEntryId(), FOLDER_TRASH,
			FOLDER_TRASH, 0);

		accountEntry.setDraftFolderId(draftFolder.getFolderId());
		accountEntry.setInboxFolderId(inboxFolder.getFolderId());
		accountEntry.setSentFolderId(sentFolder.getFolderId());
		accountEntry.setTrashFolderId(trashFolder.getFolderId());

		AccountEntryLocalServiceUtil.addAccountEntry(accountEntry);
	}

	private static Log _log = LogFactoryUtil.getLog(PrivateMailbox.class);

}