/**
 * Copyright (c) 2000-2009 Liferay, Inc. All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package com.liferay.mail.model.impl;

import com.liferay.mail.model.AccountEntry;
import com.liferay.mail.model.AccountEntrySoap;

import com.liferay.portal.SystemException;
import com.liferay.portal.kernel.bean.ReadOnlyBeanHandler;
import com.liferay.portal.kernel.util.GetterUtil;
import com.liferay.portal.kernel.util.HtmlUtil;
import com.liferay.portal.model.impl.BaseModelImpl;
import com.liferay.portal.service.ServiceContext;
import com.liferay.portal.util.PortalUtil;

import com.liferay.portlet.expando.model.ExpandoBridge;
import com.liferay.portlet.expando.util.ExpandoBridgeFactoryUtil;

import java.io.Serializable;

import java.lang.reflect.Proxy;

import java.sql.Types;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * <a href="AccountEntryModelImpl.java.html"><b><i>View Source</i></b></a>
 *
 * @author Scott Lee
 */
public class AccountEntryModelImpl extends BaseModelImpl<AccountEntry> {
	public static final String TABLE_NAME = "Mail_AccountEntry";
	public static final Object[][] TABLE_COLUMNS = {
			{ "accountEntryId", new Integer(Types.BIGINT) },
			{ "companyId", new Integer(Types.BIGINT) },
			{ "userId", new Integer(Types.BIGINT) },
			{ "createDate", new Integer(Types.TIMESTAMP) },
			{ "modifiedDate", new Integer(Types.TIMESTAMP) },
			{ "type_", new Integer(Types.VARCHAR) },
			{ "messageAddress", new Integer(Types.VARCHAR) },
			{ "inboxFolderId", new Integer(Types.BIGINT) },
			{ "draftFolderId", new Integer(Types.BIGINT) },
			{ "sentFolderId", new Integer(Types.BIGINT) },
			{ "trashFolderId", new Integer(Types.BIGINT) },
			{ "username", new Integer(Types.VARCHAR) },
			{ "senderName", new Integer(Types.VARCHAR) },
			{ "password_", new Integer(Types.VARCHAR) },
			{ "passwordSaved", new Integer(Types.BOOLEAN) },
			{ "initialized", new Integer(Types.BOOLEAN) },
			{ "signature", new Integer(Types.VARCHAR) },
			{ "useSignature", new Integer(Types.BOOLEAN) }
		};
	public static final String TABLE_SQL_CREATE = "create table Mail_AccountEntry (accountEntryId LONG not null primary key,companyId LONG,userId LONG,createDate DATE null,modifiedDate DATE null,type_ VARCHAR(75) null,messageAddress VARCHAR(75) null,inboxFolderId LONG,draftFolderId LONG,sentFolderId LONG,trashFolderId LONG,username VARCHAR(75) null,senderName VARCHAR(75) null,password_ VARCHAR(75) null,passwordSaved BOOLEAN,initialized BOOLEAN,signature STRING null,useSignature BOOLEAN)";
	public static final String TABLE_SQL_DROP = "drop table Mail_AccountEntry";
	public static final String DATA_SOURCE = "liferayDataSource";
	public static final String SESSION_FACTORY = "liferaySessionFactory";
	public static final String TX_MANAGER = "liferayTransactionManager";
	public static final boolean ENTITY_CACHE_ENABLED = GetterUtil.getBoolean(com.liferay.util.service.ServiceProps.get(
				"value.object.entity.cache.enabled.com.liferay.mail.model.AccountEntry"),
			true);
	public static final boolean FINDER_CACHE_ENABLED = GetterUtil.getBoolean(com.liferay.util.service.ServiceProps.get(
				"value.object.finder.cache.enabled.com.liferay.mail.model.AccountEntry"),
			true);

	public static AccountEntry toModel(AccountEntrySoap soapModel) {
		AccountEntry model = new AccountEntryImpl();

		model.setAccountEntryId(soapModel.getAccountEntryId());
		model.setCompanyId(soapModel.getCompanyId());
		model.setUserId(soapModel.getUserId());
		model.setCreateDate(soapModel.getCreateDate());
		model.setModifiedDate(soapModel.getModifiedDate());
		model.setType(soapModel.getType());
		model.setMessageAddress(soapModel.getMessageAddress());
		model.setInboxFolderId(soapModel.getInboxFolderId());
		model.setDraftFolderId(soapModel.getDraftFolderId());
		model.setSentFolderId(soapModel.getSentFolderId());
		model.setTrashFolderId(soapModel.getTrashFolderId());
		model.setUsername(soapModel.getUsername());
		model.setSenderName(soapModel.getSenderName());
		model.setPassword(soapModel.getPassword());
		model.setPasswordSaved(soapModel.getPasswordSaved());
		model.setInitialized(soapModel.getInitialized());
		model.setSignature(soapModel.getSignature());
		model.setUseSignature(soapModel.getUseSignature());

		return model;
	}

	public static List<AccountEntry> toModels(AccountEntrySoap[] soapModels) {
		List<AccountEntry> models = new ArrayList<AccountEntry>(soapModels.length);

		for (AccountEntrySoap soapModel : soapModels) {
			models.add(toModel(soapModel));
		}

		return models;
	}

	public static final long LOCK_EXPIRATION_TIME = GetterUtil.getLong(com.liferay.util.service.ServiceProps.get(
				"lock.expiration.time.com.liferay.mail.model.AccountEntry"));

	public AccountEntryModelImpl() {
	}

	public long getPrimaryKey() {
		return _accountEntryId;
	}

	public void setPrimaryKey(long pk) {
		setAccountEntryId(pk);
	}

	public Serializable getPrimaryKeyObj() {
		return new Long(_accountEntryId);
	}

	public long getAccountEntryId() {
		return _accountEntryId;
	}

	public void setAccountEntryId(long accountEntryId) {
		_accountEntryId = accountEntryId;
	}

	public long getCompanyId() {
		return _companyId;
	}

	public void setCompanyId(long companyId) {
		_companyId = companyId;

		if (!_setOriginalCompanyId) {
			_setOriginalCompanyId = true;

			_originalCompanyId = companyId;
		}
	}

	public long getOriginalCompanyId() {
		return _originalCompanyId;
	}

	public long getUserId() {
		return _userId;
	}

	public void setUserId(long userId) {
		_userId = userId;

		if (!_setOriginalUserId) {
			_setOriginalUserId = true;

			_originalUserId = userId;
		}
	}

	public String getUserUuid() throws SystemException {
		return PortalUtil.getUserValue(getUserId(), "uuid", _userUuid);
	}

	public void setUserUuid(String userUuid) {
		_userUuid = userUuid;
	}

	public long getOriginalUserId() {
		return _originalUserId;
	}

	public Date getCreateDate() {
		return _createDate;
	}

	public void setCreateDate(Date createDate) {
		_createDate = createDate;
	}

	public Date getModifiedDate() {
		return _modifiedDate;
	}

	public void setModifiedDate(Date modifiedDate) {
		_modifiedDate = modifiedDate;
	}

	public String getType() {
		return GetterUtil.getString(_type);
	}

	public void setType(String type) {
		_type = type;

		if (_originalType == null) {
			_originalType = type;
		}
	}

	public String getOriginalType() {
		return GetterUtil.getString(_originalType);
	}

	public String getMessageAddress() {
		return GetterUtil.getString(_messageAddress);
	}

	public void setMessageAddress(String messageAddress) {
		_messageAddress = messageAddress;

		if (_originalMessageAddress == null) {
			_originalMessageAddress = messageAddress;
		}
	}

	public String getOriginalMessageAddress() {
		return GetterUtil.getString(_originalMessageAddress);
	}

	public long getInboxFolderId() {
		return _inboxFolderId;
	}

	public void setInboxFolderId(long inboxFolderId) {
		_inboxFolderId = inboxFolderId;
	}

	public long getDraftFolderId() {
		return _draftFolderId;
	}

	public void setDraftFolderId(long draftFolderId) {
		_draftFolderId = draftFolderId;
	}

	public long getSentFolderId() {
		return _sentFolderId;
	}

	public void setSentFolderId(long sentFolderId) {
		_sentFolderId = sentFolderId;
	}

	public long getTrashFolderId() {
		return _trashFolderId;
	}

	public void setTrashFolderId(long trashFolderId) {
		_trashFolderId = trashFolderId;
	}

	public String getUsername() {
		return GetterUtil.getString(_username);
	}

	public void setUsername(String username) {
		_username = username;
	}

	public String getSenderName() {
		return GetterUtil.getString(_senderName);
	}

	public void setSenderName(String senderName) {
		_senderName = senderName;
	}

	public String getPassword() {
		return GetterUtil.getString(_password);
	}

	public void setPassword(String password) {
		_password = password;
	}

	public boolean getPasswordSaved() {
		return _passwordSaved;
	}

	public boolean isPasswordSaved() {
		return _passwordSaved;
	}

	public void setPasswordSaved(boolean passwordSaved) {
		_passwordSaved = passwordSaved;
	}

	public boolean getInitialized() {
		return _initialized;
	}

	public boolean isInitialized() {
		return _initialized;
	}

	public void setInitialized(boolean initialized) {
		_initialized = initialized;
	}

	public String getSignature() {
		return GetterUtil.getString(_signature);
	}

	public void setSignature(String signature) {
		_signature = signature;
	}

	public boolean getUseSignature() {
		return _useSignature;
	}

	public boolean isUseSignature() {
		return _useSignature;
	}

	public void setUseSignature(boolean useSignature) {
		_useSignature = useSignature;
	}

	public AccountEntry toEscapedModel() {
		if (isEscapedModel()) {
			return (AccountEntry)this;
		}
		else {
			AccountEntry model = new AccountEntryImpl();

			model.setNew(isNew());
			model.setEscapedModel(true);

			model.setAccountEntryId(getAccountEntryId());
			model.setCompanyId(getCompanyId());
			model.setUserId(getUserId());
			model.setCreateDate(getCreateDate());
			model.setModifiedDate(getModifiedDate());
			model.setType(HtmlUtil.escape(getType()));
			model.setMessageAddress(HtmlUtil.escape(getMessageAddress()));
			model.setInboxFolderId(getInboxFolderId());
			model.setDraftFolderId(getDraftFolderId());
			model.setSentFolderId(getSentFolderId());
			model.setTrashFolderId(getTrashFolderId());
			model.setUsername(HtmlUtil.escape(getUsername()));
			model.setSenderName(HtmlUtil.escape(getSenderName()));
			model.setPassword(HtmlUtil.escape(getPassword()));
			model.setPasswordSaved(getPasswordSaved());
			model.setInitialized(getInitialized());
			model.setSignature(HtmlUtil.escape(getSignature()));
			model.setUseSignature(getUseSignature());

			model = (AccountEntry)Proxy.newProxyInstance(AccountEntry.class.getClassLoader(),
					new Class[] { AccountEntry.class },
					new ReadOnlyBeanHandler(model));

			return model;
		}
	}

	public ExpandoBridge getExpandoBridge() {
		if (_expandoBridge == null) {
			_expandoBridge = ExpandoBridgeFactoryUtil.getExpandoBridge(AccountEntry.class.getName(),
					getPrimaryKey());
		}

		return _expandoBridge;
	}

	public void setExpandoBridgeAttributes(ServiceContext serviceContext) {
		getExpandoBridge().setAttributes(serviceContext);
	}

	public Object clone() {
		AccountEntryImpl clone = new AccountEntryImpl();

		clone.setAccountEntryId(getAccountEntryId());
		clone.setCompanyId(getCompanyId());
		clone.setUserId(getUserId());
		clone.setCreateDate(getCreateDate());
		clone.setModifiedDate(getModifiedDate());
		clone.setType(getType());
		clone.setMessageAddress(getMessageAddress());
		clone.setInboxFolderId(getInboxFolderId());
		clone.setDraftFolderId(getDraftFolderId());
		clone.setSentFolderId(getSentFolderId());
		clone.setTrashFolderId(getTrashFolderId());
		clone.setUsername(getUsername());
		clone.setSenderName(getSenderName());
		clone.setPassword(getPassword());
		clone.setPasswordSaved(getPasswordSaved());
		clone.setInitialized(getInitialized());
		clone.setSignature(getSignature());
		clone.setUseSignature(getUseSignature());

		return clone;
	}

	public int compareTo(AccountEntry accountEntry) {
		int value = 0;

		value = getMessageAddress().compareTo(accountEntry.getMessageAddress());

		if (value != 0) {
			return value;
		}

		return 0;
	}

	public boolean equals(Object obj) {
		if (obj == null) {
			return false;
		}

		AccountEntry accountEntry = null;

		try {
			accountEntry = (AccountEntry)obj;
		}
		catch (ClassCastException cce) {
			return false;
		}

		long pk = accountEntry.getPrimaryKey();

		if (getPrimaryKey() == pk) {
			return true;
		}
		else {
			return false;
		}
	}

	public int hashCode() {
		return (int)getPrimaryKey();
	}

	public String toString() {
		StringBuilder sb = new StringBuilder();

		sb.append("{accountEntryId=");
		sb.append(getAccountEntryId());
		sb.append(", companyId=");
		sb.append(getCompanyId());
		sb.append(", userId=");
		sb.append(getUserId());
		sb.append(", createDate=");
		sb.append(getCreateDate());
		sb.append(", modifiedDate=");
		sb.append(getModifiedDate());
		sb.append(", type=");
		sb.append(getType());
		sb.append(", messageAddress=");
		sb.append(getMessageAddress());
		sb.append(", inboxFolderId=");
		sb.append(getInboxFolderId());
		sb.append(", draftFolderId=");
		sb.append(getDraftFolderId());
		sb.append(", sentFolderId=");
		sb.append(getSentFolderId());
		sb.append(", trashFolderId=");
		sb.append(getTrashFolderId());
		sb.append(", username=");
		sb.append(getUsername());
		sb.append(", senderName=");
		sb.append(getSenderName());
		sb.append(", password=");
		sb.append(getPassword());
		sb.append(", passwordSaved=");
		sb.append(getPasswordSaved());
		sb.append(", initialized=");
		sb.append(getInitialized());
		sb.append(", signature=");
		sb.append(getSignature());
		sb.append(", useSignature=");
		sb.append(getUseSignature());
		sb.append("}");

		return sb.toString();
	}

	public String toXmlString() {
		StringBuilder sb = new StringBuilder();

		sb.append("<model><model-name>");
		sb.append("com.liferay.mail.model.AccountEntry");
		sb.append("</model-name>");

		sb.append(
			"<column><column-name>accountEntryId</column-name><column-value><![CDATA[");
		sb.append(getAccountEntryId());
		sb.append("]]></column-value></column>");
		sb.append(
			"<column><column-name>companyId</column-name><column-value><![CDATA[");
		sb.append(getCompanyId());
		sb.append("]]></column-value></column>");
		sb.append(
			"<column><column-name>userId</column-name><column-value><![CDATA[");
		sb.append(getUserId());
		sb.append("]]></column-value></column>");
		sb.append(
			"<column><column-name>createDate</column-name><column-value><![CDATA[");
		sb.append(getCreateDate());
		sb.append("]]></column-value></column>");
		sb.append(
			"<column><column-name>modifiedDate</column-name><column-value><![CDATA[");
		sb.append(getModifiedDate());
		sb.append("]]></column-value></column>");
		sb.append(
			"<column><column-name>type</column-name><column-value><![CDATA[");
		sb.append(getType());
		sb.append("]]></column-value></column>");
		sb.append(
			"<column><column-name>messageAddress</column-name><column-value><![CDATA[");
		sb.append(getMessageAddress());
		sb.append("]]></column-value></column>");
		sb.append(
			"<column><column-name>inboxFolderId</column-name><column-value><![CDATA[");
		sb.append(getInboxFolderId());
		sb.append("]]></column-value></column>");
		sb.append(
			"<column><column-name>draftFolderId</column-name><column-value><![CDATA[");
		sb.append(getDraftFolderId());
		sb.append("]]></column-value></column>");
		sb.append(
			"<column><column-name>sentFolderId</column-name><column-value><![CDATA[");
		sb.append(getSentFolderId());
		sb.append("]]></column-value></column>");
		sb.append(
			"<column><column-name>trashFolderId</column-name><column-value><![CDATA[");
		sb.append(getTrashFolderId());
		sb.append("]]></column-value></column>");
		sb.append(
			"<column><column-name>username</column-name><column-value><![CDATA[");
		sb.append(getUsername());
		sb.append("]]></column-value></column>");
		sb.append(
			"<column><column-name>senderName</column-name><column-value><![CDATA[");
		sb.append(getSenderName());
		sb.append("]]></column-value></column>");
		sb.append(
			"<column><column-name>password</column-name><column-value><![CDATA[");
		sb.append(getPassword());
		sb.append("]]></column-value></column>");
		sb.append(
			"<column><column-name>passwordSaved</column-name><column-value><![CDATA[");
		sb.append(getPasswordSaved());
		sb.append("]]></column-value></column>");
		sb.append(
			"<column><column-name>initialized</column-name><column-value><![CDATA[");
		sb.append(getInitialized());
		sb.append("]]></column-value></column>");
		sb.append(
			"<column><column-name>signature</column-name><column-value><![CDATA[");
		sb.append(getSignature());
		sb.append("]]></column-value></column>");
		sb.append(
			"<column><column-name>useSignature</column-name><column-value><![CDATA[");
		sb.append(getUseSignature());
		sb.append("]]></column-value></column>");

		sb.append("</model>");

		return sb.toString();
	}

	private long _accountEntryId;
	private long _companyId;
	private long _originalCompanyId;
	private boolean _setOriginalCompanyId;
	private long _userId;
	private String _userUuid;
	private long _originalUserId;
	private boolean _setOriginalUserId;
	private Date _createDate;
	private Date _modifiedDate;
	private String _type;
	private String _originalType;
	private String _messageAddress;
	private String _originalMessageAddress;
	private long _inboxFolderId;
	private long _draftFolderId;
	private long _sentFolderId;
	private long _trashFolderId;
	private String _username;
	private String _senderName;
	private String _password;
	private boolean _passwordSaved;
	private boolean _initialized;
	private String _signature;
	private boolean _useSignature;
	private transient ExpandoBridge _expandoBridge;
}