/**
 * Copyright (c) 2000-2009 Liferay, Inc. All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package com.liferay.mail.model.impl;

import com.liferay.mail.model.AccountSetting;
import com.liferay.mail.model.AccountSettingSoap;

import com.liferay.portal.kernel.bean.ReadOnlyBeanHandler;
import com.liferay.portal.kernel.util.GetterUtil;
import com.liferay.portal.kernel.util.HtmlUtil;
import com.liferay.portal.model.impl.BaseModelImpl;
import com.liferay.portal.service.ServiceContext;

import com.liferay.portlet.expando.model.ExpandoBridge;
import com.liferay.portlet.expando.util.ExpandoBridgeFactoryUtil;

import java.io.Serializable;

import java.lang.reflect.Proxy;

import java.sql.Types;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * <a href="AccountSettingModelImpl.java.html"><b><i>View Source</i></b></a>
 *
 * @author Scott Lee
 */
public class AccountSettingModelImpl extends BaseModelImpl<AccountSetting> {
	public static final String TABLE_NAME = "Mail_AccountSetting";
	public static final Object[][] TABLE_COLUMNS = {
			{ "accountSettingId", new Integer(Types.BIGINT) },
			{ "companyId", new Integer(Types.BIGINT) },
			{ "createDate", new Integer(Types.TIMESTAMP) },
			{ "modifiedDate", new Integer(Types.TIMESTAMP) },
			{ "accountEntryId", new Integer(Types.BIGINT) },
			{ "type_", new Integer(Types.VARCHAR) },
			{ "mailInHostName", new Integer(Types.VARCHAR) },
			{ "mailInPort", new Integer(Types.INTEGER) },
			{ "mailInSecure", new Integer(Types.BOOLEAN) },
			{ "mailOutHostName", new Integer(Types.VARCHAR) },
			{ "mailOutPort", new Integer(Types.INTEGER) },
			{ "mailOutSecure", new Integer(Types.BOOLEAN) },
			{ "defaultFolderPrefix", new Integer(Types.VARCHAR) }
		};
	public static final String TABLE_SQL_CREATE = "create table Mail_AccountSetting (accountSettingId LONG not null primary key,companyId LONG,createDate DATE null,modifiedDate DATE null,accountEntryId LONG,type_ VARCHAR(75) null,mailInHostName VARCHAR(75) null,mailInPort INTEGER,mailInSecure BOOLEAN,mailOutHostName VARCHAR(75) null,mailOutPort INTEGER,mailOutSecure BOOLEAN,defaultFolderPrefix VARCHAR(75) null)";
	public static final String TABLE_SQL_DROP = "drop table Mail_AccountSetting";
	public static final String DATA_SOURCE = "liferayDataSource";
	public static final String SESSION_FACTORY = "liferaySessionFactory";
	public static final String TX_MANAGER = "liferayTransactionManager";
	public static final boolean ENTITY_CACHE_ENABLED = GetterUtil.getBoolean(com.liferay.util.service.ServiceProps.get(
				"value.object.entity.cache.enabled.com.liferay.mail.model.AccountSetting"),
			true);
	public static final boolean FINDER_CACHE_ENABLED = GetterUtil.getBoolean(com.liferay.util.service.ServiceProps.get(
				"value.object.finder.cache.enabled.com.liferay.mail.model.AccountSetting"),
			true);

	public static AccountSetting toModel(AccountSettingSoap soapModel) {
		AccountSetting model = new AccountSettingImpl();

		model.setAccountSettingId(soapModel.getAccountSettingId());
		model.setCompanyId(soapModel.getCompanyId());
		model.setCreateDate(soapModel.getCreateDate());
		model.setModifiedDate(soapModel.getModifiedDate());
		model.setAccountEntryId(soapModel.getAccountEntryId());
		model.setType(soapModel.getType());
		model.setMailInHostName(soapModel.getMailInHostName());
		model.setMailInPort(soapModel.getMailInPort());
		model.setMailInSecure(soapModel.getMailInSecure());
		model.setMailOutHostName(soapModel.getMailOutHostName());
		model.setMailOutPort(soapModel.getMailOutPort());
		model.setMailOutSecure(soapModel.getMailOutSecure());
		model.setDefaultFolderPrefix(soapModel.getDefaultFolderPrefix());

		return model;
	}

	public static List<AccountSetting> toModels(AccountSettingSoap[] soapModels) {
		List<AccountSetting> models = new ArrayList<AccountSetting>(soapModels.length);

		for (AccountSettingSoap soapModel : soapModels) {
			models.add(toModel(soapModel));
		}

		return models;
	}

	public static final long LOCK_EXPIRATION_TIME = GetterUtil.getLong(com.liferay.util.service.ServiceProps.get(
				"lock.expiration.time.com.liferay.mail.model.AccountSetting"));

	public AccountSettingModelImpl() {
	}

	public long getPrimaryKey() {
		return _accountSettingId;
	}

	public void setPrimaryKey(long pk) {
		setAccountSettingId(pk);
	}

	public Serializable getPrimaryKeyObj() {
		return new Long(_accountSettingId);
	}

	public long getAccountSettingId() {
		return _accountSettingId;
	}

	public void setAccountSettingId(long accountSettingId) {
		_accountSettingId = accountSettingId;
	}

	public long getCompanyId() {
		return _companyId;
	}

	public void setCompanyId(long companyId) {
		_companyId = companyId;
	}

	public Date getCreateDate() {
		return _createDate;
	}

	public void setCreateDate(Date createDate) {
		_createDate = createDate;
	}

	public Date getModifiedDate() {
		return _modifiedDate;
	}

	public void setModifiedDate(Date modifiedDate) {
		_modifiedDate = modifiedDate;
	}

	public long getAccountEntryId() {
		return _accountEntryId;
	}

	public void setAccountEntryId(long accountEntryId) {
		_accountEntryId = accountEntryId;

		if (!_setOriginalAccountEntryId) {
			_setOriginalAccountEntryId = true;

			_originalAccountEntryId = accountEntryId;
		}
	}

	public long getOriginalAccountEntryId() {
		return _originalAccountEntryId;
	}

	public String getType() {
		return GetterUtil.getString(_type);
	}

	public void setType(String type) {
		_type = type;
	}

	public String getMailInHostName() {
		return GetterUtil.getString(_mailInHostName);
	}

	public void setMailInHostName(String mailInHostName) {
		_mailInHostName = mailInHostName;
	}

	public int getMailInPort() {
		return _mailInPort;
	}

	public void setMailInPort(int mailInPort) {
		_mailInPort = mailInPort;
	}

	public boolean getMailInSecure() {
		return _mailInSecure;
	}

	public boolean isMailInSecure() {
		return _mailInSecure;
	}

	public void setMailInSecure(boolean mailInSecure) {
		_mailInSecure = mailInSecure;
	}

	public String getMailOutHostName() {
		return GetterUtil.getString(_mailOutHostName);
	}

	public void setMailOutHostName(String mailOutHostName) {
		_mailOutHostName = mailOutHostName;
	}

	public int getMailOutPort() {
		return _mailOutPort;
	}

	public void setMailOutPort(int mailOutPort) {
		_mailOutPort = mailOutPort;
	}

	public boolean getMailOutSecure() {
		return _mailOutSecure;
	}

	public boolean isMailOutSecure() {
		return _mailOutSecure;
	}

	public void setMailOutSecure(boolean mailOutSecure) {
		_mailOutSecure = mailOutSecure;
	}

	public String getDefaultFolderPrefix() {
		return GetterUtil.getString(_defaultFolderPrefix);
	}

	public void setDefaultFolderPrefix(String defaultFolderPrefix) {
		_defaultFolderPrefix = defaultFolderPrefix;
	}

	public AccountSetting toEscapedModel() {
		if (isEscapedModel()) {
			return (AccountSetting)this;
		}
		else {
			AccountSetting model = new AccountSettingImpl();

			model.setNew(isNew());
			model.setEscapedModel(true);

			model.setAccountSettingId(getAccountSettingId());
			model.setCompanyId(getCompanyId());
			model.setCreateDate(getCreateDate());
			model.setModifiedDate(getModifiedDate());
			model.setAccountEntryId(getAccountEntryId());
			model.setType(HtmlUtil.escape(getType()));
			model.setMailInHostName(HtmlUtil.escape(getMailInHostName()));
			model.setMailInPort(getMailInPort());
			model.setMailInSecure(getMailInSecure());
			model.setMailOutHostName(HtmlUtil.escape(getMailOutHostName()));
			model.setMailOutPort(getMailOutPort());
			model.setMailOutSecure(getMailOutSecure());
			model.setDefaultFolderPrefix(HtmlUtil.escape(
					getDefaultFolderPrefix()));

			model = (AccountSetting)Proxy.newProxyInstance(AccountSetting.class.getClassLoader(),
					new Class[] { AccountSetting.class },
					new ReadOnlyBeanHandler(model));

			return model;
		}
	}

	public ExpandoBridge getExpandoBridge() {
		if (_expandoBridge == null) {
			_expandoBridge = ExpandoBridgeFactoryUtil.getExpandoBridge(AccountSetting.class.getName(),
					getPrimaryKey());
		}

		return _expandoBridge;
	}

	public void setExpandoBridgeAttributes(ServiceContext serviceContext) {
		getExpandoBridge().setAttributes(serviceContext);
	}

	public Object clone() {
		AccountSettingImpl clone = new AccountSettingImpl();

		clone.setAccountSettingId(getAccountSettingId());
		clone.setCompanyId(getCompanyId());
		clone.setCreateDate(getCreateDate());
		clone.setModifiedDate(getModifiedDate());
		clone.setAccountEntryId(getAccountEntryId());
		clone.setType(getType());
		clone.setMailInHostName(getMailInHostName());
		clone.setMailInPort(getMailInPort());
		clone.setMailInSecure(getMailInSecure());
		clone.setMailOutHostName(getMailOutHostName());
		clone.setMailOutPort(getMailOutPort());
		clone.setMailOutSecure(getMailOutSecure());
		clone.setDefaultFolderPrefix(getDefaultFolderPrefix());

		return clone;
	}

	public int compareTo(AccountSetting accountSetting) {
		long pk = accountSetting.getPrimaryKey();

		if (getPrimaryKey() < pk) {
			return -1;
		}
		else if (getPrimaryKey() > pk) {
			return 1;
		}
		else {
			return 0;
		}
	}

	public boolean equals(Object obj) {
		if (obj == null) {
			return false;
		}

		AccountSetting accountSetting = null;

		try {
			accountSetting = (AccountSetting)obj;
		}
		catch (ClassCastException cce) {
			return false;
		}

		long pk = accountSetting.getPrimaryKey();

		if (getPrimaryKey() == pk) {
			return true;
		}
		else {
			return false;
		}
	}

	public int hashCode() {
		return (int)getPrimaryKey();
	}

	public String toString() {
		StringBuilder sb = new StringBuilder();

		sb.append("{accountSettingId=");
		sb.append(getAccountSettingId());
		sb.append(", companyId=");
		sb.append(getCompanyId());
		sb.append(", createDate=");
		sb.append(getCreateDate());
		sb.append(", modifiedDate=");
		sb.append(getModifiedDate());
		sb.append(", accountEntryId=");
		sb.append(getAccountEntryId());
		sb.append(", type=");
		sb.append(getType());
		sb.append(", mailInHostName=");
		sb.append(getMailInHostName());
		sb.append(", mailInPort=");
		sb.append(getMailInPort());
		sb.append(", mailInSecure=");
		sb.append(getMailInSecure());
		sb.append(", mailOutHostName=");
		sb.append(getMailOutHostName());
		sb.append(", mailOutPort=");
		sb.append(getMailOutPort());
		sb.append(", mailOutSecure=");
		sb.append(getMailOutSecure());
		sb.append(", defaultFolderPrefix=");
		sb.append(getDefaultFolderPrefix());
		sb.append("}");

		return sb.toString();
	}

	public String toXmlString() {
		StringBuilder sb = new StringBuilder();

		sb.append("<model><model-name>");
		sb.append("com.liferay.mail.model.AccountSetting");
		sb.append("</model-name>");

		sb.append(
			"<column><column-name>accountSettingId</column-name><column-value><![CDATA[");
		sb.append(getAccountSettingId());
		sb.append("]]></column-value></column>");
		sb.append(
			"<column><column-name>companyId</column-name><column-value><![CDATA[");
		sb.append(getCompanyId());
		sb.append("]]></column-value></column>");
		sb.append(
			"<column><column-name>createDate</column-name><column-value><![CDATA[");
		sb.append(getCreateDate());
		sb.append("]]></column-value></column>");
		sb.append(
			"<column><column-name>modifiedDate</column-name><column-value><![CDATA[");
		sb.append(getModifiedDate());
		sb.append("]]></column-value></column>");
		sb.append(
			"<column><column-name>accountEntryId</column-name><column-value><![CDATA[");
		sb.append(getAccountEntryId());
		sb.append("]]></column-value></column>");
		sb.append(
			"<column><column-name>type</column-name><column-value><![CDATA[");
		sb.append(getType());
		sb.append("]]></column-value></column>");
		sb.append(
			"<column><column-name>mailInHostName</column-name><column-value><![CDATA[");
		sb.append(getMailInHostName());
		sb.append("]]></column-value></column>");
		sb.append(
			"<column><column-name>mailInPort</column-name><column-value><![CDATA[");
		sb.append(getMailInPort());
		sb.append("]]></column-value></column>");
		sb.append(
			"<column><column-name>mailInSecure</column-name><column-value><![CDATA[");
		sb.append(getMailInSecure());
		sb.append("]]></column-value></column>");
		sb.append(
			"<column><column-name>mailOutHostName</column-name><column-value><![CDATA[");
		sb.append(getMailOutHostName());
		sb.append("]]></column-value></column>");
		sb.append(
			"<column><column-name>mailOutPort</column-name><column-value><![CDATA[");
		sb.append(getMailOutPort());
		sb.append("]]></column-value></column>");
		sb.append(
			"<column><column-name>mailOutSecure</column-name><column-value><![CDATA[");
		sb.append(getMailOutSecure());
		sb.append("]]></column-value></column>");
		sb.append(
			"<column><column-name>defaultFolderPrefix</column-name><column-value><![CDATA[");
		sb.append(getDefaultFolderPrefix());
		sb.append("]]></column-value></column>");

		sb.append("</model>");

		return sb.toString();
	}

	private long _accountSettingId;
	private long _companyId;
	private Date _createDate;
	private Date _modifiedDate;
	private long _accountEntryId;
	private long _originalAccountEntryId;
	private boolean _setOriginalAccountEntryId;
	private String _type;
	private String _mailInHostName;
	private int _mailInPort;
	private boolean _mailInSecure;
	private String _mailOutHostName;
	private int _mailOutPort;
	private boolean _mailOutSecure;
	private String _defaultFolderPrefix;
	private transient ExpandoBridge _expandoBridge;
}