/**
 * Copyright (c) 2000-2009 Liferay, Inc. All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package com.liferay.mail.model.impl;

import com.liferay.mail.model.Attachment;
import com.liferay.mail.model.AttachmentSoap;

import com.liferay.portal.kernel.bean.ReadOnlyBeanHandler;
import com.liferay.portal.kernel.util.GetterUtil;
import com.liferay.portal.kernel.util.HtmlUtil;
import com.liferay.portal.model.impl.BaseModelImpl;
import com.liferay.portal.service.ServiceContext;

import com.liferay.portlet.expando.model.ExpandoBridge;
import com.liferay.portlet.expando.util.ExpandoBridgeFactoryUtil;

import java.io.Serializable;

import java.lang.reflect.Proxy;

import java.sql.Types;

import java.util.ArrayList;
import java.util.List;

/**
 * <a href="AttachmentModelImpl.java.html"><b><i>View Source</i></b></a>
 *
 * @author Scott Lee
 */
public class AttachmentModelImpl extends BaseModelImpl<Attachment> {
	public static final String TABLE_NAME = "Mail_Attachment";
	public static final Object[][] TABLE_COLUMNS = {
			{ "attachmentId", new Integer(Types.BIGINT) },
			{ "messageId", new Integer(Types.BIGINT) },
			{ "filename", new Integer(Types.VARCHAR) },
			{ "size_", new Integer(Types.BIGINT) },
			{ "contentPath", new Integer(Types.VARCHAR) }
		};
	public static final String TABLE_SQL_CREATE = "create table Mail_Attachment (attachmentId LONG not null primary key,messageId LONG,filename VARCHAR(150) null,size_ LONG,contentPath VARCHAR(75) null)";
	public static final String TABLE_SQL_DROP = "drop table Mail_Attachment";
	public static final String DATA_SOURCE = "liferayDataSource";
	public static final String SESSION_FACTORY = "liferaySessionFactory";
	public static final String TX_MANAGER = "liferayTransactionManager";
	public static final boolean ENTITY_CACHE_ENABLED = GetterUtil.getBoolean(com.liferay.util.service.ServiceProps.get(
				"value.object.entity.cache.enabled.com.liferay.mail.model.Attachment"),
			true);
	public static final boolean FINDER_CACHE_ENABLED = GetterUtil.getBoolean(com.liferay.util.service.ServiceProps.get(
				"value.object.finder.cache.enabled.com.liferay.mail.model.Attachment"),
			true);

	public static Attachment toModel(AttachmentSoap soapModel) {
		Attachment model = new AttachmentImpl();

		model.setAttachmentId(soapModel.getAttachmentId());
		model.setMessageId(soapModel.getMessageId());
		model.setFilename(soapModel.getFilename());
		model.setSize(soapModel.getSize());
		model.setContentPath(soapModel.getContentPath());

		return model;
	}

	public static List<Attachment> toModels(AttachmentSoap[] soapModels) {
		List<Attachment> models = new ArrayList<Attachment>(soapModels.length);

		for (AttachmentSoap soapModel : soapModels) {
			models.add(toModel(soapModel));
		}

		return models;
	}

	public static final long LOCK_EXPIRATION_TIME = GetterUtil.getLong(com.liferay.util.service.ServiceProps.get(
				"lock.expiration.time.com.liferay.mail.model.Attachment"));

	public AttachmentModelImpl() {
	}

	public long getPrimaryKey() {
		return _attachmentId;
	}

	public void setPrimaryKey(long pk) {
		setAttachmentId(pk);
	}

	public Serializable getPrimaryKeyObj() {
		return new Long(_attachmentId);
	}

	public long getAttachmentId() {
		return _attachmentId;
	}

	public void setAttachmentId(long attachmentId) {
		_attachmentId = attachmentId;
	}

	public long getMessageId() {
		return _messageId;
	}

	public void setMessageId(long messageId) {
		_messageId = messageId;
	}

	public String getFilename() {
		return GetterUtil.getString(_filename);
	}

	public void setFilename(String filename) {
		_filename = filename;
	}

	public long getSize() {
		return _size;
	}

	public void setSize(long size) {
		_size = size;
	}

	public String getContentPath() {
		return GetterUtil.getString(_contentPath);
	}

	public void setContentPath(String contentPath) {
		_contentPath = contentPath;
	}

	public Attachment toEscapedModel() {
		if (isEscapedModel()) {
			return (Attachment)this;
		}
		else {
			Attachment model = new AttachmentImpl();

			model.setNew(isNew());
			model.setEscapedModel(true);

			model.setAttachmentId(getAttachmentId());
			model.setMessageId(getMessageId());
			model.setFilename(HtmlUtil.escape(getFilename()));
			model.setSize(getSize());
			model.setContentPath(HtmlUtil.escape(getContentPath()));

			model = (Attachment)Proxy.newProxyInstance(Attachment.class.getClassLoader(),
					new Class[] { Attachment.class },
					new ReadOnlyBeanHandler(model));

			return model;
		}
	}

	public ExpandoBridge getExpandoBridge() {
		if (_expandoBridge == null) {
			_expandoBridge = ExpandoBridgeFactoryUtil.getExpandoBridge(Attachment.class.getName(),
					getPrimaryKey());
		}

		return _expandoBridge;
	}

	public void setExpandoBridgeAttributes(ServiceContext serviceContext) {
		getExpandoBridge().setAttributes(serviceContext);
	}

	public Object clone() {
		AttachmentImpl clone = new AttachmentImpl();

		clone.setAttachmentId(getAttachmentId());
		clone.setMessageId(getMessageId());
		clone.setFilename(getFilename());
		clone.setSize(getSize());
		clone.setContentPath(getContentPath());

		return clone;
	}

	public int compareTo(Attachment attachment) {
		long pk = attachment.getPrimaryKey();

		if (getPrimaryKey() < pk) {
			return -1;
		}
		else if (getPrimaryKey() > pk) {
			return 1;
		}
		else {
			return 0;
		}
	}

	public boolean equals(Object obj) {
		if (obj == null) {
			return false;
		}

		Attachment attachment = null;

		try {
			attachment = (Attachment)obj;
		}
		catch (ClassCastException cce) {
			return false;
		}

		long pk = attachment.getPrimaryKey();

		if (getPrimaryKey() == pk) {
			return true;
		}
		else {
			return false;
		}
	}

	public int hashCode() {
		return (int)getPrimaryKey();
	}

	public String toString() {
		StringBuilder sb = new StringBuilder();

		sb.append("{attachmentId=");
		sb.append(getAttachmentId());
		sb.append(", messageId=");
		sb.append(getMessageId());
		sb.append(", filename=");
		sb.append(getFilename());
		sb.append(", size=");
		sb.append(getSize());
		sb.append(", contentPath=");
		sb.append(getContentPath());
		sb.append("}");

		return sb.toString();
	}

	public String toXmlString() {
		StringBuilder sb = new StringBuilder();

		sb.append("<model><model-name>");
		sb.append("com.liferay.mail.model.Attachment");
		sb.append("</model-name>");

		sb.append(
			"<column><column-name>attachmentId</column-name><column-value><![CDATA[");
		sb.append(getAttachmentId());
		sb.append("]]></column-value></column>");
		sb.append(
			"<column><column-name>messageId</column-name><column-value><![CDATA[");
		sb.append(getMessageId());
		sb.append("]]></column-value></column>");
		sb.append(
			"<column><column-name>filename</column-name><column-value><![CDATA[");
		sb.append(getFilename());
		sb.append("]]></column-value></column>");
		sb.append(
			"<column><column-name>size</column-name><column-value><![CDATA[");
		sb.append(getSize());
		sb.append("]]></column-value></column>");
		sb.append(
			"<column><column-name>contentPath</column-name><column-value><![CDATA[");
		sb.append(getContentPath());
		sb.append("]]></column-value></column>");

		sb.append("</model>");

		return sb.toString();
	}

	private long _attachmentId;
	private long _messageId;
	private String _filename;
	private long _size;
	private String _contentPath;
	private transient ExpandoBridge _expandoBridge;
}