/**
 * Copyright (c) 2000-2009 Liferay, Inc. All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package com.liferay.mail.model.impl;

import com.liferay.mail.model.Folder;
import com.liferay.mail.model.FolderSoap;

import com.liferay.portal.kernel.bean.ReadOnlyBeanHandler;
import com.liferay.portal.kernel.util.GetterUtil;
import com.liferay.portal.kernel.util.HtmlUtil;
import com.liferay.portal.model.impl.BaseModelImpl;
import com.liferay.portal.service.ServiceContext;

import com.liferay.portlet.expando.model.ExpandoBridge;
import com.liferay.portlet.expando.util.ExpandoBridgeFactoryUtil;

import java.io.Serializable;

import java.lang.reflect.Proxy;

import java.sql.Types;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * <a href="FolderModelImpl.java.html"><b><i>View Source</i></b></a>
 *
 * @author Scott Lee
 */
public class FolderModelImpl extends BaseModelImpl<Folder> {
	public static final String TABLE_NAME = "Mail_Folder";
	public static final Object[][] TABLE_COLUMNS = {
			{ "folderId", new Integer(Types.BIGINT) },
			{ "companyId", new Integer(Types.BIGINT) },
			{ "createDate", new Integer(Types.TIMESTAMP) },
			{ "modifiedDate", new Integer(Types.TIMESTAMP) },
			{ "accountEntryId", new Integer(Types.BIGINT) },
			{ "fullName", new Integer(Types.VARCHAR) },
			{ "name", new Integer(Types.VARCHAR) },
			{ "initialized", new Integer(Types.BOOLEAN) },
			{ "remoteMessageCount", new Integer(Types.INTEGER) },
			{ "synchronizedDate", new Integer(Types.TIMESTAMP) }
		};
	public static final String TABLE_SQL_CREATE = "create table Mail_Folder (folderId LONG not null primary key,companyId LONG,createDate DATE null,modifiedDate DATE null,accountEntryId LONG,fullName VARCHAR(75) null,name VARCHAR(75) null,initialized BOOLEAN,remoteMessageCount INTEGER,synchronizedDate DATE null)";
	public static final String TABLE_SQL_DROP = "drop table Mail_Folder";
	public static final String DATA_SOURCE = "liferayDataSource";
	public static final String SESSION_FACTORY = "liferaySessionFactory";
	public static final String TX_MANAGER = "liferayTransactionManager";
	public static final boolean ENTITY_CACHE_ENABLED = GetterUtil.getBoolean(com.liferay.util.service.ServiceProps.get(
				"value.object.entity.cache.enabled.com.liferay.mail.model.Folder"),
			true);
	public static final boolean FINDER_CACHE_ENABLED = GetterUtil.getBoolean(com.liferay.util.service.ServiceProps.get(
				"value.object.finder.cache.enabled.com.liferay.mail.model.Folder"),
			true);

	public static Folder toModel(FolderSoap soapModel) {
		Folder model = new FolderImpl();

		model.setFolderId(soapModel.getFolderId());
		model.setCompanyId(soapModel.getCompanyId());
		model.setCreateDate(soapModel.getCreateDate());
		model.setModifiedDate(soapModel.getModifiedDate());
		model.setAccountEntryId(soapModel.getAccountEntryId());
		model.setFullName(soapModel.getFullName());
		model.setName(soapModel.getName());
		model.setInitialized(soapModel.getInitialized());
		model.setRemoteMessageCount(soapModel.getRemoteMessageCount());
		model.setSynchronizedDate(soapModel.getSynchronizedDate());

		return model;
	}

	public static List<Folder> toModels(FolderSoap[] soapModels) {
		List<Folder> models = new ArrayList<Folder>(soapModels.length);

		for (FolderSoap soapModel : soapModels) {
			models.add(toModel(soapModel));
		}

		return models;
	}

	public static final long LOCK_EXPIRATION_TIME = GetterUtil.getLong(com.liferay.util.service.ServiceProps.get(
				"lock.expiration.time.com.liferay.mail.model.Folder"));

	public FolderModelImpl() {
	}

	public long getPrimaryKey() {
		return _folderId;
	}

	public void setPrimaryKey(long pk) {
		setFolderId(pk);
	}

	public Serializable getPrimaryKeyObj() {
		return new Long(_folderId);
	}

	public long getFolderId() {
		return _folderId;
	}

	public void setFolderId(long folderId) {
		_folderId = folderId;
	}

	public long getCompanyId() {
		return _companyId;
	}

	public void setCompanyId(long companyId) {
		_companyId = companyId;
	}

	public Date getCreateDate() {
		return _createDate;
	}

	public void setCreateDate(Date createDate) {
		_createDate = createDate;
	}

	public Date getModifiedDate() {
		return _modifiedDate;
	}

	public void setModifiedDate(Date modifiedDate) {
		_modifiedDate = modifiedDate;
	}

	public long getAccountEntryId() {
		return _accountEntryId;
	}

	public void setAccountEntryId(long accountEntryId) {
		_accountEntryId = accountEntryId;

		if (!_setOriginalAccountEntryId) {
			_setOriginalAccountEntryId = true;

			_originalAccountEntryId = accountEntryId;
		}
	}

	public long getOriginalAccountEntryId() {
		return _originalAccountEntryId;
	}

	public String getFullName() {
		return GetterUtil.getString(_fullName);
	}

	public void setFullName(String fullName) {
		_fullName = fullName;

		if (_originalFullName == null) {
			_originalFullName = fullName;
		}
	}

	public String getOriginalFullName() {
		return GetterUtil.getString(_originalFullName);
	}

	public String getName() {
		return GetterUtil.getString(_name);
	}

	public void setName(String name) {
		_name = name;
	}

	public boolean getInitialized() {
		return _initialized;
	}

	public boolean isInitialized() {
		return _initialized;
	}

	public void setInitialized(boolean initialized) {
		_initialized = initialized;
	}

	public int getRemoteMessageCount() {
		return _remoteMessageCount;
	}

	public void setRemoteMessageCount(int remoteMessageCount) {
		_remoteMessageCount = remoteMessageCount;
	}

	public Date getSynchronizedDate() {
		return _synchronizedDate;
	}

	public void setSynchronizedDate(Date synchronizedDate) {
		_synchronizedDate = synchronizedDate;
	}

	public Folder toEscapedModel() {
		if (isEscapedModel()) {
			return (Folder)this;
		}
		else {
			Folder model = new FolderImpl();

			model.setNew(isNew());
			model.setEscapedModel(true);

			model.setFolderId(getFolderId());
			model.setCompanyId(getCompanyId());
			model.setCreateDate(getCreateDate());
			model.setModifiedDate(getModifiedDate());
			model.setAccountEntryId(getAccountEntryId());
			model.setFullName(HtmlUtil.escape(getFullName()));
			model.setName(HtmlUtil.escape(getName()));
			model.setInitialized(getInitialized());
			model.setRemoteMessageCount(getRemoteMessageCount());
			model.setSynchronizedDate(getSynchronizedDate());

			model = (Folder)Proxy.newProxyInstance(Folder.class.getClassLoader(),
					new Class[] { Folder.class }, new ReadOnlyBeanHandler(model));

			return model;
		}
	}

	public ExpandoBridge getExpandoBridge() {
		if (_expandoBridge == null) {
			_expandoBridge = ExpandoBridgeFactoryUtil.getExpandoBridge(Folder.class.getName(),
					getPrimaryKey());
		}

		return _expandoBridge;
	}

	public void setExpandoBridgeAttributes(ServiceContext serviceContext) {
		getExpandoBridge().setAttributes(serviceContext);
	}

	public Object clone() {
		FolderImpl clone = new FolderImpl();

		clone.setFolderId(getFolderId());
		clone.setCompanyId(getCompanyId());
		clone.setCreateDate(getCreateDate());
		clone.setModifiedDate(getModifiedDate());
		clone.setAccountEntryId(getAccountEntryId());
		clone.setFullName(getFullName());
		clone.setName(getName());
		clone.setInitialized(getInitialized());
		clone.setRemoteMessageCount(getRemoteMessageCount());
		clone.setSynchronizedDate(getSynchronizedDate());

		return clone;
	}

	public int compareTo(Folder folder) {
		int value = 0;

		value = getFullName().compareTo(folder.getFullName());

		if (value != 0) {
			return value;
		}

		return 0;
	}

	public boolean equals(Object obj) {
		if (obj == null) {
			return false;
		}

		Folder folder = null;

		try {
			folder = (Folder)obj;
		}
		catch (ClassCastException cce) {
			return false;
		}

		long pk = folder.getPrimaryKey();

		if (getPrimaryKey() == pk) {
			return true;
		}
		else {
			return false;
		}
	}

	public int hashCode() {
		return (int)getPrimaryKey();
	}

	public String toString() {
		StringBuilder sb = new StringBuilder();

		sb.append("{folderId=");
		sb.append(getFolderId());
		sb.append(", companyId=");
		sb.append(getCompanyId());
		sb.append(", createDate=");
		sb.append(getCreateDate());
		sb.append(", modifiedDate=");
		sb.append(getModifiedDate());
		sb.append(", accountEntryId=");
		sb.append(getAccountEntryId());
		sb.append(", fullName=");
		sb.append(getFullName());
		sb.append(", name=");
		sb.append(getName());
		sb.append(", initialized=");
		sb.append(getInitialized());
		sb.append(", remoteMessageCount=");
		sb.append(getRemoteMessageCount());
		sb.append(", synchronizedDate=");
		sb.append(getSynchronizedDate());
		sb.append("}");

		return sb.toString();
	}

	public String toXmlString() {
		StringBuilder sb = new StringBuilder();

		sb.append("<model><model-name>");
		sb.append("com.liferay.mail.model.Folder");
		sb.append("</model-name>");

		sb.append(
			"<column><column-name>folderId</column-name><column-value><![CDATA[");
		sb.append(getFolderId());
		sb.append("]]></column-value></column>");
		sb.append(
			"<column><column-name>companyId</column-name><column-value><![CDATA[");
		sb.append(getCompanyId());
		sb.append("]]></column-value></column>");
		sb.append(
			"<column><column-name>createDate</column-name><column-value><![CDATA[");
		sb.append(getCreateDate());
		sb.append("]]></column-value></column>");
		sb.append(
			"<column><column-name>modifiedDate</column-name><column-value><![CDATA[");
		sb.append(getModifiedDate());
		sb.append("]]></column-value></column>");
		sb.append(
			"<column><column-name>accountEntryId</column-name><column-value><![CDATA[");
		sb.append(getAccountEntryId());
		sb.append("]]></column-value></column>");
		sb.append(
			"<column><column-name>fullName</column-name><column-value><![CDATA[");
		sb.append(getFullName());
		sb.append("]]></column-value></column>");
		sb.append(
			"<column><column-name>name</column-name><column-value><![CDATA[");
		sb.append(getName());
		sb.append("]]></column-value></column>");
		sb.append(
			"<column><column-name>initialized</column-name><column-value><![CDATA[");
		sb.append(getInitialized());
		sb.append("]]></column-value></column>");
		sb.append(
			"<column><column-name>remoteMessageCount</column-name><column-value><![CDATA[");
		sb.append(getRemoteMessageCount());
		sb.append("]]></column-value></column>");
		sb.append(
			"<column><column-name>synchronizedDate</column-name><column-value><![CDATA[");
		sb.append(getSynchronizedDate());
		sb.append("]]></column-value></column>");

		sb.append("</model>");

		return sb.toString();
	}

	private long _folderId;
	private long _companyId;
	private Date _createDate;
	private Date _modifiedDate;
	private long _accountEntryId;
	private long _originalAccountEntryId;
	private boolean _setOriginalAccountEntryId;
	private String _fullName;
	private String _originalFullName;
	private String _name;
	private boolean _initialized;
	private int _remoteMessageCount;
	private Date _synchronizedDate;
	private transient ExpandoBridge _expandoBridge;
}