/**
 * Copyright (c) 2000-2009 Liferay, Inc. All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package com.liferay.mail.portlet;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;

import javax.portlet.ActionRequest;
import javax.portlet.ActionResponse;
import javax.portlet.PortletException;
import javax.portlet.ResourceRequest;
import javax.portlet.ResourceResponse;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;

import com.liferay.mail.MailFile;
import com.liferay.mail.MessageManager;
import com.liferay.mail.NoSuchAttachmentException;
import com.liferay.mail.mailbox.ExternalPasswordVault;
import com.liferay.mail.mailbox.PasswordRetriever;
import com.liferay.mail.mailbox.PasswordRetrieverUtil;
import com.liferay.mail.model.AccountEntry;
import com.liferay.mail.model.Attachment;
import com.liferay.mail.service.AccountEntryLocalServiceUtil;
import com.liferay.mail.service.AttachmentLocalServiceUtil;
import com.liferay.portal.PortalException;
import com.liferay.portal.SystemException;
import com.liferay.portal.kernel.json.JSONObject;
import com.liferay.portal.kernel.log.Log;
import com.liferay.portal.kernel.log.LogFactoryUtil;
import com.liferay.portal.kernel.upload.UploadPortletRequest;
import com.liferay.portal.kernel.util.FileUtil;
import com.liferay.portal.kernel.util.GetterUtil;
import com.liferay.portal.kernel.util.MimeTypesUtil;
import com.liferay.portal.kernel.util.ParamUtil;
import com.liferay.portal.kernel.util.Validator;
import com.liferay.portal.kernel.util.WebKeys;
import com.liferay.portal.theme.ThemeDisplay;
import com.liferay.portal.util.PortalUtil;
import com.liferay.util.bridges.jsp.JSPPortlet;
import com.liferay.util.servlet.PortletResponseUtil;

/**
 * <a href="MailPortlet.java.html"><b><i>View Source</i></b></a>
 *
 * @author Scott Lee
 *
 */
public class MailPortlet extends JSPPortlet {

	public void deleteAccount(
			ActionRequest actionRequest, ActionResponse actionResponse)
		throws IOException, PortalException, SystemException {

		long accountEntryId = ParamUtil.getLong(
			actionRequest, "accountEntryId");

		HttpServletRequest request = PortalUtil.getHttpServletRequest(
			actionRequest);

		MessageManager messageManager = MessageManager.getInstance(request);

		if (MessageManager.STORE_PASSWORDS_REMOTELY) {
			PasswordRetriever passwordRetriever =
				MessageManager.getPasswordRetriever();

			ExternalPasswordVault vault =
				passwordRetriever.getExternalPasswordVault();

			vault.deletePassword(request, accountEntryId);
		}

		deletePasswordInSession(actionRequest, accountEntryId);

		messageManager.deleteAccountEntry(accountEntryId);
	}

	public void processAction(
			ActionRequest actionRequest, ActionResponse actionResponse)
		throws IOException, PortletException {

		if (!isProcessActionRequest(actionRequest)) {
			return;
		}

		if (!callActionMethod(actionRequest, actionResponse)) {
			return;
		}

		String redirect = ParamUtil.getString(actionRequest, "redirect");

		if (Validator.isNotNull(redirect)) {
			actionResponse.sendRedirect(redirect);
		}
	}

	public void saveAccount(
			ActionRequest actionRequest, ActionResponse actionResponse)
		throws IOException, PortalException, SystemException {

		long accountEntryId = ParamUtil.getLong(
			actionRequest, "accountEntryId");
		boolean useLocalPartOfEmailAddressAsUsername = ParamUtil.getBoolean(
			actionRequest, "useLocalPartOfEmailAddressAsUsername");
		String messageAddress = ParamUtil.getString(
			actionRequest, "messageAddress");
		String username = ParamUtil.getString(actionRequest, "username");
		String password = ParamUtil.getString(actionRequest, "password").trim();
		boolean passwordSaved = ParamUtil.getBoolean(
			actionRequest, "passwordSaved");
		String senderName = ParamUtil.getString(actionRequest,"senderName");
		String signature = ParamUtil.getString(actionRequest, "signature");
		boolean useSignature = ParamUtil.getBoolean(actionRequest, "useSignature");
		String mailInHostName = ParamUtil.getString(
			actionRequest, "mailInHostName");
		int mailInPort = ParamUtil.getInteger(actionRequest, "mailInPort");
		boolean mailInSecure = ParamUtil.getBoolean(
			actionRequest, "mailInSecure");
		String mailOutHostName = ParamUtil.getString(
			actionRequest, "mailOutHostName");
		int mailOutPort = ParamUtil.getInteger(actionRequest, "mailOutPort");
		boolean mailOutSecure = ParamUtil.getBoolean(
			actionRequest, "mailOutSecure");
		String defaultFolderPrefix = ParamUtil.getString(
			actionRequest, "defaultFolderPrefix");

		HttpServletRequest request = PortalUtil.getHttpServletRequest(
			actionRequest);

		PasswordRetriever passwordRetriever = PasswordRetrieverUtil.clone(
			request);

		ExternalPasswordVault vault =
			passwordRetriever.getExternalPasswordVault();

		if (Validator.isNull(password) && Validator.isNotNull(accountEntryId)) {
			password = passwordRetriever.getPassword(accountEntryId);
		}

		if (useLocalPartOfEmailAddressAsUsername) {
			username = messageAddress.split("@")[0];
		}

		boolean hasRemotePasswordError = false;

		if (MessageManager.STORE_PASSWORDS_REMOTELY) {
			hasRemotePasswordError = !vault.testRemotePasswordLocker(request);
		}

		MessageManager messageManager = MessageManager.getInstance(request);

		JSONObject jsonObj = messageManager.testAccount(
			messageAddress, mailInHostName, mailInPort, mailInSecure,
			mailOutHostName, mailOutPort, mailOutSecure, hasRemotePasswordError,
			password, username, senderName, signature, useSignature);

		if (jsonObj.getBoolean("success")) {
			AccountEntry accountEntry = messageManager.addOrUpdateAccount(
				messageAddress, mailInHostName, mailInPort, mailInSecure,
				mailOutHostName, mailOutPort, mailOutSecure,
				defaultFolderPrefix, password, passwordSaved, username,senderName,
				signature,useSignature);

			if (passwordSaved) {
				if (MessageManager.STORE_PASSWORDS_REMOTELY) {
					vault.storePassword(
						request, accountEntry.getAccountEntryId(), password);
				}
			}
			else {
				if (MessageManager.STORE_PASSWORDS_REMOTELY) {
					vault.deletePassword(
						request, accountEntry.getAccountEntryId());
				}

				deletePasswordInSession(actionRequest, accountEntryId);
			}
		}

		sendRedirect(actionRequest, actionResponse, jsonObj);
	}

	public void saveOrSendMessage(
			ActionRequest actionRequest, ActionResponse actionResponse)
		throws IOException, PortalException, SystemException {

		UploadPortletRequest uploadRequest = PortalUtil.getUploadPortletRequest(
			actionRequest);

		HttpServletRequest request = PortalUtil.getHttpServletRequest(
			actionRequest);

		MessageManager messageManager = MessageManager.getInstance(request);

		long draftMessageId = ParamUtil.getLong(
			uploadRequest, "sendDraftMessageId");
		long accountEntryId = ParamUtil.getLong(
			uploadRequest, "sendAccountEntryId");
		String to = ParamUtil.getString(uploadRequest, "sendTo");
		String cc = ParamUtil.getString(uploadRequest, "sendCc");
		String bcc = ParamUtil.getString(uploadRequest, "sendBcc");
		String subject = ParamUtil.getString(uploadRequest, "sendSubject");
		String body = ParamUtil.getString(uploadRequest, "sendBody");
		boolean sendMessage = ParamUtil.getBoolean(
			uploadRequest, "sendMessage");
		long forwardedMessageId = ParamUtil.getLong(
			uploadRequest, "sendForwardMessageId");
		String forwardedAttachmentIds = ParamUtil.getString(
			uploadRequest, "sendForwardAttachmentIds");

		// Get all attachments

		int attachmentCount = ParamUtil.getInteger(
			uploadRequest, "sendAttachmentCount");

		List<MailFile> mailFiles = new ArrayList<MailFile>();

		// Get real attachments

		for (int i = 1; i <= attachmentCount; i++) {
			File file = uploadRequest.getFile("sendAttachment" + i);
			String filename = uploadRequest.getFileName(
				"sendAttachment" + i);

			if (FileUtil.getBytes(file) != null) {
				mailFiles.add(new MailFile(filename, file.length(), file));
			}
		}

		// Get pseudo forwarded attachments

		long[] attachmentIds = GetterUtil.getLongValues(
			forwardedAttachmentIds.split(","));

		for (long attachmentId : attachmentIds) {
			try {
				Attachment attachment =
					AttachmentLocalServiceUtil.getAttachment(attachmentId);

				mailFiles.add(
					new MailFile(
						attachment.getFilename(), attachment.getSize(),
						messageManager.getAttachment(
							accountEntryId, attachmentId)));
			}
			catch (NoSuchAttachmentException nsae) {
			}
		}

		// Send or save

		JSONObject jsonObj = null;

		if (sendMessage) {
			jsonObj = messageManager.sendMessage(
				accountEntryId, draftMessageId, to, cc, bcc, subject, body,
				mailFiles);
		}
		else {
			jsonObj = messageManager.saveDraft(
				accountEntryId, draftMessageId, to, cc, bcc, subject, body,
				mailFiles);
		}

		sendRedirect(actionRequest, actionResponse, jsonObj);
	}

	public void sendRedirect(
			ActionRequest actionRequest, ActionResponse actionResponse,
			JSONObject jsonObj)
		throws IOException {

		ThemeDisplay themeDisplay = (ThemeDisplay)actionRequest.getAttribute(
			WebKeys.THEME_DISPLAY);

		actionResponse.sendRedirect(
			PortalUtil.getLayoutURL(themeDisplay) +
				"/-/mail/message_result?result=" + jsonObj);
	}

	public void sendUpdateMessage(
			ActionRequest actionRequest, ActionResponse actionResponse)
		throws PortalException, SystemException {

		long accountEntryId = ParamUtil.getLong(
			actionRequest, "accountEntryId");

		HttpServletRequest request = PortalUtil.getHttpServletRequest(
			actionRequest);

		MessageManager messageManager = MessageManager.getInstance(request);

		if (Validator.isNotNull(messageManager)) {
			messageManager.sendUpdateMessage(accountEntryId);
		}
	}

	public void serveResource(
			ResourceRequest resourceRequest, ResourceResponse resourceResponse)
		throws IOException, PortletException {

		String jspPage = resourceRequest.getParameter("jspPage");

		if (jspPage.equals("/attachment.jsp")) {
			HttpServletRequest request = PortalUtil.getHttpServletRequest(
				resourceRequest);

			long accountEntryId = ParamUtil.getLong(
				resourceRequest, "accountEntryId");
			long attachmentId = ParamUtil.getLong(
				resourceRequest, "attachmentId");

			try {
				Attachment attachment =
					AttachmentLocalServiceUtil.getAttachment(attachmentId);

				MessageManager messageManager = MessageManager.getInstance(
					request);

				InputStream is = messageManager.getAttachment(
					accountEntryId, attachmentId);

				if (Validator.isNotNull(is)) {
					String contentType = MimeTypesUtil.getContentType(
						attachment.getFilename());

					PortletResponseUtil.sendFile(
						resourceResponse, attachment.getFilename(), is,
						contentType);
				}
			}
			catch (Exception e) {
				_log.error(e.getMessage());
			}
		}
		else {
			super.serveResource(resourceRequest, resourceResponse);
		}
	}

	public void storePasswordInSession(
			ActionRequest actionRequest, ActionResponse actionResponse)
		throws IOException, PortalException, SystemException {

		long accountEntryId = ParamUtil.getLong(
			actionRequest, "accountEntryId");
		String password = ParamUtil.getString(actionRequest, "password");

		AccountEntry accountEntry =
			AccountEntryLocalServiceUtil.getAccountEntry(accountEntryId);

		if (!accountEntry.isPasswordSaved()) {

			// Check to see if password is valid

			MessageManager messageManager = MessageManager.getInstance(
  				PortalUtil.getHttpServletRequest(actionRequest));

			JSONObject jsonObj = messageManager.testPassword(
				accountEntryId, password);

			if (jsonObj.getBoolean("success")) {
				storePasswordInSession(actionRequest, accountEntryId, password);
			}

			sendRedirect(actionRequest, actionResponse, jsonObj);
		}
	}

	protected void deletePasswordInSession(
			ActionRequest actionRequest, long accountEntryId)
		throws IOException, PortalException, SystemException {

		HttpServletRequest request = PortalUtil.getHttpServletRequest(
			actionRequest);

		PasswordRetriever passwordRetriever =
			MessageManager.getPasswordRetriever();

		HttpSession session = request.getSession();

		session.removeAttribute(
			passwordRetriever.getPasswordKey(accountEntryId));
	}

	protected void storePasswordInSession(
			ActionRequest actionRequest, long accountEntryId, String password)
		throws PortalException, SystemException {

		HttpServletRequest request = PortalUtil.getHttpServletRequest(
			actionRequest);

		PasswordRetriever passwordRetriever =
			MessageManager.getPasswordRetriever();

		HttpSession session = request.getSession();

		session.setAttribute(
			passwordRetriever.getPasswordKey(accountEntryId), password);
	}

	private static Log _log = LogFactoryUtil.getLog(MailPortlet.class);

}