/**
 * Copyright (c) 2000-2009 Liferay, Inc. All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package com.liferay.mail.search;

import com.liferay.mail.model.AccountEntry;
import com.liferay.mail.util.FormatUtil;
import com.liferay.portal.kernel.dao.orm.QueryUtil;
import com.liferay.portal.kernel.search.BooleanQuery;
import com.liferay.portal.kernel.search.BooleanQueryFactoryUtil;
import com.liferay.portal.kernel.search.Document;
import com.liferay.portal.kernel.search.DocumentImpl;
import com.liferay.portal.kernel.search.DocumentSummary;
import com.liferay.portal.kernel.search.Field;
import com.liferay.portal.kernel.search.Hits;
import com.liferay.portal.kernel.search.SearchEngineUtil;
import com.liferay.portal.kernel.search.SearchException;
import com.liferay.portal.kernel.util.HtmlUtil;
import com.liferay.portal.util.PortletKeys;

import java.util.Date;

import javax.portlet.PortletURL;

/**
 * <a href="Indexer.java.html"><b><i>View Source</i></b></a>
 *
 * @author Alexander Chow
 * @author Scott Lee
 *
 */
public class Indexer implements com.liferay.portal.kernel.search.Indexer {

	public static final String ACCOUNT_ENTRY_ID = "accountEntryId";

	public static final String FOLDER_ID = "folderId";

	public static final String PORTLET_ID = PortletKeys.MAIL;

	public static final String RECIPIENTS = "recipients";

	public static final String SENDER = "sender";

	public static final String SENT_DATE = "sentDate";

	public static final String SIZE = "size";

	public static void addMessage(
			long companyId, long groupId, long userId, long accountEntryId,
			long folderId, long messageId, String sender, String recipients,
			Date sentDate, String subject, String body, long size)
		throws SearchException {

		Document doc = getMessageDocument(
			companyId, groupId, userId, accountEntryId, folderId, messageId,
			sender, recipients, sentDate, subject, body, size);

		SearchEngineUtil.addDocument(companyId, doc);
	}

	public static void deleteAccount(
			long companyId, long userId, long accountEntryId)
		throws SearchException {

		BooleanQuery booleanQuery = BooleanQueryFactoryUtil.create();

		booleanQuery.addRequiredTerm(Field.PORTLET_ID, PORTLET_ID);
		booleanQuery.addRequiredTerm(Field.USER_ID, userId);
		booleanQuery.addRequiredTerm(ACCOUNT_ENTRY_ID, accountEntryId);

		Hits hits = SearchEngineUtil.search(
			companyId, booleanQuery, QueryUtil.ALL_POS, QueryUtil.ALL_POS);

		for (int i = 0; i < hits.getLength(); i++) {
			Document doc = hits.doc(i);

			SearchEngineUtil.deleteDocument(companyId, doc.get(Field.UID));
		}
	}

	public static void deleteFolder(long companyId, long userId, long folderId)
		throws SearchException {

		BooleanQuery booleanQuery = BooleanQueryFactoryUtil.create();

		booleanQuery.addRequiredTerm(Field.PORTLET_ID, PORTLET_ID);
		booleanQuery.addRequiredTerm(Field.USER_ID, userId);
		booleanQuery.addRequiredTerm(FOLDER_ID, folderId);

		Hits hits = SearchEngineUtil.search(
			companyId, booleanQuery, QueryUtil.ALL_POS, QueryUtil.ALL_POS);

		for (int i = 0; i < hits.getLength(); i++) {
			Document doc = hits.doc(i);

			SearchEngineUtil.deleteDocument(companyId, doc.get(Field.UID));
		}
	}

	public static void deleteMessage(
			long companyId, long userId, long accountEntryId, long folderId,
			long messageId)
		throws SearchException {

		String uid = getUID(userId, accountEntryId, folderId, messageId);

		SearchEngineUtil.deleteDocument(companyId, uid);
	}

	public static Document getMessageDocument(
		long companyId, long groupId, long userId, long accountEntryId,
		long folderId, long messageId, String sender, String recipients,
		Date sentDate, String subject, String body, long size) {

		body = HtmlUtil.extractText(body);

		Document doc = new DocumentImpl();

		doc.addUID(
			PORTLET_ID, String.valueOf(userId), String.valueOf(accountEntryId),
			String.valueOf(folderId), String.valueOf(messageId));

		doc.addKeyword(Field.COMPANY_ID, companyId);
		doc.addKeyword(Field.GROUP_ID, groupId);
		doc.addKeyword(Field.PORTLET_ID, PORTLET_ID);
		doc.addKeyword(Field.USER_ID, userId);
		doc.addModifiedDate();

		doc.addKeyword(ACCOUNT_ENTRY_ID, accountEntryId);
		doc.addKeyword(FOLDER_ID, folderId);
		doc.addKeyword(SENDER, sender.toLowerCase());
		doc.addKeyword(RECIPIENTS, recipients.toLowerCase());
		doc.addKeyword(SENT_DATE, FormatUtil.formatDateNormalized(sentDate));
		doc.addKeyword(Field.TITLE, subject.toLowerCase());
		doc.addText(Field.CONTENT, body);
		doc.addKeyword(SIZE, size);
		doc.addKeyword(Field.ENTRY_CLASS_NAME, AccountEntry.class.getName());
		doc.addKeyword(Field.ENTRY_CLASS_PK, messageId);

		return doc;
	}

	public static String getUID(
		long userId, long accountEntryId, long folderId, long messageId) {

		Document doc = new DocumentImpl();

		doc.addUID(
			PORTLET_ID, String.valueOf(userId), String.valueOf(accountEntryId),
			String.valueOf(folderId), String.valueOf(messageId));

		return doc.get(Field.UID);
	}

	public String[] getClassNames() {
		return _CLASS_NAMES;
	}

	public DocumentSummary getDocumentSummary(
		Document doc, PortletURL portletURL) {

		return null;
	}

	public DocumentSummary getDocumentSummary(
		Document doc, String snippet, PortletURL portletURL) {

		return null;
	}

	public void reIndex(String className, long classPK) {
	}

	public void reIndex(String[] ids) throws SearchException {
		try {
			MessageIndexer.reIndex(ids);
		}
		catch (Exception e) {
			throw new SearchException(e);
		}
	}

	private static final String[] _CLASS_NAMES = new String[0];

}