/**
 * Copyright (c) 2000-2009 Liferay, Inc. All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package com.liferay.mail.service.base;

import com.liferay.mail.model.Folder;
import com.liferay.mail.service.AccountEntryLocalService;
import com.liferay.mail.service.AccountSettingLocalService;
import com.liferay.mail.service.AttachmentLocalService;
import com.liferay.mail.service.ContactLocalService;
import com.liferay.mail.service.FolderLocalService;
import com.liferay.mail.service.MessageLocalService;
import com.liferay.mail.service.persistence.AccountEntryPersistence;
import com.liferay.mail.service.persistence.AccountSettingPersistence;
import com.liferay.mail.service.persistence.AttachmentPersistence;
import com.liferay.mail.service.persistence.ContactPersistence;
import com.liferay.mail.service.persistence.FolderPersistence;
import com.liferay.mail.service.persistence.MessageFinder;
import com.liferay.mail.service.persistence.MessagePersistence;

import com.liferay.portal.PortalException;
import com.liferay.portal.SystemException;
import com.liferay.portal.kernel.annotation.BeanReference;
import com.liferay.portal.kernel.dao.orm.DynamicQuery;
import com.liferay.portal.util.PortalUtil;

import java.util.List;

/**
 * <a href="FolderLocalServiceBaseImpl.java.html"><b><i>View Source</i></b></a>
 *
 * @author Scott Lee
 */
public abstract class FolderLocalServiceBaseImpl implements FolderLocalService {
	public Folder addFolder(Folder folder) throws SystemException {
		folder.setNew(true);

		return folderPersistence.update(folder, false);
	}

	public Folder createFolder(long folderId) {
		return folderPersistence.create(folderId);
	}

	public void deleteFolder(long folderId)
		throws PortalException, SystemException {
		folderPersistence.remove(folderId);
	}

	public void deleteFolder(Folder folder) throws SystemException {
		folderPersistence.remove(folder);
	}

	public List<Object> dynamicQuery(DynamicQuery dynamicQuery)
		throws SystemException {
		return folderPersistence.findWithDynamicQuery(dynamicQuery);
	}

	public List<Object> dynamicQuery(DynamicQuery dynamicQuery, int start,
		int end) throws SystemException {
		return folderPersistence.findWithDynamicQuery(dynamicQuery, start, end);
	}

	public Folder getFolder(long folderId)
		throws PortalException, SystemException {
		return folderPersistence.findByPrimaryKey(folderId);
	}

	public List<Folder> getFolders(int start, int end)
		throws SystemException {
		return folderPersistence.findAll(start, end);
	}

	public int getFoldersCount() throws SystemException {
		return folderPersistence.countAll();
	}

	public Folder updateFolder(Folder folder) throws SystemException {
		folder.setNew(false);

		return folderPersistence.update(folder, true);
	}

	public Folder updateFolder(Folder folder, boolean merge)
		throws SystemException {
		folder.setNew(false);

		return folderPersistence.update(folder, merge);
	}

	public AccountEntryLocalService getAccountEntryLocalService() {
		return accountEntryLocalService;
	}

	public void setAccountEntryLocalService(
		AccountEntryLocalService accountEntryLocalService) {
		this.accountEntryLocalService = accountEntryLocalService;
	}

	public AccountEntryPersistence getAccountEntryPersistence() {
		return accountEntryPersistence;
	}

	public void setAccountEntryPersistence(
		AccountEntryPersistence accountEntryPersistence) {
		this.accountEntryPersistence = accountEntryPersistence;
	}

	public AccountSettingLocalService getAccountSettingLocalService() {
		return accountSettingLocalService;
	}

	public void setAccountSettingLocalService(
		AccountSettingLocalService accountSettingLocalService) {
		this.accountSettingLocalService = accountSettingLocalService;
	}

	public AccountSettingPersistence getAccountSettingPersistence() {
		return accountSettingPersistence;
	}

	public void setAccountSettingPersistence(
		AccountSettingPersistence accountSettingPersistence) {
		this.accountSettingPersistence = accountSettingPersistence;
	}

	public AttachmentLocalService getAttachmentLocalService() {
		return attachmentLocalService;
	}

	public void setAttachmentLocalService(
		AttachmentLocalService attachmentLocalService) {
		this.attachmentLocalService = attachmentLocalService;
	}

	public AttachmentPersistence getAttachmentPersistence() {
		return attachmentPersistence;
	}

	public void setAttachmentPersistence(
		AttachmentPersistence attachmentPersistence) {
		this.attachmentPersistence = attachmentPersistence;
	}

	public ContactLocalService getContactLocalService() {
		return contactLocalService;
	}

	public void setContactLocalService(ContactLocalService contactLocalService) {
		this.contactLocalService = contactLocalService;
	}

	public ContactPersistence getContactPersistence() {
		return contactPersistence;
	}

	public void setContactPersistence(ContactPersistence contactPersistence) {
		this.contactPersistence = contactPersistence;
	}

	public FolderLocalService getFolderLocalService() {
		return folderLocalService;
	}

	public void setFolderLocalService(FolderLocalService folderLocalService) {
		this.folderLocalService = folderLocalService;
	}

	public FolderPersistence getFolderPersistence() {
		return folderPersistence;
	}

	public void setFolderPersistence(FolderPersistence folderPersistence) {
		this.folderPersistence = folderPersistence;
	}

	public MessageLocalService getMessageLocalService() {
		return messageLocalService;
	}

	public void setMessageLocalService(MessageLocalService messageLocalService) {
		this.messageLocalService = messageLocalService;
	}

	public MessagePersistence getMessagePersistence() {
		return messagePersistence;
	}

	public void setMessagePersistence(MessagePersistence messagePersistence) {
		this.messagePersistence = messagePersistence;
	}

	public MessageFinder getMessageFinder() {
		return messageFinder;
	}

	public void setMessageFinder(MessageFinder messageFinder) {
		this.messageFinder = messageFinder;
	}

	protected void runSQL(String sql) throws SystemException {
		try {
			PortalUtil.runSQL(sql);
		}
		catch (Exception e) {
			throw new SystemException(e);
		}
	}

	@BeanReference(name = "com.liferay.mail.service.AccountEntryLocalService.impl")
	protected AccountEntryLocalService accountEntryLocalService;
	@BeanReference(name = "com.liferay.mail.service.persistence.AccountEntryPersistence.impl")
	protected AccountEntryPersistence accountEntryPersistence;
	@BeanReference(name = "com.liferay.mail.service.AccountSettingLocalService.impl")
	protected AccountSettingLocalService accountSettingLocalService;
	@BeanReference(name = "com.liferay.mail.service.persistence.AccountSettingPersistence.impl")
	protected AccountSettingPersistence accountSettingPersistence;
	@BeanReference(name = "com.liferay.mail.service.AttachmentLocalService.impl")
	protected AttachmentLocalService attachmentLocalService;
	@BeanReference(name = "com.liferay.mail.service.persistence.AttachmentPersistence.impl")
	protected AttachmentPersistence attachmentPersistence;
	@BeanReference(name = "com.liferay.mail.service.ContactLocalService.impl")
	protected ContactLocalService contactLocalService;
	@BeanReference(name = "com.liferay.mail.service.persistence.ContactPersistence.impl")
	protected ContactPersistence contactPersistence;
	@BeanReference(name = "com.liferay.mail.service.FolderLocalService.impl")
	protected FolderLocalService folderLocalService;
	@BeanReference(name = "com.liferay.mail.service.persistence.FolderPersistence.impl")
	protected FolderPersistence folderPersistence;
	@BeanReference(name = "com.liferay.mail.service.MessageLocalService.impl")
	protected MessageLocalService messageLocalService;
	@BeanReference(name = "com.liferay.mail.service.persistence.MessagePersistence.impl")
	protected MessagePersistence messagePersistence;
	@BeanReference(name = "com.liferay.mail.service.persistence.MessageFinder.impl")
	protected MessageFinder messageFinder;
}