/**
 * Copyright (c) 2000-2009 Liferay, Inc. All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package com.liferay.mail.service.impl;

import com.liferay.counter.service.CounterLocalServiceUtil;
import com.liferay.mail.NoSuchAccountEntryException;
import com.liferay.mail.model.AccountEntry;
import com.liferay.mail.service.base.AccountEntryLocalServiceBaseImpl;
import com.liferay.portal.PortalException;
import com.liferay.portal.SystemException;
import com.liferay.portal.model.User;
import com.liferay.portal.service.UserLocalServiceUtil;

import java.util.Date;
import java.util.List;

/**
 * <a href="AccountEntryLocalServiceImpl.java.html"><b><i>View Source</i></b>
 * </a>
 *
 * @author Scott Lee
 *
 */
public class AccountEntryLocalServiceImpl
	extends AccountEntryLocalServiceBaseImpl {

	public AccountEntry addAccountEntry(
			long userId, String type, String messageAddress, String username,
			String password, boolean passwordSaved, String senderName, String signature, boolean useSignature)
		throws PortalException, SystemException {

		AccountEntry accountEntry = createAccountEntry(
			userId, type, messageAddress, username, password, passwordSaved, senderName, signature, useSignature);

		return accountEntryPersistence.update(accountEntry, true);
	}

	public AccountEntry createAccountEntry(
			long userId, String type, String messageAddress, String username,
			String password, boolean passwordSaved, String senderName, String signature, boolean useSignature)
		throws PortalException, SystemException {

		User user = UserLocalServiceUtil.getUserById(userId);
		Date now = new Date();

		long accountId = CounterLocalServiceUtil.increment();

		AccountEntry accountEntry = accountEntryPersistence.create(accountId);

		accountEntry.setCompanyId(user.getCompanyId());
		accountEntry.setUserId(user.getUserId());
		accountEntry.setCreateDate(now);
		accountEntry.setModifiedDate(now);
		accountEntry.setType(type);
		accountEntry.setMessageAddress(messageAddress);
		accountEntry.setUsername(username);
		accountEntry.setPasswordDecrypted(password);
		accountEntry.setPasswordSaved(passwordSaved);
		accountEntry.setInitialized(false);
		accountEntry.setSenderName(senderName);
		
		if(useSignature){
			accountEntry.setSignature(signature);
		}else{
			accountEntry.setSignature("");
		}
		accountEntry.setUseSignature(useSignature);
		
		return accountEntry;
	}

	public AccountEntry createMockAccountEntry(
			long userId, String type, String messageAddress, String username,
			String password, String senderName, String signature, boolean useSignature)
		throws PortalException, SystemException {

		return createAccountEntry(
			userId, type, messageAddress, username, password, false, senderName, signature,
			 useSignature);
	}

	public List<AccountEntry> getCompanyAccountEntries(long companyId)
		throws SystemException {

		return accountEntryPersistence.findByCompanyId(companyId);
	}

	public AccountEntry getCompanyAccountEntry(
			long companyId, String type, String messageAddress)
		throws NoSuchAccountEntryException, SystemException {

		return accountEntryPersistence.findByC_T_M(
			companyId, type, messageAddress);
	}

	public List<AccountEntry> getUserAccountEntries(long userId)
		throws SystemException {

		return accountEntryPersistence.findByUserId(userId);
	}

	public List<AccountEntry> getUserAccountEntries(long userId, String type)
		throws SystemException {

		return accountEntryPersistence.findByU_T(userId, type);
	}

	public List<AccountEntry> getUserAccountEntries(
			long userId, String type, boolean initialized)
		throws SystemException {

		return accountEntryPersistence.findByU_T_I(userId, type, initialized);
	}

	public AccountEntry getUserAccountEntry(
			long userId, String type, String messageAddress)
		throws NoSuchAccountEntryException, SystemException {

		return accountEntryPersistence.findByU_T_M(
			userId, type, messageAddress);
	}

	public AccountEntry updateAccountEntry(
			long accountEntryId, String messageAddress, String username,
			String password, boolean passwordSaved, String senderName, String signature, 
			boolean useSignature)
		throws PortalException, SystemException {

		AccountEntry accountEntry = getAccountEntry(accountEntryId);

		accountEntry.setMessageAddress(messageAddress);
		accountEntry.setUsername(username);
		accountEntry.setPasswordDecrypted(password);
		accountEntry.setPasswordSaved(passwordSaved);
		accountEntry.setSenderName(senderName);
		
		if(useSignature){
			accountEntry.setSignature(signature);
		}else {
			accountEntry.setSignature("");
		}
		accountEntry.setUseSignature(useSignature);


		return accountEntryPersistence.update(accountEntry);
	}

}