/**
 * Copyright (c) 2000-2009 Liferay, Inc. All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package com.liferay.mail.service.impl;

import com.liferay.counter.service.CounterLocalServiceUtil;
import com.liferay.mail.model.AccountSetting;
import com.liferay.mail.service.base.AccountSettingLocalServiceBaseImpl;
import com.liferay.portal.PortalException;
import com.liferay.portal.SystemException;

import java.util.Date;

/**
 * <a href="AccountSettingLocalServiceImpl.java.html"><b><i>View Source</i></b>
 * </a>
 *
 * @author Scott Lee
 *
 */
public class AccountSettingLocalServiceImpl
	extends AccountSettingLocalServiceBaseImpl {

	public AccountSetting addAccountSettings(
			long companyId, long accountEntryId, String type,
			String mailInHostName, int mailInPort, boolean mailInSecure,
			String mailOutHostName, int mailOutPort, boolean mailOutSecure,
			String defaultFolderPrefix)
		throws SystemException {

		AccountSetting accountSetting = createAccountSettings(
			companyId, accountEntryId, type, mailInHostName, mailInPort,
			mailInSecure, mailOutHostName, mailOutPort, mailOutSecure,
			defaultFolderPrefix);

		return accountSettingPersistence.update(accountSetting);
	}

	public AccountSetting createAccountSettings(
			long companyId, long accountEntryId, String type,
			String mailInHostName, int mailInPort, boolean mailInSecure,
			String mailOutHostName, int mailOutPort, boolean mailOutSecure,
			String defaultFolderPrefix)
		throws SystemException {

		long accountSettingId = CounterLocalServiceUtil.increment();

		AccountSetting accountSetting = accountSettingPersistence.create(
			accountSettingId);

		Date now = new Date();

		accountSetting.setCompanyId(companyId);
		accountSetting.setCreateDate(now);
		accountSetting.setModifiedDate(now);

		accountSetting.setAccountEntryId(accountEntryId);
		accountSetting.setType(type);
		accountSetting.setMailInHostName(mailInHostName);
		accountSetting.setMailInPort(mailInPort);
		accountSetting.setMailInSecure(mailInSecure);
		accountSetting.setMailOutHostName(mailOutHostName);
		accountSetting.setMailOutPort(mailOutPort);
		accountSetting.setMailOutSecure(mailOutSecure);
		accountSetting.setDefaultFolderPrefix(defaultFolderPrefix);

		return accountSetting;
	}

	public void deleteAEAccountSetting(long accountEntryId)
		throws PortalException, SystemException {

		accountSettingPersistence.removeByAccountEntryId(accountEntryId);
	}

	public AccountSetting getAEAccountSetting(long accountEntryId)
		throws PortalException, SystemException {

		return accountSettingPersistence.findByAccountEntryId(accountEntryId);
	}

	public AccountSetting updateAccountSettings(
			long accountSettingId, String mailInHostName, int mailInPort,
			boolean mailInSecure, String mailOutHostName, int mailOutPort,
			boolean mailOutSecure, String defaultFolderPrefix)
		throws PortalException, SystemException {

		AccountSetting accountSetting = getAccountSetting(
			accountSettingId);

		accountSetting.setMailInHostName(mailInHostName);
		accountSetting.setMailInPort(mailInPort);
		accountSetting.setMailInSecure(mailInSecure);
		accountSetting.setMailOutHostName(mailOutHostName);
		accountSetting.setMailOutPort(mailOutPort);
		accountSetting.setMailOutSecure(mailOutSecure);
		accountSetting.setDefaultFolderPrefix(defaultFolderPrefix);

		return accountSettingPersistence.update(accountSetting);
	}

}