/**
 * Copyright (c) 2000-2009 Liferay, Inc. All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package com.liferay.mail.service.impl;

import com.liferay.counter.service.CounterLocalServiceUtil;
import com.liferay.mail.model.Attachment;
import com.liferay.mail.service.base.AttachmentLocalServiceBaseImpl;
import com.liferay.portal.SystemException;
import com.liferay.portal.kernel.util.StringPool;
import com.liferay.portal.kernel.util.StringUtil;

import java.util.List;

/**
 * <a href="AttachmentLocalServiceImpl.java.html"><b><i>View Source</i></b></a>
 *
 * @author Scott Lee
 *
 */
public class AttachmentLocalServiceImpl extends AttachmentLocalServiceBaseImpl {

	public Attachment addAttachment(long messageId, String filename, long size)
		throws SystemException {

		return addAttachment(messageId, filename, size, StringPool.BLANK);
	}

	public Attachment addAttachment(
			long messageId, String filename, long size, String contentPath)
		throws SystemException {

		long attachmentId = CounterLocalServiceUtil.increment();

		Attachment attachment = attachmentPersistence.create(attachmentId);

		attachment.setMessageId(messageId);
		attachment.setFilename(StringUtil.shorten(filename, 150));
		attachment.setSize(size);
		attachment.setContentPath(contentPath);

		return attachmentPersistence.update(attachment);
	}

	public List<Attachment> getAttachments(long messageId)
		throws SystemException {

		return attachmentPersistence.findByMessageId(messageId);
	}

}