/**
 * Copyright (c) 2000-2009 Liferay, Inc. All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package com.liferay.mail.service.impl;

import com.liferay.counter.service.CounterLocalServiceUtil;
import com.liferay.mail.model.Folder;
import com.liferay.mail.service.base.FolderLocalServiceBaseImpl;
import com.liferay.portal.PortalException;
import com.liferay.portal.SystemException;
import com.liferay.portal.model.User;
import com.liferay.portal.service.UserLocalServiceUtil;

import java.util.Date;
import java.util.List;

/**
 * <a href="FolderLocalServiceImpl.java.html"><b><i>View Source</i></b></a>
 *
 * @author Scott Lee
 *
 */
public class FolderLocalServiceImpl extends FolderLocalServiceBaseImpl {

	public Folder addFolder(
			long userId, long accountEntryId, String fullName, String name,
			int remoteMessageCount)
		throws PortalException, SystemException {

		Folder folder = createFolder(
			userId, accountEntryId, fullName, name, remoteMessageCount);

		return folderPersistence.update(folder);
	}

	public Folder getFolder(long accountEntryId, String name)
		throws PortalException, SystemException {

		return folderPersistence.findByA_N(accountEntryId, name);
	}

	public List<Folder> getFolders(long accountEntryId) throws SystemException {
		return folderPersistence.findByAccountEntryId(accountEntryId);
	}

	public Folder renameFolder(long folderId, String fullName, String name)
		throws PortalException, SystemException {

		Folder folder = getFolder(folderId);

		folder.setFullName(fullName);
		folder.setName(name);

		return folderPersistence.update(folder);
	}

	public Folder updateRemoteMessageCount(
			long folderId, int remoteMessageCount)
		throws PortalException, SystemException {

		Folder folder = getFolder(folderId);

		folder.setRemoteMessageCount(remoteMessageCount);
		folder.setInitialized(true);
		folder.setSynchronizedDate(new Date());

		return folderPersistence.update(folder);
	}

	protected Folder createFolder(
			long userId, long accountEntryId, String fullName, String name,
			int remoteMessageCount)
		throws PortalException, SystemException {

		Date now = new Date();

		User user = UserLocalServiceUtil.getUserById(userId);

		long folderId = CounterLocalServiceUtil.increment();

		Folder folder = folderPersistence.create(folderId);

		folder.setCompanyId(user.getCompanyId());
		folder.setCreateDate(now);
		folder.setModifiedDate(now);
		folder.setAccountEntryId(accountEntryId);
		folder.setFullName(fullName);
		folder.setName(name);
		folder.setInitialized(false);
		folder.setRemoteMessageCount(remoteMessageCount);
		folder.setSynchronizedDate(now);

		return folder;
	}

}