/**
 * Copyright (c) 2000-2009 Liferay, Inc. All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package com.liferay.mail.service.persistence;

import com.liferay.mail.NoSuchAccountEntryException;
import com.liferay.mail.model.AccountEntry;
import com.liferay.mail.model.impl.AccountEntryImpl;
import com.liferay.mail.model.impl.AccountEntryModelImpl;

import com.liferay.portal.SystemException;
import com.liferay.portal.kernel.annotation.BeanReference;
import com.liferay.portal.kernel.cache.CacheRegistry;
import com.liferay.portal.kernel.dao.orm.DynamicQuery;
import com.liferay.portal.kernel.dao.orm.EntityCacheUtil;
import com.liferay.portal.kernel.dao.orm.FinderCacheUtil;
import com.liferay.portal.kernel.dao.orm.FinderPath;
import com.liferay.portal.kernel.dao.orm.Query;
import com.liferay.portal.kernel.dao.orm.QueryPos;
import com.liferay.portal.kernel.dao.orm.QueryUtil;
import com.liferay.portal.kernel.dao.orm.Session;
import com.liferay.portal.kernel.log.Log;
import com.liferay.portal.kernel.log.LogFactoryUtil;
import com.liferay.portal.kernel.util.GetterUtil;
import com.liferay.portal.kernel.util.OrderByComparator;
import com.liferay.portal.kernel.util.StringPool;
import com.liferay.portal.kernel.util.StringUtil;
import com.liferay.portal.kernel.util.Validator;
import com.liferay.portal.model.ModelListener;
import com.liferay.portal.service.persistence.BatchSessionUtil;
import com.liferay.portal.service.persistence.impl.BasePersistenceImpl;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

/**
 * <a href="AccountEntryPersistenceImpl.java.html"><b><i>View Source</i></b></a>
 *
 * @author Scott Lee
 */
public class AccountEntryPersistenceImpl extends BasePersistenceImpl
	implements AccountEntryPersistence {
	public static final String FINDER_CLASS_NAME_ENTITY = AccountEntryImpl.class.getName();
	public static final String FINDER_CLASS_NAME_LIST = FINDER_CLASS_NAME_ENTITY +
		".List";
	public static final FinderPath FINDER_PATH_FIND_BY_COMPANYID = new FinderPath(AccountEntryModelImpl.ENTITY_CACHE_ENABLED,
			AccountEntryModelImpl.FINDER_CACHE_ENABLED, FINDER_CLASS_NAME_LIST,
			"findByCompanyId", new String[] { Long.class.getName() });
	public static final FinderPath FINDER_PATH_FIND_BY_OBC_COMPANYID = new FinderPath(AccountEntryModelImpl.ENTITY_CACHE_ENABLED,
			AccountEntryModelImpl.FINDER_CACHE_ENABLED, FINDER_CLASS_NAME_LIST,
			"findByCompanyId",
			new String[] {
				Long.class.getName(),
				
			"java.lang.Integer", "java.lang.Integer",
				"com.liferay.portal.kernel.util.OrderByComparator"
			});
	public static final FinderPath FINDER_PATH_COUNT_BY_COMPANYID = new FinderPath(AccountEntryModelImpl.ENTITY_CACHE_ENABLED,
			AccountEntryModelImpl.FINDER_CACHE_ENABLED, FINDER_CLASS_NAME_LIST,
			"countByCompanyId", new String[] { Long.class.getName() });
	public static final FinderPath FINDER_PATH_FETCH_BY_C_T_M = new FinderPath(AccountEntryModelImpl.ENTITY_CACHE_ENABLED,
			AccountEntryModelImpl.FINDER_CACHE_ENABLED,
			FINDER_CLASS_NAME_ENTITY, "fetchByC_T_M",
			new String[] {
				Long.class.getName(), String.class.getName(),
				String.class.getName()
			});
	public static final FinderPath FINDER_PATH_COUNT_BY_C_T_M = new FinderPath(AccountEntryModelImpl.ENTITY_CACHE_ENABLED,
			AccountEntryModelImpl.FINDER_CACHE_ENABLED, FINDER_CLASS_NAME_LIST,
			"countByC_T_M",
			new String[] {
				Long.class.getName(), String.class.getName(),
				String.class.getName()
			});
	public static final FinderPath FINDER_PATH_FIND_BY_USERID = new FinderPath(AccountEntryModelImpl.ENTITY_CACHE_ENABLED,
			AccountEntryModelImpl.FINDER_CACHE_ENABLED, FINDER_CLASS_NAME_LIST,
			"findByUserId", new String[] { Long.class.getName() });
	public static final FinderPath FINDER_PATH_FIND_BY_OBC_USERID = new FinderPath(AccountEntryModelImpl.ENTITY_CACHE_ENABLED,
			AccountEntryModelImpl.FINDER_CACHE_ENABLED, FINDER_CLASS_NAME_LIST,
			"findByUserId",
			new String[] {
				Long.class.getName(),
				
			"java.lang.Integer", "java.lang.Integer",
				"com.liferay.portal.kernel.util.OrderByComparator"
			});
	public static final FinderPath FINDER_PATH_COUNT_BY_USERID = new FinderPath(AccountEntryModelImpl.ENTITY_CACHE_ENABLED,
			AccountEntryModelImpl.FINDER_CACHE_ENABLED, FINDER_CLASS_NAME_LIST,
			"countByUserId", new String[] { Long.class.getName() });
	public static final FinderPath FINDER_PATH_FIND_BY_U_T = new FinderPath(AccountEntryModelImpl.ENTITY_CACHE_ENABLED,
			AccountEntryModelImpl.FINDER_CACHE_ENABLED, FINDER_CLASS_NAME_LIST,
			"findByU_T",
			new String[] { Long.class.getName(), String.class.getName() });
	public static final FinderPath FINDER_PATH_FIND_BY_OBC_U_T = new FinderPath(AccountEntryModelImpl.ENTITY_CACHE_ENABLED,
			AccountEntryModelImpl.FINDER_CACHE_ENABLED, FINDER_CLASS_NAME_LIST,
			"findByU_T",
			new String[] {
				Long.class.getName(), String.class.getName(),
				
			"java.lang.Integer", "java.lang.Integer",
				"com.liferay.portal.kernel.util.OrderByComparator"
			});
	public static final FinderPath FINDER_PATH_COUNT_BY_U_T = new FinderPath(AccountEntryModelImpl.ENTITY_CACHE_ENABLED,
			AccountEntryModelImpl.FINDER_CACHE_ENABLED, FINDER_CLASS_NAME_LIST,
			"countByU_T",
			new String[] { Long.class.getName(), String.class.getName() });
	public static final FinderPath FINDER_PATH_FIND_BY_U_T_I = new FinderPath(AccountEntryModelImpl.ENTITY_CACHE_ENABLED,
			AccountEntryModelImpl.FINDER_CACHE_ENABLED, FINDER_CLASS_NAME_LIST,
			"findByU_T_I",
			new String[] {
				Long.class.getName(), String.class.getName(),
				Boolean.class.getName()
			});
	public static final FinderPath FINDER_PATH_FIND_BY_OBC_U_T_I = new FinderPath(AccountEntryModelImpl.ENTITY_CACHE_ENABLED,
			AccountEntryModelImpl.FINDER_CACHE_ENABLED, FINDER_CLASS_NAME_LIST,
			"findByU_T_I",
			new String[] {
				Long.class.getName(), String.class.getName(),
				Boolean.class.getName(),
				
			"java.lang.Integer", "java.lang.Integer",
				"com.liferay.portal.kernel.util.OrderByComparator"
			});
	public static final FinderPath FINDER_PATH_COUNT_BY_U_T_I = new FinderPath(AccountEntryModelImpl.ENTITY_CACHE_ENABLED,
			AccountEntryModelImpl.FINDER_CACHE_ENABLED, FINDER_CLASS_NAME_LIST,
			"countByU_T_I",
			new String[] {
				Long.class.getName(), String.class.getName(),
				Boolean.class.getName()
			});
	public static final FinderPath FINDER_PATH_FETCH_BY_U_T_M = new FinderPath(AccountEntryModelImpl.ENTITY_CACHE_ENABLED,
			AccountEntryModelImpl.FINDER_CACHE_ENABLED,
			FINDER_CLASS_NAME_ENTITY, "fetchByU_T_M",
			new String[] {
				Long.class.getName(), String.class.getName(),
				String.class.getName()
			});
	public static final FinderPath FINDER_PATH_COUNT_BY_U_T_M = new FinderPath(AccountEntryModelImpl.ENTITY_CACHE_ENABLED,
			AccountEntryModelImpl.FINDER_CACHE_ENABLED, FINDER_CLASS_NAME_LIST,
			"countByU_T_M",
			new String[] {
				Long.class.getName(), String.class.getName(),
				String.class.getName()
			});
	public static final FinderPath FINDER_PATH_FIND_ALL = new FinderPath(AccountEntryModelImpl.ENTITY_CACHE_ENABLED,
			AccountEntryModelImpl.FINDER_CACHE_ENABLED, FINDER_CLASS_NAME_LIST,
			"findAll", new String[0]);
	public static final FinderPath FINDER_PATH_COUNT_ALL = new FinderPath(AccountEntryModelImpl.ENTITY_CACHE_ENABLED,
			AccountEntryModelImpl.FINDER_CACHE_ENABLED, FINDER_CLASS_NAME_LIST,
			"countAll", new String[0]);

	public void cacheResult(AccountEntry accountEntry) {
		EntityCacheUtil.putResult(AccountEntryModelImpl.ENTITY_CACHE_ENABLED,
			AccountEntryImpl.class, accountEntry.getPrimaryKey(), accountEntry);

		FinderCacheUtil.putResult(FINDER_PATH_FETCH_BY_C_T_M,
			new Object[] {
				new Long(accountEntry.getCompanyId()),
				
			accountEntry.getType(),
				
			accountEntry.getMessageAddress()
			}, accountEntry);

		FinderCacheUtil.putResult(FINDER_PATH_FETCH_BY_U_T_M,
			new Object[] {
				new Long(accountEntry.getUserId()),
				
			accountEntry.getType(),
				
			accountEntry.getMessageAddress()
			}, accountEntry);
	}

	public void cacheResult(List<AccountEntry> accountEntries) {
		for (AccountEntry accountEntry : accountEntries) {
			if (EntityCacheUtil.getResult(
						AccountEntryModelImpl.ENTITY_CACHE_ENABLED,
						AccountEntryImpl.class, accountEntry.getPrimaryKey(),
						this) == null) {
				cacheResult(accountEntry);
			}
		}
	}

	public void clearCache() {
		CacheRegistry.clear(AccountEntryImpl.class.getName());
		EntityCacheUtil.clearCache(AccountEntryImpl.class.getName());
		FinderCacheUtil.clearCache(FINDER_CLASS_NAME_ENTITY);
		FinderCacheUtil.clearCache(FINDER_CLASS_NAME_LIST);
	}

	public AccountEntry create(long accountEntryId) {
		AccountEntry accountEntry = new AccountEntryImpl();

		accountEntry.setNew(true);
		accountEntry.setPrimaryKey(accountEntryId);

		return accountEntry;
	}

	public AccountEntry remove(long accountEntryId)
		throws NoSuchAccountEntryException, SystemException {
		Session session = null;

		try {
			session = openSession();

			AccountEntry accountEntry = (AccountEntry)session.get(AccountEntryImpl.class,
					new Long(accountEntryId));

			if (accountEntry == null) {
				if (_log.isWarnEnabled()) {
					_log.warn("No AccountEntry exists with the primary key " +
						accountEntryId);
				}

				throw new NoSuchAccountEntryException(
					"No AccountEntry exists with the primary key " +
					accountEntryId);
			}

			return remove(accountEntry);
		}
		catch (NoSuchAccountEntryException nsee) {
			throw nsee;
		}
		catch (Exception e) {
			throw processException(e);
		}
		finally {
			closeSession(session);
		}
	}

	public AccountEntry remove(AccountEntry accountEntry)
		throws SystemException {
		for (ModelListener<AccountEntry> listener : listeners) {
			listener.onBeforeRemove(accountEntry);
		}

		accountEntry = removeImpl(accountEntry);

		for (ModelListener<AccountEntry> listener : listeners) {
			listener.onAfterRemove(accountEntry);
		}

		return accountEntry;
	}

	protected AccountEntry removeImpl(AccountEntry accountEntry)
		throws SystemException {
		Session session = null;

		try {
			session = openSession();

			if (accountEntry.isCachedModel() || BatchSessionUtil.isEnabled()) {
				Object staleObject = session.get(AccountEntryImpl.class,
						accountEntry.getPrimaryKeyObj());

				if (staleObject != null) {
					session.evict(staleObject);
				}
			}

			session.delete(accountEntry);

			session.flush();
		}
		catch (Exception e) {
			throw processException(e);
		}
		finally {
			closeSession(session);
		}

		FinderCacheUtil.clearCache(FINDER_CLASS_NAME_LIST);

		AccountEntryModelImpl accountEntryModelImpl = (AccountEntryModelImpl)accountEntry;

		FinderCacheUtil.removeResult(FINDER_PATH_FETCH_BY_C_T_M,
			new Object[] {
				new Long(accountEntryModelImpl.getOriginalCompanyId()),
				
			accountEntryModelImpl.getOriginalType(),
				
			accountEntryModelImpl.getOriginalMessageAddress()
			});

		FinderCacheUtil.removeResult(FINDER_PATH_FETCH_BY_U_T_M,
			new Object[] {
				new Long(accountEntryModelImpl.getOriginalUserId()),
				
			accountEntryModelImpl.getOriginalType(),
				
			accountEntryModelImpl.getOriginalMessageAddress()
			});

		EntityCacheUtil.removeResult(AccountEntryModelImpl.ENTITY_CACHE_ENABLED,
			AccountEntryImpl.class, accountEntry.getPrimaryKey());

		return accountEntry;
	}

	public AccountEntry update(AccountEntry accountEntry)
		throws SystemException {
		if (_log.isWarnEnabled()) {
			_log.warn(
				"Using the deprecated update(AccountEntry accountEntry) method. Use update(AccountEntry accountEntry, boolean merge) instead.");
		}

		return update(accountEntry, false);
	}

	public AccountEntry update(AccountEntry accountEntry, boolean merge)
		throws SystemException {
		boolean isNew = accountEntry.isNew();

		for (ModelListener<AccountEntry> listener : listeners) {
			if (isNew) {
				listener.onBeforeCreate(accountEntry);
			}
			else {
				listener.onBeforeUpdate(accountEntry);
			}
		}

		accountEntry = updateImpl(accountEntry, merge);

		for (ModelListener<AccountEntry> listener : listeners) {
			if (isNew) {
				listener.onAfterCreate(accountEntry);
			}
			else {
				listener.onAfterUpdate(accountEntry);
			}
		}

		return accountEntry;
	}

	public AccountEntry updateImpl(
		com.liferay.mail.model.AccountEntry accountEntry, boolean merge)
		throws SystemException {
		boolean isNew = accountEntry.isNew();

		AccountEntryModelImpl accountEntryModelImpl = (AccountEntryModelImpl)accountEntry;

		Session session = null;

		try {
			session = openSession();

			BatchSessionUtil.update(session, accountEntry, merge);

			accountEntry.setNew(false);
		}
		catch (Exception e) {
			throw processException(e);
		}
		finally {
			closeSession(session);
		}

		FinderCacheUtil.clearCache(FINDER_CLASS_NAME_LIST);

		EntityCacheUtil.putResult(AccountEntryModelImpl.ENTITY_CACHE_ENABLED,
			AccountEntryImpl.class, accountEntry.getPrimaryKey(), accountEntry);

		if (!isNew &&
				((accountEntry.getCompanyId() != accountEntryModelImpl.getOriginalCompanyId()) ||
				!Validator.equals(accountEntry.getType(),
					accountEntryModelImpl.getOriginalType()) ||
				!Validator.equals(accountEntry.getMessageAddress(),
					accountEntryModelImpl.getOriginalMessageAddress()))) {
			FinderCacheUtil.removeResult(FINDER_PATH_FETCH_BY_C_T_M,
				new Object[] {
					new Long(accountEntryModelImpl.getOriginalCompanyId()),
					
				accountEntryModelImpl.getOriginalType(),
					
				accountEntryModelImpl.getOriginalMessageAddress()
				});
		}

		if (isNew ||
				((accountEntry.getCompanyId() != accountEntryModelImpl.getOriginalCompanyId()) ||
				!Validator.equals(accountEntry.getType(),
					accountEntryModelImpl.getOriginalType()) ||
				!Validator.equals(accountEntry.getMessageAddress(),
					accountEntryModelImpl.getOriginalMessageAddress()))) {
			FinderCacheUtil.putResult(FINDER_PATH_FETCH_BY_C_T_M,
				new Object[] {
					new Long(accountEntry.getCompanyId()),
					
				accountEntry.getType(),
					
				accountEntry.getMessageAddress()
				}, accountEntry);
		}

		if (!isNew &&
				((accountEntry.getUserId() != accountEntryModelImpl.getOriginalUserId()) ||
				!Validator.equals(accountEntry.getType(),
					accountEntryModelImpl.getOriginalType()) ||
				!Validator.equals(accountEntry.getMessageAddress(),
					accountEntryModelImpl.getOriginalMessageAddress()))) {
			FinderCacheUtil.removeResult(FINDER_PATH_FETCH_BY_U_T_M,
				new Object[] {
					new Long(accountEntryModelImpl.getOriginalUserId()),
					
				accountEntryModelImpl.getOriginalType(),
					
				accountEntryModelImpl.getOriginalMessageAddress()
				});
		}

		if (isNew ||
				((accountEntry.getUserId() != accountEntryModelImpl.getOriginalUserId()) ||
				!Validator.equals(accountEntry.getType(),
					accountEntryModelImpl.getOriginalType()) ||
				!Validator.equals(accountEntry.getMessageAddress(),
					accountEntryModelImpl.getOriginalMessageAddress()))) {
			FinderCacheUtil.putResult(FINDER_PATH_FETCH_BY_U_T_M,
				new Object[] {
					new Long(accountEntry.getUserId()),
					
				accountEntry.getType(),
					
				accountEntry.getMessageAddress()
				}, accountEntry);
		}

		return accountEntry;
	}

	public AccountEntry findByPrimaryKey(long accountEntryId)
		throws NoSuchAccountEntryException, SystemException {
		AccountEntry accountEntry = fetchByPrimaryKey(accountEntryId);

		if (accountEntry == null) {
			if (_log.isWarnEnabled()) {
				_log.warn("No AccountEntry exists with the primary key " +
					accountEntryId);
			}

			throw new NoSuchAccountEntryException(
				"No AccountEntry exists with the primary key " +
				accountEntryId);
		}

		return accountEntry;
	}

	public AccountEntry fetchByPrimaryKey(long accountEntryId)
		throws SystemException {
		AccountEntry accountEntry = (AccountEntry)EntityCacheUtil.getResult(AccountEntryModelImpl.ENTITY_CACHE_ENABLED,
				AccountEntryImpl.class, accountEntryId, this);

		if (accountEntry == null) {
			Session session = null;

			try {
				session = openSession();

				accountEntry = (AccountEntry)session.get(AccountEntryImpl.class,
						new Long(accountEntryId));
			}
			catch (Exception e) {
				throw processException(e);
			}
			finally {
				if (accountEntry != null) {
					cacheResult(accountEntry);
				}

				closeSession(session);
			}
		}

		return accountEntry;
	}

	public List<AccountEntry> findByCompanyId(long companyId)
		throws SystemException {
		Object[] finderArgs = new Object[] { new Long(companyId) };

		List<AccountEntry> list = (List<AccountEntry>)FinderCacheUtil.getResult(FINDER_PATH_FIND_BY_COMPANYID,
				finderArgs, this);

		if (list == null) {
			Session session = null;

			try {
				session = openSession();

				StringBuilder query = new StringBuilder();

				query.append(
					"SELECT accountEntry FROM AccountEntry accountEntry WHERE ");

				query.append("accountEntry.companyId = ?");

				query.append(" ");

				query.append("ORDER BY ");

				query.append("accountEntry.messageAddress ASC");

				Query q = session.createQuery(query.toString());

				QueryPos qPos = QueryPos.getInstance(q);

				qPos.add(companyId);

				list = q.list();
			}
			catch (Exception e) {
				throw processException(e);
			}
			finally {
				if (list == null) {
					list = new ArrayList<AccountEntry>();
				}

				cacheResult(list);

				FinderCacheUtil.putResult(FINDER_PATH_FIND_BY_COMPANYID,
					finderArgs, list);

				closeSession(session);
			}
		}

		return list;
	}

	public List<AccountEntry> findByCompanyId(long companyId, int start, int end)
		throws SystemException {
		return findByCompanyId(companyId, start, end, null);
	}

	public List<AccountEntry> findByCompanyId(long companyId, int start,
		int end, OrderByComparator obc) throws SystemException {
		Object[] finderArgs = new Object[] {
				new Long(companyId),
				
				String.valueOf(start), String.valueOf(end), String.valueOf(obc)
			};

		List<AccountEntry> list = (List<AccountEntry>)FinderCacheUtil.getResult(FINDER_PATH_FIND_BY_OBC_COMPANYID,
				finderArgs, this);

		if (list == null) {
			Session session = null;

			try {
				session = openSession();

				StringBuilder query = new StringBuilder();

				query.append(
					"SELECT accountEntry FROM AccountEntry accountEntry WHERE ");

				query.append("accountEntry.companyId = ?");

				query.append(" ");

				if (obc != null) {
					query.append("ORDER BY ");

					String[] orderByFields = obc.getOrderByFields();

					for (int i = 0; i < orderByFields.length; i++) {
						query.append("accountEntry.");
						query.append(orderByFields[i]);

						if (obc.isAscending()) {
							query.append(" ASC");
						}
						else {
							query.append(" DESC");
						}

						if ((i + 1) < orderByFields.length) {
							query.append(", ");
						}
					}
				}

				else {
					query.append("ORDER BY ");

					query.append("accountEntry.messageAddress ASC");
				}

				Query q = session.createQuery(query.toString());

				QueryPos qPos = QueryPos.getInstance(q);

				qPos.add(companyId);

				list = (List<AccountEntry>)QueryUtil.list(q, getDialect(),
						start, end);
			}
			catch (Exception e) {
				throw processException(e);
			}
			finally {
				if (list == null) {
					list = new ArrayList<AccountEntry>();
				}

				cacheResult(list);

				FinderCacheUtil.putResult(FINDER_PATH_FIND_BY_OBC_COMPANYID,
					finderArgs, list);

				closeSession(session);
			}
		}

		return list;
	}

	public AccountEntry findByCompanyId_First(long companyId,
		OrderByComparator obc)
		throws NoSuchAccountEntryException, SystemException {
		List<AccountEntry> list = findByCompanyId(companyId, 0, 1, obc);

		if (list.isEmpty()) {
			StringBuilder msg = new StringBuilder();

			msg.append("No AccountEntry exists with the key {");

			msg.append("companyId=" + companyId);

			msg.append(StringPool.CLOSE_CURLY_BRACE);

			throw new NoSuchAccountEntryException(msg.toString());
		}
		else {
			return list.get(0);
		}
	}

	public AccountEntry findByCompanyId_Last(long companyId,
		OrderByComparator obc)
		throws NoSuchAccountEntryException, SystemException {
		int count = countByCompanyId(companyId);

		List<AccountEntry> list = findByCompanyId(companyId, count - 1, count,
				obc);

		if (list.isEmpty()) {
			StringBuilder msg = new StringBuilder();

			msg.append("No AccountEntry exists with the key {");

			msg.append("companyId=" + companyId);

			msg.append(StringPool.CLOSE_CURLY_BRACE);

			throw new NoSuchAccountEntryException(msg.toString());
		}
		else {
			return list.get(0);
		}
	}

	public AccountEntry[] findByCompanyId_PrevAndNext(long accountEntryId,
		long companyId, OrderByComparator obc)
		throws NoSuchAccountEntryException, SystemException {
		AccountEntry accountEntry = findByPrimaryKey(accountEntryId);

		int count = countByCompanyId(companyId);

		Session session = null;

		try {
			session = openSession();

			StringBuilder query = new StringBuilder();

			query.append(
				"SELECT accountEntry FROM AccountEntry accountEntry WHERE ");

			query.append("accountEntry.companyId = ?");

			query.append(" ");

			if (obc != null) {
				query.append("ORDER BY ");

				String[] orderByFields = obc.getOrderByFields();

				for (int i = 0; i < orderByFields.length; i++) {
					query.append("accountEntry.");
					query.append(orderByFields[i]);

					if (obc.isAscending()) {
						query.append(" ASC");
					}
					else {
						query.append(" DESC");
					}

					if ((i + 1) < orderByFields.length) {
						query.append(", ");
					}
				}
			}

			else {
				query.append("ORDER BY ");

				query.append("accountEntry.messageAddress ASC");
			}

			Query q = session.createQuery(query.toString());

			QueryPos qPos = QueryPos.getInstance(q);

			qPos.add(companyId);

			Object[] objArray = QueryUtil.getPrevAndNext(q, count, obc,
					accountEntry);

			AccountEntry[] array = new AccountEntryImpl[3];

			array[0] = (AccountEntry)objArray[0];
			array[1] = (AccountEntry)objArray[1];
			array[2] = (AccountEntry)objArray[2];

			return array;
		}
		catch (Exception e) {
			throw processException(e);
		}
		finally {
			closeSession(session);
		}
	}

	public AccountEntry findByC_T_M(long companyId, String type,
		String messageAddress)
		throws NoSuchAccountEntryException, SystemException {
		AccountEntry accountEntry = fetchByC_T_M(companyId, type, messageAddress);

		if (accountEntry == null) {
			StringBuilder msg = new StringBuilder();

			msg.append("No AccountEntry exists with the key {");

			msg.append("companyId=" + companyId);

			msg.append(", ");
			msg.append("type=" + type);

			msg.append(", ");
			msg.append("messageAddress=" + messageAddress);

			msg.append(StringPool.CLOSE_CURLY_BRACE);

			if (_log.isWarnEnabled()) {
				_log.warn(msg.toString());
			}

			throw new NoSuchAccountEntryException(msg.toString());
		}

		return accountEntry;
	}

	public AccountEntry fetchByC_T_M(long companyId, String type,
		String messageAddress) throws SystemException {
		return fetchByC_T_M(companyId, type, messageAddress, true);
	}

	public AccountEntry fetchByC_T_M(long companyId, String type,
		String messageAddress, boolean retrieveFromCache)
		throws SystemException {
		Object[] finderArgs = new Object[] {
				new Long(companyId),
				
				type,
				
				messageAddress
			};

		Object result = null;

		if (retrieveFromCache) {
			result = FinderCacheUtil.getResult(FINDER_PATH_FETCH_BY_C_T_M,
					finderArgs, this);
		}

		if (result == null) {
			Session session = null;

			try {
				session = openSession();

				StringBuilder query = new StringBuilder();

				query.append(
					"SELECT accountEntry FROM AccountEntry accountEntry WHERE ");

				query.append("accountEntry.companyId = ?");

				query.append(" AND ");

				if (type == null) {
					query.append("accountEntry.type IS NULL");
				}
				else {
					query.append("accountEntry.type = ?");
				}

				query.append(" AND ");

				if (messageAddress == null) {
					query.append("accountEntry.messageAddress IS NULL");
				}
				else {
					query.append("accountEntry.messageAddress = ?");
				}

				query.append(" ");

				query.append("ORDER BY ");

				query.append("accountEntry.messageAddress ASC");

				Query q = session.createQuery(query.toString());

				QueryPos qPos = QueryPos.getInstance(q);

				qPos.add(companyId);

				if (type != null) {
					qPos.add(type);
				}

				if (messageAddress != null) {
					qPos.add(messageAddress);
				}

				List<AccountEntry> list = q.list();

				result = list;

				AccountEntry accountEntry = null;

				if (list.isEmpty()) {
					FinderCacheUtil.putResult(FINDER_PATH_FETCH_BY_C_T_M,
						finderArgs, list);
				}
				else {
					accountEntry = list.get(0);

					cacheResult(accountEntry);

					if ((accountEntry.getCompanyId() != companyId) ||
							(accountEntry.getType() == null) ||
							!accountEntry.getType().equals(type) ||
							(accountEntry.getMessageAddress() == null) ||
							!accountEntry.getMessageAddress()
											 .equals(messageAddress)) {
						FinderCacheUtil.putResult(FINDER_PATH_FETCH_BY_C_T_M,
							finderArgs, accountEntry);
					}
				}

				return accountEntry;
			}
			catch (Exception e) {
				throw processException(e);
			}
			finally {
				if (result == null) {
					FinderCacheUtil.putResult(FINDER_PATH_FETCH_BY_C_T_M,
						finderArgs, new ArrayList<AccountEntry>());
				}

				closeSession(session);
			}
		}
		else {
			if (result instanceof List<?>) {
				return null;
			}
			else {
				return (AccountEntry)result;
			}
		}
	}

	public List<AccountEntry> findByUserId(long userId)
		throws SystemException {
		Object[] finderArgs = new Object[] { new Long(userId) };

		List<AccountEntry> list = (List<AccountEntry>)FinderCacheUtil.getResult(FINDER_PATH_FIND_BY_USERID,
				finderArgs, this);

		if (list == null) {
			Session session = null;

			try {
				session = openSession();

				StringBuilder query = new StringBuilder();

				query.append(
					"SELECT accountEntry FROM AccountEntry accountEntry WHERE ");

				query.append("accountEntry.userId = ?");

				query.append(" ");

				query.append("ORDER BY ");

				query.append("accountEntry.messageAddress ASC");

				Query q = session.createQuery(query.toString());

				QueryPos qPos = QueryPos.getInstance(q);

				qPos.add(userId);

				list = q.list();
			}
			catch (Exception e) {
				throw processException(e);
			}
			finally {
				if (list == null) {
					list = new ArrayList<AccountEntry>();
				}

				cacheResult(list);

				FinderCacheUtil.putResult(FINDER_PATH_FIND_BY_USERID,
					finderArgs, list);

				closeSession(session);
			}
		}

		return list;
	}

	public List<AccountEntry> findByUserId(long userId, int start, int end)
		throws SystemException {
		return findByUserId(userId, start, end, null);
	}

	public List<AccountEntry> findByUserId(long userId, int start, int end,
		OrderByComparator obc) throws SystemException {
		Object[] finderArgs = new Object[] {
				new Long(userId),
				
				String.valueOf(start), String.valueOf(end), String.valueOf(obc)
			};

		List<AccountEntry> list = (List<AccountEntry>)FinderCacheUtil.getResult(FINDER_PATH_FIND_BY_OBC_USERID,
				finderArgs, this);

		if (list == null) {
			Session session = null;

			try {
				session = openSession();

				StringBuilder query = new StringBuilder();

				query.append(
					"SELECT accountEntry FROM AccountEntry accountEntry WHERE ");

				query.append("accountEntry.userId = ?");

				query.append(" ");

				if (obc != null) {
					query.append("ORDER BY ");

					String[] orderByFields = obc.getOrderByFields();

					for (int i = 0; i < orderByFields.length; i++) {
						query.append("accountEntry.");
						query.append(orderByFields[i]);

						if (obc.isAscending()) {
							query.append(" ASC");
						}
						else {
							query.append(" DESC");
						}

						if ((i + 1) < orderByFields.length) {
							query.append(", ");
						}
					}
				}

				else {
					query.append("ORDER BY ");

					query.append("accountEntry.messageAddress ASC");
				}

				Query q = session.createQuery(query.toString());

				QueryPos qPos = QueryPos.getInstance(q);

				qPos.add(userId);

				list = (List<AccountEntry>)QueryUtil.list(q, getDialect(),
						start, end);
			}
			catch (Exception e) {
				throw processException(e);
			}
			finally {
				if (list == null) {
					list = new ArrayList<AccountEntry>();
				}

				cacheResult(list);

				FinderCacheUtil.putResult(FINDER_PATH_FIND_BY_OBC_USERID,
					finderArgs, list);

				closeSession(session);
			}
		}

		return list;
	}

	public AccountEntry findByUserId_First(long userId, OrderByComparator obc)
		throws NoSuchAccountEntryException, SystemException {
		List<AccountEntry> list = findByUserId(userId, 0, 1, obc);

		if (list.isEmpty()) {
			StringBuilder msg = new StringBuilder();

			msg.append("No AccountEntry exists with the key {");

			msg.append("userId=" + userId);

			msg.append(StringPool.CLOSE_CURLY_BRACE);

			throw new NoSuchAccountEntryException(msg.toString());
		}
		else {
			return list.get(0);
		}
	}

	public AccountEntry findByUserId_Last(long userId, OrderByComparator obc)
		throws NoSuchAccountEntryException, SystemException {
		int count = countByUserId(userId);

		List<AccountEntry> list = findByUserId(userId, count - 1, count, obc);

		if (list.isEmpty()) {
			StringBuilder msg = new StringBuilder();

			msg.append("No AccountEntry exists with the key {");

			msg.append("userId=" + userId);

			msg.append(StringPool.CLOSE_CURLY_BRACE);

			throw new NoSuchAccountEntryException(msg.toString());
		}
		else {
			return list.get(0);
		}
	}

	public AccountEntry[] findByUserId_PrevAndNext(long accountEntryId,
		long userId, OrderByComparator obc)
		throws NoSuchAccountEntryException, SystemException {
		AccountEntry accountEntry = findByPrimaryKey(accountEntryId);

		int count = countByUserId(userId);

		Session session = null;

		try {
			session = openSession();

			StringBuilder query = new StringBuilder();

			query.append(
				"SELECT accountEntry FROM AccountEntry accountEntry WHERE ");

			query.append("accountEntry.userId = ?");

			query.append(" ");

			if (obc != null) {
				query.append("ORDER BY ");

				String[] orderByFields = obc.getOrderByFields();

				for (int i = 0; i < orderByFields.length; i++) {
					query.append("accountEntry.");
					query.append(orderByFields[i]);

					if (obc.isAscending()) {
						query.append(" ASC");
					}
					else {
						query.append(" DESC");
					}

					if ((i + 1) < orderByFields.length) {
						query.append(", ");
					}
				}
			}

			else {
				query.append("ORDER BY ");

				query.append("accountEntry.messageAddress ASC");
			}

			Query q = session.createQuery(query.toString());

			QueryPos qPos = QueryPos.getInstance(q);

			qPos.add(userId);

			Object[] objArray = QueryUtil.getPrevAndNext(q, count, obc,
					accountEntry);

			AccountEntry[] array = new AccountEntryImpl[3];

			array[0] = (AccountEntry)objArray[0];
			array[1] = (AccountEntry)objArray[1];
			array[2] = (AccountEntry)objArray[2];

			return array;
		}
		catch (Exception e) {
			throw processException(e);
		}
		finally {
			closeSession(session);
		}
	}

	public List<AccountEntry> findByU_T(long userId, String type)
		throws SystemException {
		Object[] finderArgs = new Object[] { new Long(userId), type };

		List<AccountEntry> list = (List<AccountEntry>)FinderCacheUtil.getResult(FINDER_PATH_FIND_BY_U_T,
				finderArgs, this);

		if (list == null) {
			Session session = null;

			try {
				session = openSession();

				StringBuilder query = new StringBuilder();

				query.append(
					"SELECT accountEntry FROM AccountEntry accountEntry WHERE ");

				query.append("accountEntry.userId = ?");

				query.append(" AND ");

				if (type == null) {
					query.append("accountEntry.type IS NULL");
				}
				else {
					query.append("accountEntry.type = ?");
				}

				query.append(" ");

				query.append("ORDER BY ");

				query.append("accountEntry.messageAddress ASC");

				Query q = session.createQuery(query.toString());

				QueryPos qPos = QueryPos.getInstance(q);

				qPos.add(userId);

				if (type != null) {
					qPos.add(type);
				}

				list = q.list();
			}
			catch (Exception e) {
				throw processException(e);
			}
			finally {
				if (list == null) {
					list = new ArrayList<AccountEntry>();
				}

				cacheResult(list);

				FinderCacheUtil.putResult(FINDER_PATH_FIND_BY_U_T, finderArgs,
					list);

				closeSession(session);
			}
		}

		return list;
	}

	public List<AccountEntry> findByU_T(long userId, String type, int start,
		int end) throws SystemException {
		return findByU_T(userId, type, start, end, null);
	}

	public List<AccountEntry> findByU_T(long userId, String type, int start,
		int end, OrderByComparator obc) throws SystemException {
		Object[] finderArgs = new Object[] {
				new Long(userId),
				
				type,
				
				String.valueOf(start), String.valueOf(end), String.valueOf(obc)
			};

		List<AccountEntry> list = (List<AccountEntry>)FinderCacheUtil.getResult(FINDER_PATH_FIND_BY_OBC_U_T,
				finderArgs, this);

		if (list == null) {
			Session session = null;

			try {
				session = openSession();

				StringBuilder query = new StringBuilder();

				query.append(
					"SELECT accountEntry FROM AccountEntry accountEntry WHERE ");

				query.append("accountEntry.userId = ?");

				query.append(" AND ");

				if (type == null) {
					query.append("accountEntry.type IS NULL");
				}
				else {
					query.append("accountEntry.type = ?");
				}

				query.append(" ");

				if (obc != null) {
					query.append("ORDER BY ");

					String[] orderByFields = obc.getOrderByFields();

					for (int i = 0; i < orderByFields.length; i++) {
						query.append("accountEntry.");
						query.append(orderByFields[i]);

						if (obc.isAscending()) {
							query.append(" ASC");
						}
						else {
							query.append(" DESC");
						}

						if ((i + 1) < orderByFields.length) {
							query.append(", ");
						}
					}
				}

				else {
					query.append("ORDER BY ");

					query.append("accountEntry.messageAddress ASC");
				}

				Query q = session.createQuery(query.toString());

				QueryPos qPos = QueryPos.getInstance(q);

				qPos.add(userId);

				if (type != null) {
					qPos.add(type);
				}

				list = (List<AccountEntry>)QueryUtil.list(q, getDialect(),
						start, end);
			}
			catch (Exception e) {
				throw processException(e);
			}
			finally {
				if (list == null) {
					list = new ArrayList<AccountEntry>();
				}

				cacheResult(list);

				FinderCacheUtil.putResult(FINDER_PATH_FIND_BY_OBC_U_T,
					finderArgs, list);

				closeSession(session);
			}
		}

		return list;
	}

	public AccountEntry findByU_T_First(long userId, String type,
		OrderByComparator obc)
		throws NoSuchAccountEntryException, SystemException {
		List<AccountEntry> list = findByU_T(userId, type, 0, 1, obc);

		if (list.isEmpty()) {
			StringBuilder msg = new StringBuilder();

			msg.append("No AccountEntry exists with the key {");

			msg.append("userId=" + userId);

			msg.append(", ");
			msg.append("type=" + type);

			msg.append(StringPool.CLOSE_CURLY_BRACE);

			throw new NoSuchAccountEntryException(msg.toString());
		}
		else {
			return list.get(0);
		}
	}

	public AccountEntry findByU_T_Last(long userId, String type,
		OrderByComparator obc)
		throws NoSuchAccountEntryException, SystemException {
		int count = countByU_T(userId, type);

		List<AccountEntry> list = findByU_T(userId, type, count - 1, count, obc);

		if (list.isEmpty()) {
			StringBuilder msg = new StringBuilder();

			msg.append("No AccountEntry exists with the key {");

			msg.append("userId=" + userId);

			msg.append(", ");
			msg.append("type=" + type);

			msg.append(StringPool.CLOSE_CURLY_BRACE);

			throw new NoSuchAccountEntryException(msg.toString());
		}
		else {
			return list.get(0);
		}
	}

	public AccountEntry[] findByU_T_PrevAndNext(long accountEntryId,
		long userId, String type, OrderByComparator obc)
		throws NoSuchAccountEntryException, SystemException {
		AccountEntry accountEntry = findByPrimaryKey(accountEntryId);

		int count = countByU_T(userId, type);

		Session session = null;

		try {
			session = openSession();

			StringBuilder query = new StringBuilder();

			query.append(
				"SELECT accountEntry FROM AccountEntry accountEntry WHERE ");

			query.append("accountEntry.userId = ?");

			query.append(" AND ");

			if (type == null) {
				query.append("accountEntry.type IS NULL");
			}
			else {
				query.append("accountEntry.type = ?");
			}

			query.append(" ");

			if (obc != null) {
				query.append("ORDER BY ");

				String[] orderByFields = obc.getOrderByFields();

				for (int i = 0; i < orderByFields.length; i++) {
					query.append("accountEntry.");
					query.append(orderByFields[i]);

					if (obc.isAscending()) {
						query.append(" ASC");
					}
					else {
						query.append(" DESC");
					}

					if ((i + 1) < orderByFields.length) {
						query.append(", ");
					}
				}
			}

			else {
				query.append("ORDER BY ");

				query.append("accountEntry.messageAddress ASC");
			}

			Query q = session.createQuery(query.toString());

			QueryPos qPos = QueryPos.getInstance(q);

			qPos.add(userId);

			if (type != null) {
				qPos.add(type);
			}

			Object[] objArray = QueryUtil.getPrevAndNext(q, count, obc,
					accountEntry);

			AccountEntry[] array = new AccountEntryImpl[3];

			array[0] = (AccountEntry)objArray[0];
			array[1] = (AccountEntry)objArray[1];
			array[2] = (AccountEntry)objArray[2];

			return array;
		}
		catch (Exception e) {
			throw processException(e);
		}
		finally {
			closeSession(session);
		}
	}

	public List<AccountEntry> findByU_T_I(long userId, String type,
		boolean initialized) throws SystemException {
		Object[] finderArgs = new Object[] {
				new Long(userId),
				
				type, Boolean.valueOf(initialized)
			};

		List<AccountEntry> list = (List<AccountEntry>)FinderCacheUtil.getResult(FINDER_PATH_FIND_BY_U_T_I,
				finderArgs, this);

		if (list == null) {
			Session session = null;

			try {
				session = openSession();

				StringBuilder query = new StringBuilder();

				query.append(
					"SELECT accountEntry FROM AccountEntry accountEntry WHERE ");

				query.append("accountEntry.userId = ?");

				query.append(" AND ");

				if (type == null) {
					query.append("accountEntry.type IS NULL");
				}
				else {
					query.append("accountEntry.type = ?");
				}

				query.append(" AND ");

				query.append("accountEntry.initialized = ?");

				query.append(" ");

				query.append("ORDER BY ");

				query.append("accountEntry.messageAddress ASC");

				Query q = session.createQuery(query.toString());

				QueryPos qPos = QueryPos.getInstance(q);

				qPos.add(userId);

				if (type != null) {
					qPos.add(type);
				}

				qPos.add(initialized);

				list = q.list();
			}
			catch (Exception e) {
				throw processException(e);
			}
			finally {
				if (list == null) {
					list = new ArrayList<AccountEntry>();
				}

				cacheResult(list);

				FinderCacheUtil.putResult(FINDER_PATH_FIND_BY_U_T_I,
					finderArgs, list);

				closeSession(session);
			}
		}

		return list;
	}

	public List<AccountEntry> findByU_T_I(long userId, String type,
		boolean initialized, int start, int end) throws SystemException {
		return findByU_T_I(userId, type, initialized, start, end, null);
	}

	public List<AccountEntry> findByU_T_I(long userId, String type,
		boolean initialized, int start, int end, OrderByComparator obc)
		throws SystemException {
		Object[] finderArgs = new Object[] {
				new Long(userId),
				
				type, Boolean.valueOf(initialized),
				
				String.valueOf(start), String.valueOf(end), String.valueOf(obc)
			};

		List<AccountEntry> list = (List<AccountEntry>)FinderCacheUtil.getResult(FINDER_PATH_FIND_BY_OBC_U_T_I,
				finderArgs, this);

		if (list == null) {
			Session session = null;

			try {
				session = openSession();

				StringBuilder query = new StringBuilder();

				query.append(
					"SELECT accountEntry FROM AccountEntry accountEntry WHERE ");

				query.append("accountEntry.userId = ?");

				query.append(" AND ");

				if (type == null) {
					query.append("accountEntry.type IS NULL");
				}
				else {
					query.append("accountEntry.type = ?");
				}

				query.append(" AND ");

				query.append("accountEntry.initialized = ?");

				query.append(" ");

				if (obc != null) {
					query.append("ORDER BY ");

					String[] orderByFields = obc.getOrderByFields();

					for (int i = 0; i < orderByFields.length; i++) {
						query.append("accountEntry.");
						query.append(orderByFields[i]);

						if (obc.isAscending()) {
							query.append(" ASC");
						}
						else {
							query.append(" DESC");
						}

						if ((i + 1) < orderByFields.length) {
							query.append(", ");
						}
					}
				}

				else {
					query.append("ORDER BY ");

					query.append("accountEntry.messageAddress ASC");
				}

				Query q = session.createQuery(query.toString());

				QueryPos qPos = QueryPos.getInstance(q);

				qPos.add(userId);

				if (type != null) {
					qPos.add(type);
				}

				qPos.add(initialized);

				list = (List<AccountEntry>)QueryUtil.list(q, getDialect(),
						start, end);
			}
			catch (Exception e) {
				throw processException(e);
			}
			finally {
				if (list == null) {
					list = new ArrayList<AccountEntry>();
				}

				cacheResult(list);

				FinderCacheUtil.putResult(FINDER_PATH_FIND_BY_OBC_U_T_I,
					finderArgs, list);

				closeSession(session);
			}
		}

		return list;
	}

	public AccountEntry findByU_T_I_First(long userId, String type,
		boolean initialized, OrderByComparator obc)
		throws NoSuchAccountEntryException, SystemException {
		List<AccountEntry> list = findByU_T_I(userId, type, initialized, 0, 1,
				obc);

		if (list.isEmpty()) {
			StringBuilder msg = new StringBuilder();

			msg.append("No AccountEntry exists with the key {");

			msg.append("userId=" + userId);

			msg.append(", ");
			msg.append("type=" + type);

			msg.append(", ");
			msg.append("initialized=" + initialized);

			msg.append(StringPool.CLOSE_CURLY_BRACE);

			throw new NoSuchAccountEntryException(msg.toString());
		}
		else {
			return list.get(0);
		}
	}

	public AccountEntry findByU_T_I_Last(long userId, String type,
		boolean initialized, OrderByComparator obc)
		throws NoSuchAccountEntryException, SystemException {
		int count = countByU_T_I(userId, type, initialized);

		List<AccountEntry> list = findByU_T_I(userId, type, initialized,
				count - 1, count, obc);

		if (list.isEmpty()) {
			StringBuilder msg = new StringBuilder();

			msg.append("No AccountEntry exists with the key {");

			msg.append("userId=" + userId);

			msg.append(", ");
			msg.append("type=" + type);

			msg.append(", ");
			msg.append("initialized=" + initialized);

			msg.append(StringPool.CLOSE_CURLY_BRACE);

			throw new NoSuchAccountEntryException(msg.toString());
		}
		else {
			return list.get(0);
		}
	}

	public AccountEntry[] findByU_T_I_PrevAndNext(long accountEntryId,
		long userId, String type, boolean initialized, OrderByComparator obc)
		throws NoSuchAccountEntryException, SystemException {
		AccountEntry accountEntry = findByPrimaryKey(accountEntryId);

		int count = countByU_T_I(userId, type, initialized);

		Session session = null;

		try {
			session = openSession();

			StringBuilder query = new StringBuilder();

			query.append(
				"SELECT accountEntry FROM AccountEntry accountEntry WHERE ");

			query.append("accountEntry.userId = ?");

			query.append(" AND ");

			if (type == null) {
				query.append("accountEntry.type IS NULL");
			}
			else {
				query.append("accountEntry.type = ?");
			}

			query.append(" AND ");

			query.append("accountEntry.initialized = ?");

			query.append(" ");

			if (obc != null) {
				query.append("ORDER BY ");

				String[] orderByFields = obc.getOrderByFields();

				for (int i = 0; i < orderByFields.length; i++) {
					query.append("accountEntry.");
					query.append(orderByFields[i]);

					if (obc.isAscending()) {
						query.append(" ASC");
					}
					else {
						query.append(" DESC");
					}

					if ((i + 1) < orderByFields.length) {
						query.append(", ");
					}
				}
			}

			else {
				query.append("ORDER BY ");

				query.append("accountEntry.messageAddress ASC");
			}

			Query q = session.createQuery(query.toString());

			QueryPos qPos = QueryPos.getInstance(q);

			qPos.add(userId);

			if (type != null) {
				qPos.add(type);
			}

			qPos.add(initialized);

			Object[] objArray = QueryUtil.getPrevAndNext(q, count, obc,
					accountEntry);

			AccountEntry[] array = new AccountEntryImpl[3];

			array[0] = (AccountEntry)objArray[0];
			array[1] = (AccountEntry)objArray[1];
			array[2] = (AccountEntry)objArray[2];

			return array;
		}
		catch (Exception e) {
			throw processException(e);
		}
		finally {
			closeSession(session);
		}
	}

	public AccountEntry findByU_T_M(long userId, String type,
		String messageAddress)
		throws NoSuchAccountEntryException, SystemException {
		AccountEntry accountEntry = fetchByU_T_M(userId, type, messageAddress);

		if (accountEntry == null) {
			StringBuilder msg = new StringBuilder();

			msg.append("No AccountEntry exists with the key {");

			msg.append("userId=" + userId);

			msg.append(", ");
			msg.append("type=" + type);

			msg.append(", ");
			msg.append("messageAddress=" + messageAddress);

			msg.append(StringPool.CLOSE_CURLY_BRACE);

			if (_log.isWarnEnabled()) {
				_log.warn(msg.toString());
			}

			throw new NoSuchAccountEntryException(msg.toString());
		}

		return accountEntry;
	}

	public AccountEntry fetchByU_T_M(long userId, String type,
		String messageAddress) throws SystemException {
		return fetchByU_T_M(userId, type, messageAddress, true);
	}

	public AccountEntry fetchByU_T_M(long userId, String type,
		String messageAddress, boolean retrieveFromCache)
		throws SystemException {
		Object[] finderArgs = new Object[] {
				new Long(userId),
				
				type,
				
				messageAddress
			};

		Object result = null;

		if (retrieveFromCache) {
			result = FinderCacheUtil.getResult(FINDER_PATH_FETCH_BY_U_T_M,
					finderArgs, this);
		}

		if (result == null) {
			Session session = null;

			try {
				session = openSession();

				StringBuilder query = new StringBuilder();

				query.append(
					"SELECT accountEntry FROM AccountEntry accountEntry WHERE ");

				query.append("accountEntry.userId = ?");

				query.append(" AND ");

				if (type == null) {
					query.append("accountEntry.type IS NULL");
				}
				else {
					query.append("accountEntry.type = ?");
				}

				query.append(" AND ");

				if (messageAddress == null) {
					query.append("accountEntry.messageAddress IS NULL");
				}
				else {
					query.append("accountEntry.messageAddress = ?");
				}

				query.append(" ");

				query.append("ORDER BY ");

				query.append("accountEntry.messageAddress ASC");

				Query q = session.createQuery(query.toString());

				QueryPos qPos = QueryPos.getInstance(q);

				qPos.add(userId);

				if (type != null) {
					qPos.add(type);
				}

				if (messageAddress != null) {
					qPos.add(messageAddress);
				}

				List<AccountEntry> list = q.list();

				result = list;

				AccountEntry accountEntry = null;

				if (list.isEmpty()) {
					FinderCacheUtil.putResult(FINDER_PATH_FETCH_BY_U_T_M,
						finderArgs, list);
				}
				else {
					accountEntry = list.get(0);

					cacheResult(accountEntry);

					if ((accountEntry.getUserId() != userId) ||
							(accountEntry.getType() == null) ||
							!accountEntry.getType().equals(type) ||
							(accountEntry.getMessageAddress() == null) ||
							!accountEntry.getMessageAddress()
											 .equals(messageAddress)) {
						FinderCacheUtil.putResult(FINDER_PATH_FETCH_BY_U_T_M,
							finderArgs, accountEntry);
					}
				}

				return accountEntry;
			}
			catch (Exception e) {
				throw processException(e);
			}
			finally {
				if (result == null) {
					FinderCacheUtil.putResult(FINDER_PATH_FETCH_BY_U_T_M,
						finderArgs, new ArrayList<AccountEntry>());
				}

				closeSession(session);
			}
		}
		else {
			if (result instanceof List<?>) {
				return null;
			}
			else {
				return (AccountEntry)result;
			}
		}
	}

	public List<Object> findWithDynamicQuery(DynamicQuery dynamicQuery)
		throws SystemException {
		Session session = null;

		try {
			session = openSession();

			dynamicQuery.compile(session);

			return dynamicQuery.list();
		}
		catch (Exception e) {
			throw processException(e);
		}
		finally {
			closeSession(session);
		}
	}

	public List<Object> findWithDynamicQuery(DynamicQuery dynamicQuery,
		int start, int end) throws SystemException {
		Session session = null;

		try {
			session = openSession();

			dynamicQuery.setLimit(start, end);

			dynamicQuery.compile(session);

			return dynamicQuery.list();
		}
		catch (Exception e) {
			throw processException(e);
		}
		finally {
			closeSession(session);
		}
	}

	public List<AccountEntry> findAll() throws SystemException {
		return findAll(QueryUtil.ALL_POS, QueryUtil.ALL_POS, null);
	}

	public List<AccountEntry> findAll(int start, int end)
		throws SystemException {
		return findAll(start, end, null);
	}

	public List<AccountEntry> findAll(int start, int end, OrderByComparator obc)
		throws SystemException {
		Object[] finderArgs = new Object[] {
				String.valueOf(start), String.valueOf(end), String.valueOf(obc)
			};

		List<AccountEntry> list = (List<AccountEntry>)FinderCacheUtil.getResult(FINDER_PATH_FIND_ALL,
				finderArgs, this);

		if (list == null) {
			Session session = null;

			try {
				session = openSession();

				StringBuilder query = new StringBuilder();

				query.append(
					"SELECT accountEntry FROM AccountEntry accountEntry ");

				if (obc != null) {
					query.append("ORDER BY ");

					String[] orderByFields = obc.getOrderByFields();

					for (int i = 0; i < orderByFields.length; i++) {
						query.append("accountEntry.");
						query.append(orderByFields[i]);

						if (obc.isAscending()) {
							query.append(" ASC");
						}
						else {
							query.append(" DESC");
						}

						if ((i + 1) < orderByFields.length) {
							query.append(", ");
						}
					}
				}

				else {
					query.append("ORDER BY ");

					query.append("accountEntry.messageAddress ASC");
				}

				Query q = session.createQuery(query.toString());

				if (obc == null) {
					list = (List<AccountEntry>)QueryUtil.list(q, getDialect(),
							start, end, false);

					Collections.sort(list);
				}
				else {
					list = (List<AccountEntry>)QueryUtil.list(q, getDialect(),
							start, end);
				}
			}
			catch (Exception e) {
				throw processException(e);
			}
			finally {
				if (list == null) {
					list = new ArrayList<AccountEntry>();
				}

				cacheResult(list);

				FinderCacheUtil.putResult(FINDER_PATH_FIND_ALL, finderArgs, list);

				closeSession(session);
			}
		}

		return list;
	}

	public void removeByCompanyId(long companyId) throws SystemException {
		for (AccountEntry accountEntry : findByCompanyId(companyId)) {
			remove(accountEntry);
		}
	}

	public void removeByC_T_M(long companyId, String type, String messageAddress)
		throws NoSuchAccountEntryException, SystemException {
		AccountEntry accountEntry = findByC_T_M(companyId, type, messageAddress);

		remove(accountEntry);
	}

	public void removeByUserId(long userId) throws SystemException {
		for (AccountEntry accountEntry : findByUserId(userId)) {
			remove(accountEntry);
		}
	}

	public void removeByU_T(long userId, String type) throws SystemException {
		for (AccountEntry accountEntry : findByU_T(userId, type)) {
			remove(accountEntry);
		}
	}

	public void removeByU_T_I(long userId, String type, boolean initialized)
		throws SystemException {
		for (AccountEntry accountEntry : findByU_T_I(userId, type, initialized)) {
			remove(accountEntry);
		}
	}

	public void removeByU_T_M(long userId, String type, String messageAddress)
		throws NoSuchAccountEntryException, SystemException {
		AccountEntry accountEntry = findByU_T_M(userId, type, messageAddress);

		remove(accountEntry);
	}

	public void removeAll() throws SystemException {
		for (AccountEntry accountEntry : findAll()) {
			remove(accountEntry);
		}
	}

	public int countByCompanyId(long companyId) throws SystemException {
		Object[] finderArgs = new Object[] { new Long(companyId) };

		Long count = (Long)FinderCacheUtil.getResult(FINDER_PATH_COUNT_BY_COMPANYID,
				finderArgs, this);

		if (count == null) {
			Session session = null;

			try {
				session = openSession();

				StringBuilder query = new StringBuilder();

				query.append("SELECT COUNT(accountEntry) ");
				query.append("FROM AccountEntry accountEntry WHERE ");

				query.append("accountEntry.companyId = ?");

				query.append(" ");

				Query q = session.createQuery(query.toString());

				QueryPos qPos = QueryPos.getInstance(q);

				qPos.add(companyId);

				count = (Long)q.uniqueResult();
			}
			catch (Exception e) {
				throw processException(e);
			}
			finally {
				if (count == null) {
					count = Long.valueOf(0);
				}

				FinderCacheUtil.putResult(FINDER_PATH_COUNT_BY_COMPANYID,
					finderArgs, count);

				closeSession(session);
			}
		}

		return count.intValue();
	}

	public int countByC_T_M(long companyId, String type, String messageAddress)
		throws SystemException {
		Object[] finderArgs = new Object[] {
				new Long(companyId),
				
				type,
				
				messageAddress
			};

		Long count = (Long)FinderCacheUtil.getResult(FINDER_PATH_COUNT_BY_C_T_M,
				finderArgs, this);

		if (count == null) {
			Session session = null;

			try {
				session = openSession();

				StringBuilder query = new StringBuilder();

				query.append("SELECT COUNT(accountEntry) ");
				query.append("FROM AccountEntry accountEntry WHERE ");

				query.append("accountEntry.companyId = ?");

				query.append(" AND ");

				if (type == null) {
					query.append("accountEntry.type IS NULL");
				}
				else {
					query.append("accountEntry.type = ?");
				}

				query.append(" AND ");

				if (messageAddress == null) {
					query.append("accountEntry.messageAddress IS NULL");
				}
				else {
					query.append("accountEntry.messageAddress = ?");
				}

				query.append(" ");

				Query q = session.createQuery(query.toString());

				QueryPos qPos = QueryPos.getInstance(q);

				qPos.add(companyId);

				if (type != null) {
					qPos.add(type);
				}

				if (messageAddress != null) {
					qPos.add(messageAddress);
				}

				count = (Long)q.uniqueResult();
			}
			catch (Exception e) {
				throw processException(e);
			}
			finally {
				if (count == null) {
					count = Long.valueOf(0);
				}

				FinderCacheUtil.putResult(FINDER_PATH_COUNT_BY_C_T_M,
					finderArgs, count);

				closeSession(session);
			}
		}

		return count.intValue();
	}

	public int countByUserId(long userId) throws SystemException {
		Object[] finderArgs = new Object[] { new Long(userId) };

		Long count = (Long)FinderCacheUtil.getResult(FINDER_PATH_COUNT_BY_USERID,
				finderArgs, this);

		if (count == null) {
			Session session = null;

			try {
				session = openSession();

				StringBuilder query = new StringBuilder();

				query.append("SELECT COUNT(accountEntry) ");
				query.append("FROM AccountEntry accountEntry WHERE ");

				query.append("accountEntry.userId = ?");

				query.append(" ");

				Query q = session.createQuery(query.toString());

				QueryPos qPos = QueryPos.getInstance(q);

				qPos.add(userId);

				count = (Long)q.uniqueResult();
			}
			catch (Exception e) {
				throw processException(e);
			}
			finally {
				if (count == null) {
					count = Long.valueOf(0);
				}

				FinderCacheUtil.putResult(FINDER_PATH_COUNT_BY_USERID,
					finderArgs, count);

				closeSession(session);
			}
		}

		return count.intValue();
	}

	public int countByU_T(long userId, String type) throws SystemException {
		Object[] finderArgs = new Object[] { new Long(userId), type };

		Long count = (Long)FinderCacheUtil.getResult(FINDER_PATH_COUNT_BY_U_T,
				finderArgs, this);

		if (count == null) {
			Session session = null;

			try {
				session = openSession();

				StringBuilder query = new StringBuilder();

				query.append("SELECT COUNT(accountEntry) ");
				query.append("FROM AccountEntry accountEntry WHERE ");

				query.append("accountEntry.userId = ?");

				query.append(" AND ");

				if (type == null) {
					query.append("accountEntry.type IS NULL");
				}
				else {
					query.append("accountEntry.type = ?");
				}

				query.append(" ");

				Query q = session.createQuery(query.toString());

				QueryPos qPos = QueryPos.getInstance(q);

				qPos.add(userId);

				if (type != null) {
					qPos.add(type);
				}

				count = (Long)q.uniqueResult();
			}
			catch (Exception e) {
				throw processException(e);
			}
			finally {
				if (count == null) {
					count = Long.valueOf(0);
				}

				FinderCacheUtil.putResult(FINDER_PATH_COUNT_BY_U_T, finderArgs,
					count);

				closeSession(session);
			}
		}

		return count.intValue();
	}

	public int countByU_T_I(long userId, String type, boolean initialized)
		throws SystemException {
		Object[] finderArgs = new Object[] {
				new Long(userId),
				
				type, Boolean.valueOf(initialized)
			};

		Long count = (Long)FinderCacheUtil.getResult(FINDER_PATH_COUNT_BY_U_T_I,
				finderArgs, this);

		if (count == null) {
			Session session = null;

			try {
				session = openSession();

				StringBuilder query = new StringBuilder();

				query.append("SELECT COUNT(accountEntry) ");
				query.append("FROM AccountEntry accountEntry WHERE ");

				query.append("accountEntry.userId = ?");

				query.append(" AND ");

				if (type == null) {
					query.append("accountEntry.type IS NULL");
				}
				else {
					query.append("accountEntry.type = ?");
				}

				query.append(" AND ");

				query.append("accountEntry.initialized = ?");

				query.append(" ");

				Query q = session.createQuery(query.toString());

				QueryPos qPos = QueryPos.getInstance(q);

				qPos.add(userId);

				if (type != null) {
					qPos.add(type);
				}

				qPos.add(initialized);

				count = (Long)q.uniqueResult();
			}
			catch (Exception e) {
				throw processException(e);
			}
			finally {
				if (count == null) {
					count = Long.valueOf(0);
				}

				FinderCacheUtil.putResult(FINDER_PATH_COUNT_BY_U_T_I,
					finderArgs, count);

				closeSession(session);
			}
		}

		return count.intValue();
	}

	public int countByU_T_M(long userId, String type, String messageAddress)
		throws SystemException {
		Object[] finderArgs = new Object[] {
				new Long(userId),
				
				type,
				
				messageAddress
			};

		Long count = (Long)FinderCacheUtil.getResult(FINDER_PATH_COUNT_BY_U_T_M,
				finderArgs, this);

		if (count == null) {
			Session session = null;

			try {
				session = openSession();

				StringBuilder query = new StringBuilder();

				query.append("SELECT COUNT(accountEntry) ");
				query.append("FROM AccountEntry accountEntry WHERE ");

				query.append("accountEntry.userId = ?");

				query.append(" AND ");

				if (type == null) {
					query.append("accountEntry.type IS NULL");
				}
				else {
					query.append("accountEntry.type = ?");
				}

				query.append(" AND ");

				if (messageAddress == null) {
					query.append("accountEntry.messageAddress IS NULL");
				}
				else {
					query.append("accountEntry.messageAddress = ?");
				}

				query.append(" ");

				Query q = session.createQuery(query.toString());

				QueryPos qPos = QueryPos.getInstance(q);

				qPos.add(userId);

				if (type != null) {
					qPos.add(type);
				}

				if (messageAddress != null) {
					qPos.add(messageAddress);
				}

				count = (Long)q.uniqueResult();
			}
			catch (Exception e) {
				throw processException(e);
			}
			finally {
				if (count == null) {
					count = Long.valueOf(0);
				}

				FinderCacheUtil.putResult(FINDER_PATH_COUNT_BY_U_T_M,
					finderArgs, count);

				closeSession(session);
			}
		}

		return count.intValue();
	}

	public int countAll() throws SystemException {
		Object[] finderArgs = new Object[0];

		Long count = (Long)FinderCacheUtil.getResult(FINDER_PATH_COUNT_ALL,
				finderArgs, this);

		if (count == null) {
			Session session = null;

			try {
				session = openSession();

				Query q = session.createQuery(
						"SELECT COUNT(accountEntry) FROM AccountEntry accountEntry");

				count = (Long)q.uniqueResult();
			}
			catch (Exception e) {
				throw processException(e);
			}
			finally {
				if (count == null) {
					count = Long.valueOf(0);
				}

				FinderCacheUtil.putResult(FINDER_PATH_COUNT_ALL, finderArgs,
					count);

				closeSession(session);
			}
		}

		return count.intValue();
	}

	public void afterPropertiesSet() {
		String[] listenerClassNames = StringUtil.split(GetterUtil.getString(
					com.liferay.util.service.ServiceProps.get(
						"value.object.listener.com.liferay.mail.model.AccountEntry")));

		if (listenerClassNames.length > 0) {
			try {
				List<ModelListener<AccountEntry>> listenersList = new ArrayList<ModelListener<AccountEntry>>();

				for (String listenerClassName : listenerClassNames) {
					listenersList.add((ModelListener<AccountEntry>)Class.forName(
							listenerClassName).newInstance());
				}

				listeners = listenersList.toArray(new ModelListener[listenersList.size()]);
			}
			catch (Exception e) {
				_log.error(e);
			}
		}
	}

	@BeanReference(name = "com.liferay.mail.service.persistence.AccountEntryPersistence.impl")
	protected com.liferay.mail.service.persistence.AccountEntryPersistence accountEntryPersistence;
	@BeanReference(name = "com.liferay.mail.service.persistence.AccountSettingPersistence.impl")
	protected com.liferay.mail.service.persistence.AccountSettingPersistence accountSettingPersistence;
	@BeanReference(name = "com.liferay.mail.service.persistence.AttachmentPersistence.impl")
	protected com.liferay.mail.service.persistence.AttachmentPersistence attachmentPersistence;
	@BeanReference(name = "com.liferay.mail.service.persistence.ContactPersistence.impl")
	protected com.liferay.mail.service.persistence.ContactPersistence contactPersistence;
	@BeanReference(name = "com.liferay.mail.service.persistence.FolderPersistence.impl")
	protected com.liferay.mail.service.persistence.FolderPersistence folderPersistence;
	@BeanReference(name = "com.liferay.mail.service.persistence.MessagePersistence.impl")
	protected com.liferay.mail.service.persistence.MessagePersistence messagePersistence;
	private static Log _log = LogFactoryUtil.getLog(AccountEntryPersistenceImpl.class);
}