/**
 * Copyright (c) 2000-2009 Liferay, Inc. All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package com.liferay.mail.service.persistence;

import com.liferay.mail.NoSuchAccountSettingException;
import com.liferay.mail.model.AccountSetting;
import com.liferay.mail.model.impl.AccountSettingImpl;
import com.liferay.mail.model.impl.AccountSettingModelImpl;

import com.liferay.portal.SystemException;
import com.liferay.portal.kernel.annotation.BeanReference;
import com.liferay.portal.kernel.cache.CacheRegistry;
import com.liferay.portal.kernel.dao.orm.DynamicQuery;
import com.liferay.portal.kernel.dao.orm.EntityCacheUtil;
import com.liferay.portal.kernel.dao.orm.FinderCacheUtil;
import com.liferay.portal.kernel.dao.orm.FinderPath;
import com.liferay.portal.kernel.dao.orm.Query;
import com.liferay.portal.kernel.dao.orm.QueryPos;
import com.liferay.portal.kernel.dao.orm.QueryUtil;
import com.liferay.portal.kernel.dao.orm.Session;
import com.liferay.portal.kernel.log.Log;
import com.liferay.portal.kernel.log.LogFactoryUtil;
import com.liferay.portal.kernel.util.GetterUtil;
import com.liferay.portal.kernel.util.OrderByComparator;
import com.liferay.portal.kernel.util.StringPool;
import com.liferay.portal.kernel.util.StringUtil;
import com.liferay.portal.model.ModelListener;
import com.liferay.portal.service.persistence.BatchSessionUtil;
import com.liferay.portal.service.persistence.impl.BasePersistenceImpl;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

/**
 * <a href="AccountSettingPersistenceImpl.java.html"><b><i>View Source</i></b>
 * </a>
 *
 * @author Scott Lee
 */
public class AccountSettingPersistenceImpl extends BasePersistenceImpl
	implements AccountSettingPersistence {
	public static final String FINDER_CLASS_NAME_ENTITY = AccountSettingImpl.class.getName();
	public static final String FINDER_CLASS_NAME_LIST = FINDER_CLASS_NAME_ENTITY +
		".List";
	public static final FinderPath FINDER_PATH_FETCH_BY_ACCOUNTENTRYID = new FinderPath(AccountSettingModelImpl.ENTITY_CACHE_ENABLED,
			AccountSettingModelImpl.FINDER_CACHE_ENABLED,
			FINDER_CLASS_NAME_ENTITY, "fetchByAccountEntryId",
			new String[] { Long.class.getName() });
	public static final FinderPath FINDER_PATH_COUNT_BY_ACCOUNTENTRYID = new FinderPath(AccountSettingModelImpl.ENTITY_CACHE_ENABLED,
			AccountSettingModelImpl.FINDER_CACHE_ENABLED,
			FINDER_CLASS_NAME_LIST, "countByAccountEntryId",
			new String[] { Long.class.getName() });
	public static final FinderPath FINDER_PATH_FIND_BY_COMPANYID = new FinderPath(AccountSettingModelImpl.ENTITY_CACHE_ENABLED,
			AccountSettingModelImpl.FINDER_CACHE_ENABLED,
			FINDER_CLASS_NAME_LIST, "findByCompanyId",
			new String[] { Long.class.getName() });
	public static final FinderPath FINDER_PATH_FIND_BY_OBC_COMPANYID = new FinderPath(AccountSettingModelImpl.ENTITY_CACHE_ENABLED,
			AccountSettingModelImpl.FINDER_CACHE_ENABLED,
			FINDER_CLASS_NAME_LIST, "findByCompanyId",
			new String[] {
				Long.class.getName(),
				
			"java.lang.Integer", "java.lang.Integer",
				"com.liferay.portal.kernel.util.OrderByComparator"
			});
	public static final FinderPath FINDER_PATH_COUNT_BY_COMPANYID = new FinderPath(AccountSettingModelImpl.ENTITY_CACHE_ENABLED,
			AccountSettingModelImpl.FINDER_CACHE_ENABLED,
			FINDER_CLASS_NAME_LIST, "countByCompanyId",
			new String[] { Long.class.getName() });
	public static final FinderPath FINDER_PATH_FIND_BY_C_T = new FinderPath(AccountSettingModelImpl.ENTITY_CACHE_ENABLED,
			AccountSettingModelImpl.FINDER_CACHE_ENABLED,
			FINDER_CLASS_NAME_LIST, "findByC_T",
			new String[] { Long.class.getName(), String.class.getName() });
	public static final FinderPath FINDER_PATH_FIND_BY_OBC_C_T = new FinderPath(AccountSettingModelImpl.ENTITY_CACHE_ENABLED,
			AccountSettingModelImpl.FINDER_CACHE_ENABLED,
			FINDER_CLASS_NAME_LIST, "findByC_T",
			new String[] {
				Long.class.getName(), String.class.getName(),
				
			"java.lang.Integer", "java.lang.Integer",
				"com.liferay.portal.kernel.util.OrderByComparator"
			});
	public static final FinderPath FINDER_PATH_COUNT_BY_C_T = new FinderPath(AccountSettingModelImpl.ENTITY_CACHE_ENABLED,
			AccountSettingModelImpl.FINDER_CACHE_ENABLED,
			FINDER_CLASS_NAME_LIST, "countByC_T",
			new String[] { Long.class.getName(), String.class.getName() });
	public static final FinderPath FINDER_PATH_FIND_ALL = new FinderPath(AccountSettingModelImpl.ENTITY_CACHE_ENABLED,
			AccountSettingModelImpl.FINDER_CACHE_ENABLED,
			FINDER_CLASS_NAME_LIST, "findAll", new String[0]);
	public static final FinderPath FINDER_PATH_COUNT_ALL = new FinderPath(AccountSettingModelImpl.ENTITY_CACHE_ENABLED,
			AccountSettingModelImpl.FINDER_CACHE_ENABLED,
			FINDER_CLASS_NAME_LIST, "countAll", new String[0]);

	public void cacheResult(AccountSetting accountSetting) {
		EntityCacheUtil.putResult(AccountSettingModelImpl.ENTITY_CACHE_ENABLED,
			AccountSettingImpl.class, accountSetting.getPrimaryKey(),
			accountSetting);

		FinderCacheUtil.putResult(FINDER_PATH_FETCH_BY_ACCOUNTENTRYID,
			new Object[] { new Long(accountSetting.getAccountEntryId()) },
			accountSetting);
	}

	public void cacheResult(List<AccountSetting> accountSettings) {
		for (AccountSetting accountSetting : accountSettings) {
			if (EntityCacheUtil.getResult(
						AccountSettingModelImpl.ENTITY_CACHE_ENABLED,
						AccountSettingImpl.class,
						accountSetting.getPrimaryKey(), this) == null) {
				cacheResult(accountSetting);
			}
		}
	}

	public void clearCache() {
		CacheRegistry.clear(AccountSettingImpl.class.getName());
		EntityCacheUtil.clearCache(AccountSettingImpl.class.getName());
		FinderCacheUtil.clearCache(FINDER_CLASS_NAME_ENTITY);
		FinderCacheUtil.clearCache(FINDER_CLASS_NAME_LIST);
	}

	public AccountSetting create(long accountSettingId) {
		AccountSetting accountSetting = new AccountSettingImpl();

		accountSetting.setNew(true);
		accountSetting.setPrimaryKey(accountSettingId);

		return accountSetting;
	}

	public AccountSetting remove(long accountSettingId)
		throws NoSuchAccountSettingException, SystemException {
		Session session = null;

		try {
			session = openSession();

			AccountSetting accountSetting = (AccountSetting)session.get(AccountSettingImpl.class,
					new Long(accountSettingId));

			if (accountSetting == null) {
				if (_log.isWarnEnabled()) {
					_log.warn("No AccountSetting exists with the primary key " +
						accountSettingId);
				}

				throw new NoSuchAccountSettingException(
					"No AccountSetting exists with the primary key " +
					accountSettingId);
			}

			return remove(accountSetting);
		}
		catch (NoSuchAccountSettingException nsee) {
			throw nsee;
		}
		catch (Exception e) {
			throw processException(e);
		}
		finally {
			closeSession(session);
		}
	}

	public AccountSetting remove(AccountSetting accountSetting)
		throws SystemException {
		for (ModelListener<AccountSetting> listener : listeners) {
			listener.onBeforeRemove(accountSetting);
		}

		accountSetting = removeImpl(accountSetting);

		for (ModelListener<AccountSetting> listener : listeners) {
			listener.onAfterRemove(accountSetting);
		}

		return accountSetting;
	}

	protected AccountSetting removeImpl(AccountSetting accountSetting)
		throws SystemException {
		Session session = null;

		try {
			session = openSession();

			if (accountSetting.isCachedModel() || BatchSessionUtil.isEnabled()) {
				Object staleObject = session.get(AccountSettingImpl.class,
						accountSetting.getPrimaryKeyObj());

				if (staleObject != null) {
					session.evict(staleObject);
				}
			}

			session.delete(accountSetting);

			session.flush();
		}
		catch (Exception e) {
			throw processException(e);
		}
		finally {
			closeSession(session);
		}

		FinderCacheUtil.clearCache(FINDER_CLASS_NAME_LIST);

		AccountSettingModelImpl accountSettingModelImpl = (AccountSettingModelImpl)accountSetting;

		FinderCacheUtil.removeResult(FINDER_PATH_FETCH_BY_ACCOUNTENTRYID,
			new Object[] {
				new Long(accountSettingModelImpl.getOriginalAccountEntryId())
			});

		EntityCacheUtil.removeResult(AccountSettingModelImpl.ENTITY_CACHE_ENABLED,
			AccountSettingImpl.class, accountSetting.getPrimaryKey());

		return accountSetting;
	}

	public AccountSetting update(AccountSetting accountSetting)
		throws SystemException {
		if (_log.isWarnEnabled()) {
			_log.warn(
				"Using the deprecated update(AccountSetting accountSetting) method. Use update(AccountSetting accountSetting, boolean merge) instead.");
		}

		return update(accountSetting, false);
	}

	public AccountSetting update(AccountSetting accountSetting, boolean merge)
		throws SystemException {
		boolean isNew = accountSetting.isNew();

		for (ModelListener<AccountSetting> listener : listeners) {
			if (isNew) {
				listener.onBeforeCreate(accountSetting);
			}
			else {
				listener.onBeforeUpdate(accountSetting);
			}
		}

		accountSetting = updateImpl(accountSetting, merge);

		for (ModelListener<AccountSetting> listener : listeners) {
			if (isNew) {
				listener.onAfterCreate(accountSetting);
			}
			else {
				listener.onAfterUpdate(accountSetting);
			}
		}

		return accountSetting;
	}

	public AccountSetting updateImpl(
		com.liferay.mail.model.AccountSetting accountSetting, boolean merge)
		throws SystemException {
		boolean isNew = accountSetting.isNew();

		AccountSettingModelImpl accountSettingModelImpl = (AccountSettingModelImpl)accountSetting;

		Session session = null;

		try {
			session = openSession();

			BatchSessionUtil.update(session, accountSetting, merge);

			accountSetting.setNew(false);
		}
		catch (Exception e) {
			throw processException(e);
		}
		finally {
			closeSession(session);
		}

		FinderCacheUtil.clearCache(FINDER_CLASS_NAME_LIST);

		EntityCacheUtil.putResult(AccountSettingModelImpl.ENTITY_CACHE_ENABLED,
			AccountSettingImpl.class, accountSetting.getPrimaryKey(),
			accountSetting);

		if (!isNew &&
				(accountSetting.getAccountEntryId() != accountSettingModelImpl.getOriginalAccountEntryId())) {
			FinderCacheUtil.removeResult(FINDER_PATH_FETCH_BY_ACCOUNTENTRYID,
				new Object[] {
					new Long(accountSettingModelImpl.getOriginalAccountEntryId())
				});
		}

		if (isNew ||
				(accountSetting.getAccountEntryId() != accountSettingModelImpl.getOriginalAccountEntryId())) {
			FinderCacheUtil.putResult(FINDER_PATH_FETCH_BY_ACCOUNTENTRYID,
				new Object[] { new Long(accountSetting.getAccountEntryId()) },
				accountSetting);
		}

		return accountSetting;
	}

	public AccountSetting findByPrimaryKey(long accountSettingId)
		throws NoSuchAccountSettingException, SystemException {
		AccountSetting accountSetting = fetchByPrimaryKey(accountSettingId);

		if (accountSetting == null) {
			if (_log.isWarnEnabled()) {
				_log.warn("No AccountSetting exists with the primary key " +
					accountSettingId);
			}

			throw new NoSuchAccountSettingException(
				"No AccountSetting exists with the primary key " +
				accountSettingId);
		}

		return accountSetting;
	}

	public AccountSetting fetchByPrimaryKey(long accountSettingId)
		throws SystemException {
		AccountSetting accountSetting = (AccountSetting)EntityCacheUtil.getResult(AccountSettingModelImpl.ENTITY_CACHE_ENABLED,
				AccountSettingImpl.class, accountSettingId, this);

		if (accountSetting == null) {
			Session session = null;

			try {
				session = openSession();

				accountSetting = (AccountSetting)session.get(AccountSettingImpl.class,
						new Long(accountSettingId));
			}
			catch (Exception e) {
				throw processException(e);
			}
			finally {
				if (accountSetting != null) {
					cacheResult(accountSetting);
				}

				closeSession(session);
			}
		}

		return accountSetting;
	}

	public AccountSetting findByAccountEntryId(long accountEntryId)
		throws NoSuchAccountSettingException, SystemException {
		AccountSetting accountSetting = fetchByAccountEntryId(accountEntryId);

		if (accountSetting == null) {
			StringBuilder msg = new StringBuilder();

			msg.append("No AccountSetting exists with the key {");

			msg.append("accountEntryId=" + accountEntryId);

			msg.append(StringPool.CLOSE_CURLY_BRACE);

			if (_log.isWarnEnabled()) {
				_log.warn(msg.toString());
			}

			throw new NoSuchAccountSettingException(msg.toString());
		}

		return accountSetting;
	}

	public AccountSetting fetchByAccountEntryId(long accountEntryId)
		throws SystemException {
		return fetchByAccountEntryId(accountEntryId, true);
	}

	public AccountSetting fetchByAccountEntryId(long accountEntryId,
		boolean retrieveFromCache) throws SystemException {
		Object[] finderArgs = new Object[] { new Long(accountEntryId) };

		Object result = null;

		if (retrieveFromCache) {
			result = FinderCacheUtil.getResult(FINDER_PATH_FETCH_BY_ACCOUNTENTRYID,
					finderArgs, this);
		}

		if (result == null) {
			Session session = null;

			try {
				session = openSession();

				StringBuilder query = new StringBuilder();

				query.append(
					"SELECT accountSetting FROM AccountSetting accountSetting WHERE ");

				query.append("accountSetting.accountEntryId = ?");

				query.append(" ");

				Query q = session.createQuery(query.toString());

				QueryPos qPos = QueryPos.getInstance(q);

				qPos.add(accountEntryId);

				List<AccountSetting> list = q.list();

				result = list;

				AccountSetting accountSetting = null;

				if (list.isEmpty()) {
					FinderCacheUtil.putResult(FINDER_PATH_FETCH_BY_ACCOUNTENTRYID,
						finderArgs, list);
				}
				else {
					accountSetting = list.get(0);

					cacheResult(accountSetting);

					if ((accountSetting.getAccountEntryId() != accountEntryId)) {
						FinderCacheUtil.putResult(FINDER_PATH_FETCH_BY_ACCOUNTENTRYID,
							finderArgs, accountSetting);
					}
				}

				return accountSetting;
			}
			catch (Exception e) {
				throw processException(e);
			}
			finally {
				if (result == null) {
					FinderCacheUtil.putResult(FINDER_PATH_FETCH_BY_ACCOUNTENTRYID,
						finderArgs, new ArrayList<AccountSetting>());
				}

				closeSession(session);
			}
		}
		else {
			if (result instanceof List<?>) {
				return null;
			}
			else {
				return (AccountSetting)result;
			}
		}
	}

	public List<AccountSetting> findByCompanyId(long companyId)
		throws SystemException {
		Object[] finderArgs = new Object[] { new Long(companyId) };

		List<AccountSetting> list = (List<AccountSetting>)FinderCacheUtil.getResult(FINDER_PATH_FIND_BY_COMPANYID,
				finderArgs, this);

		if (list == null) {
			Session session = null;

			try {
				session = openSession();

				StringBuilder query = new StringBuilder();

				query.append(
					"SELECT accountSetting FROM AccountSetting accountSetting WHERE ");

				query.append("accountSetting.companyId = ?");

				query.append(" ");

				Query q = session.createQuery(query.toString());

				QueryPos qPos = QueryPos.getInstance(q);

				qPos.add(companyId);

				list = q.list();
			}
			catch (Exception e) {
				throw processException(e);
			}
			finally {
				if (list == null) {
					list = new ArrayList<AccountSetting>();
				}

				cacheResult(list);

				FinderCacheUtil.putResult(FINDER_PATH_FIND_BY_COMPANYID,
					finderArgs, list);

				closeSession(session);
			}
		}

		return list;
	}

	public List<AccountSetting> findByCompanyId(long companyId, int start,
		int end) throws SystemException {
		return findByCompanyId(companyId, start, end, null);
	}

	public List<AccountSetting> findByCompanyId(long companyId, int start,
		int end, OrderByComparator obc) throws SystemException {
		Object[] finderArgs = new Object[] {
				new Long(companyId),
				
				String.valueOf(start), String.valueOf(end), String.valueOf(obc)
			};

		List<AccountSetting> list = (List<AccountSetting>)FinderCacheUtil.getResult(FINDER_PATH_FIND_BY_OBC_COMPANYID,
				finderArgs, this);

		if (list == null) {
			Session session = null;

			try {
				session = openSession();

				StringBuilder query = new StringBuilder();

				query.append(
					"SELECT accountSetting FROM AccountSetting accountSetting WHERE ");

				query.append("accountSetting.companyId = ?");

				query.append(" ");

				if (obc != null) {
					query.append("ORDER BY ");

					String[] orderByFields = obc.getOrderByFields();

					for (int i = 0; i < orderByFields.length; i++) {
						query.append("accountSetting.");
						query.append(orderByFields[i]);

						if (obc.isAscending()) {
							query.append(" ASC");
						}
						else {
							query.append(" DESC");
						}

						if ((i + 1) < orderByFields.length) {
							query.append(", ");
						}
					}
				}

				Query q = session.createQuery(query.toString());

				QueryPos qPos = QueryPos.getInstance(q);

				qPos.add(companyId);

				list = (List<AccountSetting>)QueryUtil.list(q, getDialect(),
						start, end);
			}
			catch (Exception e) {
				throw processException(e);
			}
			finally {
				if (list == null) {
					list = new ArrayList<AccountSetting>();
				}

				cacheResult(list);

				FinderCacheUtil.putResult(FINDER_PATH_FIND_BY_OBC_COMPANYID,
					finderArgs, list);

				closeSession(session);
			}
		}

		return list;
	}

	public AccountSetting findByCompanyId_First(long companyId,
		OrderByComparator obc)
		throws NoSuchAccountSettingException, SystemException {
		List<AccountSetting> list = findByCompanyId(companyId, 0, 1, obc);

		if (list.isEmpty()) {
			StringBuilder msg = new StringBuilder();

			msg.append("No AccountSetting exists with the key {");

			msg.append("companyId=" + companyId);

			msg.append(StringPool.CLOSE_CURLY_BRACE);

			throw new NoSuchAccountSettingException(msg.toString());
		}
		else {
			return list.get(0);
		}
	}

	public AccountSetting findByCompanyId_Last(long companyId,
		OrderByComparator obc)
		throws NoSuchAccountSettingException, SystemException {
		int count = countByCompanyId(companyId);

		List<AccountSetting> list = findByCompanyId(companyId, count - 1,
				count, obc);

		if (list.isEmpty()) {
			StringBuilder msg = new StringBuilder();

			msg.append("No AccountSetting exists with the key {");

			msg.append("companyId=" + companyId);

			msg.append(StringPool.CLOSE_CURLY_BRACE);

			throw new NoSuchAccountSettingException(msg.toString());
		}
		else {
			return list.get(0);
		}
	}

	public AccountSetting[] findByCompanyId_PrevAndNext(long accountSettingId,
		long companyId, OrderByComparator obc)
		throws NoSuchAccountSettingException, SystemException {
		AccountSetting accountSetting = findByPrimaryKey(accountSettingId);

		int count = countByCompanyId(companyId);

		Session session = null;

		try {
			session = openSession();

			StringBuilder query = new StringBuilder();

			query.append(
				"SELECT accountSetting FROM AccountSetting accountSetting WHERE ");

			query.append("accountSetting.companyId = ?");

			query.append(" ");

			if (obc != null) {
				query.append("ORDER BY ");

				String[] orderByFields = obc.getOrderByFields();

				for (int i = 0; i < orderByFields.length; i++) {
					query.append("accountSetting.");
					query.append(orderByFields[i]);

					if (obc.isAscending()) {
						query.append(" ASC");
					}
					else {
						query.append(" DESC");
					}

					if ((i + 1) < orderByFields.length) {
						query.append(", ");
					}
				}
			}

			Query q = session.createQuery(query.toString());

			QueryPos qPos = QueryPos.getInstance(q);

			qPos.add(companyId);

			Object[] objArray = QueryUtil.getPrevAndNext(q, count, obc,
					accountSetting);

			AccountSetting[] array = new AccountSettingImpl[3];

			array[0] = (AccountSetting)objArray[0];
			array[1] = (AccountSetting)objArray[1];
			array[2] = (AccountSetting)objArray[2];

			return array;
		}
		catch (Exception e) {
			throw processException(e);
		}
		finally {
			closeSession(session);
		}
	}

	public List<AccountSetting> findByC_T(long companyId, String type)
		throws SystemException {
		Object[] finderArgs = new Object[] { new Long(companyId), type };

		List<AccountSetting> list = (List<AccountSetting>)FinderCacheUtil.getResult(FINDER_PATH_FIND_BY_C_T,
				finderArgs, this);

		if (list == null) {
			Session session = null;

			try {
				session = openSession();

				StringBuilder query = new StringBuilder();

				query.append(
					"SELECT accountSetting FROM AccountSetting accountSetting WHERE ");

				query.append("accountSetting.companyId = ?");

				query.append(" AND ");

				if (type == null) {
					query.append("accountSetting.type IS NULL");
				}
				else {
					query.append("accountSetting.type = ?");
				}

				query.append(" ");

				Query q = session.createQuery(query.toString());

				QueryPos qPos = QueryPos.getInstance(q);

				qPos.add(companyId);

				if (type != null) {
					qPos.add(type);
				}

				list = q.list();
			}
			catch (Exception e) {
				throw processException(e);
			}
			finally {
				if (list == null) {
					list = new ArrayList<AccountSetting>();
				}

				cacheResult(list);

				FinderCacheUtil.putResult(FINDER_PATH_FIND_BY_C_T, finderArgs,
					list);

				closeSession(session);
			}
		}

		return list;
	}

	public List<AccountSetting> findByC_T(long companyId, String type,
		int start, int end) throws SystemException {
		return findByC_T(companyId, type, start, end, null);
	}

	public List<AccountSetting> findByC_T(long companyId, String type,
		int start, int end, OrderByComparator obc) throws SystemException {
		Object[] finderArgs = new Object[] {
				new Long(companyId),
				
				type,
				
				String.valueOf(start), String.valueOf(end), String.valueOf(obc)
			};

		List<AccountSetting> list = (List<AccountSetting>)FinderCacheUtil.getResult(FINDER_PATH_FIND_BY_OBC_C_T,
				finderArgs, this);

		if (list == null) {
			Session session = null;

			try {
				session = openSession();

				StringBuilder query = new StringBuilder();

				query.append(
					"SELECT accountSetting FROM AccountSetting accountSetting WHERE ");

				query.append("accountSetting.companyId = ?");

				query.append(" AND ");

				if (type == null) {
					query.append("accountSetting.type IS NULL");
				}
				else {
					query.append("accountSetting.type = ?");
				}

				query.append(" ");

				if (obc != null) {
					query.append("ORDER BY ");

					String[] orderByFields = obc.getOrderByFields();

					for (int i = 0; i < orderByFields.length; i++) {
						query.append("accountSetting.");
						query.append(orderByFields[i]);

						if (obc.isAscending()) {
							query.append(" ASC");
						}
						else {
							query.append(" DESC");
						}

						if ((i + 1) < orderByFields.length) {
							query.append(", ");
						}
					}
				}

				Query q = session.createQuery(query.toString());

				QueryPos qPos = QueryPos.getInstance(q);

				qPos.add(companyId);

				if (type != null) {
					qPos.add(type);
				}

				list = (List<AccountSetting>)QueryUtil.list(q, getDialect(),
						start, end);
			}
			catch (Exception e) {
				throw processException(e);
			}
			finally {
				if (list == null) {
					list = new ArrayList<AccountSetting>();
				}

				cacheResult(list);

				FinderCacheUtil.putResult(FINDER_PATH_FIND_BY_OBC_C_T,
					finderArgs, list);

				closeSession(session);
			}
		}

		return list;
	}

	public AccountSetting findByC_T_First(long companyId, String type,
		OrderByComparator obc)
		throws NoSuchAccountSettingException, SystemException {
		List<AccountSetting> list = findByC_T(companyId, type, 0, 1, obc);

		if (list.isEmpty()) {
			StringBuilder msg = new StringBuilder();

			msg.append("No AccountSetting exists with the key {");

			msg.append("companyId=" + companyId);

			msg.append(", ");
			msg.append("type=" + type);

			msg.append(StringPool.CLOSE_CURLY_BRACE);

			throw new NoSuchAccountSettingException(msg.toString());
		}
		else {
			return list.get(0);
		}
	}

	public AccountSetting findByC_T_Last(long companyId, String type,
		OrderByComparator obc)
		throws NoSuchAccountSettingException, SystemException {
		int count = countByC_T(companyId, type);

		List<AccountSetting> list = findByC_T(companyId, type, count - 1,
				count, obc);

		if (list.isEmpty()) {
			StringBuilder msg = new StringBuilder();

			msg.append("No AccountSetting exists with the key {");

			msg.append("companyId=" + companyId);

			msg.append(", ");
			msg.append("type=" + type);

			msg.append(StringPool.CLOSE_CURLY_BRACE);

			throw new NoSuchAccountSettingException(msg.toString());
		}
		else {
			return list.get(0);
		}
	}

	public AccountSetting[] findByC_T_PrevAndNext(long accountSettingId,
		long companyId, String type, OrderByComparator obc)
		throws NoSuchAccountSettingException, SystemException {
		AccountSetting accountSetting = findByPrimaryKey(accountSettingId);

		int count = countByC_T(companyId, type);

		Session session = null;

		try {
			session = openSession();

			StringBuilder query = new StringBuilder();

			query.append(
				"SELECT accountSetting FROM AccountSetting accountSetting WHERE ");

			query.append("accountSetting.companyId = ?");

			query.append(" AND ");

			if (type == null) {
				query.append("accountSetting.type IS NULL");
			}
			else {
				query.append("accountSetting.type = ?");
			}

			query.append(" ");

			if (obc != null) {
				query.append("ORDER BY ");

				String[] orderByFields = obc.getOrderByFields();

				for (int i = 0; i < orderByFields.length; i++) {
					query.append("accountSetting.");
					query.append(orderByFields[i]);

					if (obc.isAscending()) {
						query.append(" ASC");
					}
					else {
						query.append(" DESC");
					}

					if ((i + 1) < orderByFields.length) {
						query.append(", ");
					}
				}
			}

			Query q = session.createQuery(query.toString());

			QueryPos qPos = QueryPos.getInstance(q);

			qPos.add(companyId);

			if (type != null) {
				qPos.add(type);
			}

			Object[] objArray = QueryUtil.getPrevAndNext(q, count, obc,
					accountSetting);

			AccountSetting[] array = new AccountSettingImpl[3];

			array[0] = (AccountSetting)objArray[0];
			array[1] = (AccountSetting)objArray[1];
			array[2] = (AccountSetting)objArray[2];

			return array;
		}
		catch (Exception e) {
			throw processException(e);
		}
		finally {
			closeSession(session);
		}
	}

	public List<Object> findWithDynamicQuery(DynamicQuery dynamicQuery)
		throws SystemException {
		Session session = null;

		try {
			session = openSession();

			dynamicQuery.compile(session);

			return dynamicQuery.list();
		}
		catch (Exception e) {
			throw processException(e);
		}
		finally {
			closeSession(session);
		}
	}

	public List<Object> findWithDynamicQuery(DynamicQuery dynamicQuery,
		int start, int end) throws SystemException {
		Session session = null;

		try {
			session = openSession();

			dynamicQuery.setLimit(start, end);

			dynamicQuery.compile(session);

			return dynamicQuery.list();
		}
		catch (Exception e) {
			throw processException(e);
		}
		finally {
			closeSession(session);
		}
	}

	public List<AccountSetting> findAll() throws SystemException {
		return findAll(QueryUtil.ALL_POS, QueryUtil.ALL_POS, null);
	}

	public List<AccountSetting> findAll(int start, int end)
		throws SystemException {
		return findAll(start, end, null);
	}

	public List<AccountSetting> findAll(int start, int end,
		OrderByComparator obc) throws SystemException {
		Object[] finderArgs = new Object[] {
				String.valueOf(start), String.valueOf(end), String.valueOf(obc)
			};

		List<AccountSetting> list = (List<AccountSetting>)FinderCacheUtil.getResult(FINDER_PATH_FIND_ALL,
				finderArgs, this);

		if (list == null) {
			Session session = null;

			try {
				session = openSession();

				StringBuilder query = new StringBuilder();

				query.append(
					"SELECT accountSetting FROM AccountSetting accountSetting ");

				if (obc != null) {
					query.append("ORDER BY ");

					String[] orderByFields = obc.getOrderByFields();

					for (int i = 0; i < orderByFields.length; i++) {
						query.append("accountSetting.");
						query.append(orderByFields[i]);

						if (obc.isAscending()) {
							query.append(" ASC");
						}
						else {
							query.append(" DESC");
						}

						if ((i + 1) < orderByFields.length) {
							query.append(", ");
						}
					}
				}

				Query q = session.createQuery(query.toString());

				if (obc == null) {
					list = (List<AccountSetting>)QueryUtil.list(q,
							getDialect(), start, end, false);

					Collections.sort(list);
				}
				else {
					list = (List<AccountSetting>)QueryUtil.list(q,
							getDialect(), start, end);
				}
			}
			catch (Exception e) {
				throw processException(e);
			}
			finally {
				if (list == null) {
					list = new ArrayList<AccountSetting>();
				}

				cacheResult(list);

				FinderCacheUtil.putResult(FINDER_PATH_FIND_ALL, finderArgs, list);

				closeSession(session);
			}
		}

		return list;
	}

	public void removeByAccountEntryId(long accountEntryId)
		throws NoSuchAccountSettingException, SystemException {
		AccountSetting accountSetting = findByAccountEntryId(accountEntryId);

		remove(accountSetting);
	}

	public void removeByCompanyId(long companyId) throws SystemException {
		for (AccountSetting accountSetting : findByCompanyId(companyId)) {
			remove(accountSetting);
		}
	}

	public void removeByC_T(long companyId, String type)
		throws SystemException {
		for (AccountSetting accountSetting : findByC_T(companyId, type)) {
			remove(accountSetting);
		}
	}

	public void removeAll() throws SystemException {
		for (AccountSetting accountSetting : findAll()) {
			remove(accountSetting);
		}
	}

	public int countByAccountEntryId(long accountEntryId)
		throws SystemException {
		Object[] finderArgs = new Object[] { new Long(accountEntryId) };

		Long count = (Long)FinderCacheUtil.getResult(FINDER_PATH_COUNT_BY_ACCOUNTENTRYID,
				finderArgs, this);

		if (count == null) {
			Session session = null;

			try {
				session = openSession();

				StringBuilder query = new StringBuilder();

				query.append("SELECT COUNT(accountSetting) ");
				query.append("FROM AccountSetting accountSetting WHERE ");

				query.append("accountSetting.accountEntryId = ?");

				query.append(" ");

				Query q = session.createQuery(query.toString());

				QueryPos qPos = QueryPos.getInstance(q);

				qPos.add(accountEntryId);

				count = (Long)q.uniqueResult();
			}
			catch (Exception e) {
				throw processException(e);
			}
			finally {
				if (count == null) {
					count = Long.valueOf(0);
				}

				FinderCacheUtil.putResult(FINDER_PATH_COUNT_BY_ACCOUNTENTRYID,
					finderArgs, count);

				closeSession(session);
			}
		}

		return count.intValue();
	}

	public int countByCompanyId(long companyId) throws SystemException {
		Object[] finderArgs = new Object[] { new Long(companyId) };

		Long count = (Long)FinderCacheUtil.getResult(FINDER_PATH_COUNT_BY_COMPANYID,
				finderArgs, this);

		if (count == null) {
			Session session = null;

			try {
				session = openSession();

				StringBuilder query = new StringBuilder();

				query.append("SELECT COUNT(accountSetting) ");
				query.append("FROM AccountSetting accountSetting WHERE ");

				query.append("accountSetting.companyId = ?");

				query.append(" ");

				Query q = session.createQuery(query.toString());

				QueryPos qPos = QueryPos.getInstance(q);

				qPos.add(companyId);

				count = (Long)q.uniqueResult();
			}
			catch (Exception e) {
				throw processException(e);
			}
			finally {
				if (count == null) {
					count = Long.valueOf(0);
				}

				FinderCacheUtil.putResult(FINDER_PATH_COUNT_BY_COMPANYID,
					finderArgs, count);

				closeSession(session);
			}
		}

		return count.intValue();
	}

	public int countByC_T(long companyId, String type)
		throws SystemException {
		Object[] finderArgs = new Object[] { new Long(companyId), type };

		Long count = (Long)FinderCacheUtil.getResult(FINDER_PATH_COUNT_BY_C_T,
				finderArgs, this);

		if (count == null) {
			Session session = null;

			try {
				session = openSession();

				StringBuilder query = new StringBuilder();

				query.append("SELECT COUNT(accountSetting) ");
				query.append("FROM AccountSetting accountSetting WHERE ");

				query.append("accountSetting.companyId = ?");

				query.append(" AND ");

				if (type == null) {
					query.append("accountSetting.type IS NULL");
				}
				else {
					query.append("accountSetting.type = ?");
				}

				query.append(" ");

				Query q = session.createQuery(query.toString());

				QueryPos qPos = QueryPos.getInstance(q);

				qPos.add(companyId);

				if (type != null) {
					qPos.add(type);
				}

				count = (Long)q.uniqueResult();
			}
			catch (Exception e) {
				throw processException(e);
			}
			finally {
				if (count == null) {
					count = Long.valueOf(0);
				}

				FinderCacheUtil.putResult(FINDER_PATH_COUNT_BY_C_T, finderArgs,
					count);

				closeSession(session);
			}
		}

		return count.intValue();
	}

	public int countAll() throws SystemException {
		Object[] finderArgs = new Object[0];

		Long count = (Long)FinderCacheUtil.getResult(FINDER_PATH_COUNT_ALL,
				finderArgs, this);

		if (count == null) {
			Session session = null;

			try {
				session = openSession();

				Query q = session.createQuery(
						"SELECT COUNT(accountSetting) FROM AccountSetting accountSetting");

				count = (Long)q.uniqueResult();
			}
			catch (Exception e) {
				throw processException(e);
			}
			finally {
				if (count == null) {
					count = Long.valueOf(0);
				}

				FinderCacheUtil.putResult(FINDER_PATH_COUNT_ALL, finderArgs,
					count);

				closeSession(session);
			}
		}

		return count.intValue();
	}

	public void afterPropertiesSet() {
		String[] listenerClassNames = StringUtil.split(GetterUtil.getString(
					com.liferay.util.service.ServiceProps.get(
						"value.object.listener.com.liferay.mail.model.AccountSetting")));

		if (listenerClassNames.length > 0) {
			try {
				List<ModelListener<AccountSetting>> listenersList = new ArrayList<ModelListener<AccountSetting>>();

				for (String listenerClassName : listenerClassNames) {
					listenersList.add((ModelListener<AccountSetting>)Class.forName(
							listenerClassName).newInstance());
				}

				listeners = listenersList.toArray(new ModelListener[listenersList.size()]);
			}
			catch (Exception e) {
				_log.error(e);
			}
		}
	}

	@BeanReference(name = "com.liferay.mail.service.persistence.AccountEntryPersistence.impl")
	protected com.liferay.mail.service.persistence.AccountEntryPersistence accountEntryPersistence;
	@BeanReference(name = "com.liferay.mail.service.persistence.AccountSettingPersistence.impl")
	protected com.liferay.mail.service.persistence.AccountSettingPersistence accountSettingPersistence;
	@BeanReference(name = "com.liferay.mail.service.persistence.AttachmentPersistence.impl")
	protected com.liferay.mail.service.persistence.AttachmentPersistence attachmentPersistence;
	@BeanReference(name = "com.liferay.mail.service.persistence.ContactPersistence.impl")
	protected com.liferay.mail.service.persistence.ContactPersistence contactPersistence;
	@BeanReference(name = "com.liferay.mail.service.persistence.FolderPersistence.impl")
	protected com.liferay.mail.service.persistence.FolderPersistence folderPersistence;
	@BeanReference(name = "com.liferay.mail.service.persistence.MessagePersistence.impl")
	protected com.liferay.mail.service.persistence.MessagePersistence messagePersistence;
	private static Log _log = LogFactoryUtil.getLog(AccountSettingPersistenceImpl.class);
}