/**
 * Copyright (c) 2000-2009 Liferay, Inc. All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package com.liferay.mail.service.persistence;

import com.liferay.mail.NoSuchAttachmentException;
import com.liferay.mail.model.Attachment;
import com.liferay.mail.model.impl.AttachmentImpl;
import com.liferay.mail.model.impl.AttachmentModelImpl;

import com.liferay.portal.SystemException;
import com.liferay.portal.kernel.annotation.BeanReference;
import com.liferay.portal.kernel.cache.CacheRegistry;
import com.liferay.portal.kernel.dao.orm.DynamicQuery;
import com.liferay.portal.kernel.dao.orm.EntityCacheUtil;
import com.liferay.portal.kernel.dao.orm.FinderCacheUtil;
import com.liferay.portal.kernel.dao.orm.FinderPath;
import com.liferay.portal.kernel.dao.orm.Query;
import com.liferay.portal.kernel.dao.orm.QueryPos;
import com.liferay.portal.kernel.dao.orm.QueryUtil;
import com.liferay.portal.kernel.dao.orm.Session;
import com.liferay.portal.kernel.log.Log;
import com.liferay.portal.kernel.log.LogFactoryUtil;
import com.liferay.portal.kernel.util.GetterUtil;
import com.liferay.portal.kernel.util.OrderByComparator;
import com.liferay.portal.kernel.util.StringPool;
import com.liferay.portal.kernel.util.StringUtil;
import com.liferay.portal.model.ModelListener;
import com.liferay.portal.service.persistence.BatchSessionUtil;
import com.liferay.portal.service.persistence.impl.BasePersistenceImpl;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

/**
 * <a href="AttachmentPersistenceImpl.java.html"><b><i>View Source</i></b></a>
 *
 * @author Scott Lee
 */
public class AttachmentPersistenceImpl extends BasePersistenceImpl
	implements AttachmentPersistence {
	public static final String FINDER_CLASS_NAME_ENTITY = AttachmentImpl.class.getName();
	public static final String FINDER_CLASS_NAME_LIST = FINDER_CLASS_NAME_ENTITY +
		".List";
	public static final FinderPath FINDER_PATH_FIND_BY_MESSAGEID = new FinderPath(AttachmentModelImpl.ENTITY_CACHE_ENABLED,
			AttachmentModelImpl.FINDER_CACHE_ENABLED, FINDER_CLASS_NAME_LIST,
			"findByMessageId", new String[] { Long.class.getName() });
	public static final FinderPath FINDER_PATH_FIND_BY_OBC_MESSAGEID = new FinderPath(AttachmentModelImpl.ENTITY_CACHE_ENABLED,
			AttachmentModelImpl.FINDER_CACHE_ENABLED, FINDER_CLASS_NAME_LIST,
			"findByMessageId",
			new String[] {
				Long.class.getName(),
				
			"java.lang.Integer", "java.lang.Integer",
				"com.liferay.portal.kernel.util.OrderByComparator"
			});
	public static final FinderPath FINDER_PATH_COUNT_BY_MESSAGEID = new FinderPath(AttachmentModelImpl.ENTITY_CACHE_ENABLED,
			AttachmentModelImpl.FINDER_CACHE_ENABLED, FINDER_CLASS_NAME_LIST,
			"countByMessageId", new String[] { Long.class.getName() });
	public static final FinderPath FINDER_PATH_FIND_ALL = new FinderPath(AttachmentModelImpl.ENTITY_CACHE_ENABLED,
			AttachmentModelImpl.FINDER_CACHE_ENABLED, FINDER_CLASS_NAME_LIST,
			"findAll", new String[0]);
	public static final FinderPath FINDER_PATH_COUNT_ALL = new FinderPath(AttachmentModelImpl.ENTITY_CACHE_ENABLED,
			AttachmentModelImpl.FINDER_CACHE_ENABLED, FINDER_CLASS_NAME_LIST,
			"countAll", new String[0]);

	public void cacheResult(Attachment attachment) {
		EntityCacheUtil.putResult(AttachmentModelImpl.ENTITY_CACHE_ENABLED,
			AttachmentImpl.class, attachment.getPrimaryKey(), attachment);
	}

	public void cacheResult(List<Attachment> attachments) {
		for (Attachment attachment : attachments) {
			if (EntityCacheUtil.getResult(
						AttachmentModelImpl.ENTITY_CACHE_ENABLED,
						AttachmentImpl.class, attachment.getPrimaryKey(), this) == null) {
				cacheResult(attachment);
			}
		}
	}

	public void clearCache() {
		CacheRegistry.clear(AttachmentImpl.class.getName());
		EntityCacheUtil.clearCache(AttachmentImpl.class.getName());
		FinderCacheUtil.clearCache(FINDER_CLASS_NAME_ENTITY);
		FinderCacheUtil.clearCache(FINDER_CLASS_NAME_LIST);
	}

	public Attachment create(long attachmentId) {
		Attachment attachment = new AttachmentImpl();

		attachment.setNew(true);
		attachment.setPrimaryKey(attachmentId);

		return attachment;
	}

	public Attachment remove(long attachmentId)
		throws NoSuchAttachmentException, SystemException {
		Session session = null;

		try {
			session = openSession();

			Attachment attachment = (Attachment)session.get(AttachmentImpl.class,
					new Long(attachmentId));

			if (attachment == null) {
				if (_log.isWarnEnabled()) {
					_log.warn("No Attachment exists with the primary key " +
						attachmentId);
				}

				throw new NoSuchAttachmentException(
					"No Attachment exists with the primary key " +
					attachmentId);
			}

			return remove(attachment);
		}
		catch (NoSuchAttachmentException nsee) {
			throw nsee;
		}
		catch (Exception e) {
			throw processException(e);
		}
		finally {
			closeSession(session);
		}
	}

	public Attachment remove(Attachment attachment) throws SystemException {
		for (ModelListener<Attachment> listener : listeners) {
			listener.onBeforeRemove(attachment);
		}

		attachment = removeImpl(attachment);

		for (ModelListener<Attachment> listener : listeners) {
			listener.onAfterRemove(attachment);
		}

		return attachment;
	}

	protected Attachment removeImpl(Attachment attachment)
		throws SystemException {
		Session session = null;

		try {
			session = openSession();

			if (attachment.isCachedModel() || BatchSessionUtil.isEnabled()) {
				Object staleObject = session.get(AttachmentImpl.class,
						attachment.getPrimaryKeyObj());

				if (staleObject != null) {
					session.evict(staleObject);
				}
			}

			session.delete(attachment);

			session.flush();
		}
		catch (Exception e) {
			throw processException(e);
		}
		finally {
			closeSession(session);
		}

		FinderCacheUtil.clearCache(FINDER_CLASS_NAME_LIST);

		EntityCacheUtil.removeResult(AttachmentModelImpl.ENTITY_CACHE_ENABLED,
			AttachmentImpl.class, attachment.getPrimaryKey());

		return attachment;
	}

	public Attachment update(Attachment attachment) throws SystemException {
		if (_log.isWarnEnabled()) {
			_log.warn(
				"Using the deprecated update(Attachment attachment) method. Use update(Attachment attachment, boolean merge) instead.");
		}

		return update(attachment, false);
	}

	public Attachment update(Attachment attachment, boolean merge)
		throws SystemException {
		boolean isNew = attachment.isNew();

		for (ModelListener<Attachment> listener : listeners) {
			if (isNew) {
				listener.onBeforeCreate(attachment);
			}
			else {
				listener.onBeforeUpdate(attachment);
			}
		}

		attachment = updateImpl(attachment, merge);

		for (ModelListener<Attachment> listener : listeners) {
			if (isNew) {
				listener.onAfterCreate(attachment);
			}
			else {
				listener.onAfterUpdate(attachment);
			}
		}

		return attachment;
	}

	public Attachment updateImpl(com.liferay.mail.model.Attachment attachment,
		boolean merge) throws SystemException {
		Session session = null;

		try {
			session = openSession();

			BatchSessionUtil.update(session, attachment, merge);

			attachment.setNew(false);
		}
		catch (Exception e) {
			throw processException(e);
		}
		finally {
			closeSession(session);
		}

		FinderCacheUtil.clearCache(FINDER_CLASS_NAME_LIST);

		EntityCacheUtil.putResult(AttachmentModelImpl.ENTITY_CACHE_ENABLED,
			AttachmentImpl.class, attachment.getPrimaryKey(), attachment);

		return attachment;
	}

	public Attachment findByPrimaryKey(long attachmentId)
		throws NoSuchAttachmentException, SystemException {
		Attachment attachment = fetchByPrimaryKey(attachmentId);

		if (attachment == null) {
			if (_log.isWarnEnabled()) {
				_log.warn("No Attachment exists with the primary key " +
					attachmentId);
			}

			throw new NoSuchAttachmentException(
				"No Attachment exists with the primary key " + attachmentId);
		}

		return attachment;
	}

	public Attachment fetchByPrimaryKey(long attachmentId)
		throws SystemException {
		Attachment attachment = (Attachment)EntityCacheUtil.getResult(AttachmentModelImpl.ENTITY_CACHE_ENABLED,
				AttachmentImpl.class, attachmentId, this);

		if (attachment == null) {
			Session session = null;

			try {
				session = openSession();

				attachment = (Attachment)session.get(AttachmentImpl.class,
						new Long(attachmentId));
			}
			catch (Exception e) {
				throw processException(e);
			}
			finally {
				if (attachment != null) {
					cacheResult(attachment);
				}

				closeSession(session);
			}
		}

		return attachment;
	}

	public List<Attachment> findByMessageId(long messageId)
		throws SystemException {
		Object[] finderArgs = new Object[] { new Long(messageId) };

		List<Attachment> list = (List<Attachment>)FinderCacheUtil.getResult(FINDER_PATH_FIND_BY_MESSAGEID,
				finderArgs, this);

		if (list == null) {
			Session session = null;

			try {
				session = openSession();

				StringBuilder query = new StringBuilder();

				query.append(
					"SELECT attachment FROM Attachment attachment WHERE ");

				query.append("attachment.messageId = ?");

				query.append(" ");

				Query q = session.createQuery(query.toString());

				QueryPos qPos = QueryPos.getInstance(q);

				qPos.add(messageId);

				list = q.list();
			}
			catch (Exception e) {
				throw processException(e);
			}
			finally {
				if (list == null) {
					list = new ArrayList<Attachment>();
				}

				cacheResult(list);

				FinderCacheUtil.putResult(FINDER_PATH_FIND_BY_MESSAGEID,
					finderArgs, list);

				closeSession(session);
			}
		}

		return list;
	}

	public List<Attachment> findByMessageId(long messageId, int start, int end)
		throws SystemException {
		return findByMessageId(messageId, start, end, null);
	}

	public List<Attachment> findByMessageId(long messageId, int start, int end,
		OrderByComparator obc) throws SystemException {
		Object[] finderArgs = new Object[] {
				new Long(messageId),
				
				String.valueOf(start), String.valueOf(end), String.valueOf(obc)
			};

		List<Attachment> list = (List<Attachment>)FinderCacheUtil.getResult(FINDER_PATH_FIND_BY_OBC_MESSAGEID,
				finderArgs, this);

		if (list == null) {
			Session session = null;

			try {
				session = openSession();

				StringBuilder query = new StringBuilder();

				query.append(
					"SELECT attachment FROM Attachment attachment WHERE ");

				query.append("attachment.messageId = ?");

				query.append(" ");

				if (obc != null) {
					query.append("ORDER BY ");

					String[] orderByFields = obc.getOrderByFields();

					for (int i = 0; i < orderByFields.length; i++) {
						query.append("attachment.");
						query.append(orderByFields[i]);

						if (obc.isAscending()) {
							query.append(" ASC");
						}
						else {
							query.append(" DESC");
						}

						if ((i + 1) < orderByFields.length) {
							query.append(", ");
						}
					}
				}

				Query q = session.createQuery(query.toString());

				QueryPos qPos = QueryPos.getInstance(q);

				qPos.add(messageId);

				list = (List<Attachment>)QueryUtil.list(q, getDialect(), start,
						end);
			}
			catch (Exception e) {
				throw processException(e);
			}
			finally {
				if (list == null) {
					list = new ArrayList<Attachment>();
				}

				cacheResult(list);

				FinderCacheUtil.putResult(FINDER_PATH_FIND_BY_OBC_MESSAGEID,
					finderArgs, list);

				closeSession(session);
			}
		}

		return list;
	}

	public Attachment findByMessageId_First(long messageId,
		OrderByComparator obc)
		throws NoSuchAttachmentException, SystemException {
		List<Attachment> list = findByMessageId(messageId, 0, 1, obc);

		if (list.isEmpty()) {
			StringBuilder msg = new StringBuilder();

			msg.append("No Attachment exists with the key {");

			msg.append("messageId=" + messageId);

			msg.append(StringPool.CLOSE_CURLY_BRACE);

			throw new NoSuchAttachmentException(msg.toString());
		}
		else {
			return list.get(0);
		}
	}

	public Attachment findByMessageId_Last(long messageId, OrderByComparator obc)
		throws NoSuchAttachmentException, SystemException {
		int count = countByMessageId(messageId);

		List<Attachment> list = findByMessageId(messageId, count - 1, count, obc);

		if (list.isEmpty()) {
			StringBuilder msg = new StringBuilder();

			msg.append("No Attachment exists with the key {");

			msg.append("messageId=" + messageId);

			msg.append(StringPool.CLOSE_CURLY_BRACE);

			throw new NoSuchAttachmentException(msg.toString());
		}
		else {
			return list.get(0);
		}
	}

	public Attachment[] findByMessageId_PrevAndNext(long attachmentId,
		long messageId, OrderByComparator obc)
		throws NoSuchAttachmentException, SystemException {
		Attachment attachment = findByPrimaryKey(attachmentId);

		int count = countByMessageId(messageId);

		Session session = null;

		try {
			session = openSession();

			StringBuilder query = new StringBuilder();

			query.append("SELECT attachment FROM Attachment attachment WHERE ");

			query.append("attachment.messageId = ?");

			query.append(" ");

			if (obc != null) {
				query.append("ORDER BY ");

				String[] orderByFields = obc.getOrderByFields();

				for (int i = 0; i < orderByFields.length; i++) {
					query.append("attachment.");
					query.append(orderByFields[i]);

					if (obc.isAscending()) {
						query.append(" ASC");
					}
					else {
						query.append(" DESC");
					}

					if ((i + 1) < orderByFields.length) {
						query.append(", ");
					}
				}
			}

			Query q = session.createQuery(query.toString());

			QueryPos qPos = QueryPos.getInstance(q);

			qPos.add(messageId);

			Object[] objArray = QueryUtil.getPrevAndNext(q, count, obc,
					attachment);

			Attachment[] array = new AttachmentImpl[3];

			array[0] = (Attachment)objArray[0];
			array[1] = (Attachment)objArray[1];
			array[2] = (Attachment)objArray[2];

			return array;
		}
		catch (Exception e) {
			throw processException(e);
		}
		finally {
			closeSession(session);
		}
	}

	public List<Object> findWithDynamicQuery(DynamicQuery dynamicQuery)
		throws SystemException {
		Session session = null;

		try {
			session = openSession();

			dynamicQuery.compile(session);

			return dynamicQuery.list();
		}
		catch (Exception e) {
			throw processException(e);
		}
		finally {
			closeSession(session);
		}
	}

	public List<Object> findWithDynamicQuery(DynamicQuery dynamicQuery,
		int start, int end) throws SystemException {
		Session session = null;

		try {
			session = openSession();

			dynamicQuery.setLimit(start, end);

			dynamicQuery.compile(session);

			return dynamicQuery.list();
		}
		catch (Exception e) {
			throw processException(e);
		}
		finally {
			closeSession(session);
		}
	}

	public List<Attachment> findAll() throws SystemException {
		return findAll(QueryUtil.ALL_POS, QueryUtil.ALL_POS, null);
	}

	public List<Attachment> findAll(int start, int end)
		throws SystemException {
		return findAll(start, end, null);
	}

	public List<Attachment> findAll(int start, int end, OrderByComparator obc)
		throws SystemException {
		Object[] finderArgs = new Object[] {
				String.valueOf(start), String.valueOf(end), String.valueOf(obc)
			};

		List<Attachment> list = (List<Attachment>)FinderCacheUtil.getResult(FINDER_PATH_FIND_ALL,
				finderArgs, this);

		if (list == null) {
			Session session = null;

			try {
				session = openSession();

				StringBuilder query = new StringBuilder();

				query.append("SELECT attachment FROM Attachment attachment ");

				if (obc != null) {
					query.append("ORDER BY ");

					String[] orderByFields = obc.getOrderByFields();

					for (int i = 0; i < orderByFields.length; i++) {
						query.append("attachment.");
						query.append(orderByFields[i]);

						if (obc.isAscending()) {
							query.append(" ASC");
						}
						else {
							query.append(" DESC");
						}

						if ((i + 1) < orderByFields.length) {
							query.append(", ");
						}
					}
				}

				Query q = session.createQuery(query.toString());

				if (obc == null) {
					list = (List<Attachment>)QueryUtil.list(q, getDialect(),
							start, end, false);

					Collections.sort(list);
				}
				else {
					list = (List<Attachment>)QueryUtil.list(q, getDialect(),
							start, end);
				}
			}
			catch (Exception e) {
				throw processException(e);
			}
			finally {
				if (list == null) {
					list = new ArrayList<Attachment>();
				}

				cacheResult(list);

				FinderCacheUtil.putResult(FINDER_PATH_FIND_ALL, finderArgs, list);

				closeSession(session);
			}
		}

		return list;
	}

	public void removeByMessageId(long messageId) throws SystemException {
		for (Attachment attachment : findByMessageId(messageId)) {
			remove(attachment);
		}
	}

	public void removeAll() throws SystemException {
		for (Attachment attachment : findAll()) {
			remove(attachment);
		}
	}

	public int countByMessageId(long messageId) throws SystemException {
		Object[] finderArgs = new Object[] { new Long(messageId) };

		Long count = (Long)FinderCacheUtil.getResult(FINDER_PATH_COUNT_BY_MESSAGEID,
				finderArgs, this);

		if (count == null) {
			Session session = null;

			try {
				session = openSession();

				StringBuilder query = new StringBuilder();

				query.append("SELECT COUNT(attachment) ");
				query.append("FROM Attachment attachment WHERE ");

				query.append("attachment.messageId = ?");

				query.append(" ");

				Query q = session.createQuery(query.toString());

				QueryPos qPos = QueryPos.getInstance(q);

				qPos.add(messageId);

				count = (Long)q.uniqueResult();
			}
			catch (Exception e) {
				throw processException(e);
			}
			finally {
				if (count == null) {
					count = Long.valueOf(0);
				}

				FinderCacheUtil.putResult(FINDER_PATH_COUNT_BY_MESSAGEID,
					finderArgs, count);

				closeSession(session);
			}
		}

		return count.intValue();
	}

	public int countAll() throws SystemException {
		Object[] finderArgs = new Object[0];

		Long count = (Long)FinderCacheUtil.getResult(FINDER_PATH_COUNT_ALL,
				finderArgs, this);

		if (count == null) {
			Session session = null;

			try {
				session = openSession();

				Query q = session.createQuery(
						"SELECT COUNT(attachment) FROM Attachment attachment");

				count = (Long)q.uniqueResult();
			}
			catch (Exception e) {
				throw processException(e);
			}
			finally {
				if (count == null) {
					count = Long.valueOf(0);
				}

				FinderCacheUtil.putResult(FINDER_PATH_COUNT_ALL, finderArgs,
					count);

				closeSession(session);
			}
		}

		return count.intValue();
	}

	public void afterPropertiesSet() {
		String[] listenerClassNames = StringUtil.split(GetterUtil.getString(
					com.liferay.util.service.ServiceProps.get(
						"value.object.listener.com.liferay.mail.model.Attachment")));

		if (listenerClassNames.length > 0) {
			try {
				List<ModelListener<Attachment>> listenersList = new ArrayList<ModelListener<Attachment>>();

				for (String listenerClassName : listenerClassNames) {
					listenersList.add((ModelListener<Attachment>)Class.forName(
							listenerClassName).newInstance());
				}

				listeners = listenersList.toArray(new ModelListener[listenersList.size()]);
			}
			catch (Exception e) {
				_log.error(e);
			}
		}
	}

	@BeanReference(name = "com.liferay.mail.service.persistence.AccountEntryPersistence.impl")
	protected com.liferay.mail.service.persistence.AccountEntryPersistence accountEntryPersistence;
	@BeanReference(name = "com.liferay.mail.service.persistence.AccountSettingPersistence.impl")
	protected com.liferay.mail.service.persistence.AccountSettingPersistence accountSettingPersistence;
	@BeanReference(name = "com.liferay.mail.service.persistence.AttachmentPersistence.impl")
	protected com.liferay.mail.service.persistence.AttachmentPersistence attachmentPersistence;
	@BeanReference(name = "com.liferay.mail.service.persistence.ContactPersistence.impl")
	protected com.liferay.mail.service.persistence.ContactPersistence contactPersistence;
	@BeanReference(name = "com.liferay.mail.service.persistence.FolderPersistence.impl")
	protected com.liferay.mail.service.persistence.FolderPersistence folderPersistence;
	@BeanReference(name = "com.liferay.mail.service.persistence.MessagePersistence.impl")
	protected com.liferay.mail.service.persistence.MessagePersistence messagePersistence;
	private static Log _log = LogFactoryUtil.getLog(AttachmentPersistenceImpl.class);
}