/**
 * Copyright (c) 2000-2009 Liferay, Inc. All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package com.liferay.mail.service.persistence;

import com.liferay.mail.NoSuchFolderException;
import com.liferay.mail.model.Folder;
import com.liferay.mail.model.impl.FolderImpl;
import com.liferay.mail.model.impl.FolderModelImpl;

import com.liferay.portal.SystemException;
import com.liferay.portal.kernel.annotation.BeanReference;
import com.liferay.portal.kernel.cache.CacheRegistry;
import com.liferay.portal.kernel.dao.orm.DynamicQuery;
import com.liferay.portal.kernel.dao.orm.EntityCacheUtil;
import com.liferay.portal.kernel.dao.orm.FinderCacheUtil;
import com.liferay.portal.kernel.dao.orm.FinderPath;
import com.liferay.portal.kernel.dao.orm.Query;
import com.liferay.portal.kernel.dao.orm.QueryPos;
import com.liferay.portal.kernel.dao.orm.QueryUtil;
import com.liferay.portal.kernel.dao.orm.Session;
import com.liferay.portal.kernel.log.Log;
import com.liferay.portal.kernel.log.LogFactoryUtil;
import com.liferay.portal.kernel.util.GetterUtil;
import com.liferay.portal.kernel.util.OrderByComparator;
import com.liferay.portal.kernel.util.StringPool;
import com.liferay.portal.kernel.util.StringUtil;
import com.liferay.portal.kernel.util.Validator;
import com.liferay.portal.model.ModelListener;
import com.liferay.portal.service.persistence.BatchSessionUtil;
import com.liferay.portal.service.persistence.impl.BasePersistenceImpl;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

/**
 * <a href="FolderPersistenceImpl.java.html"><b><i>View Source</i></b></a>
 *
 * @author Scott Lee
 */
public class FolderPersistenceImpl extends BasePersistenceImpl
	implements FolderPersistence {
	public static final String FINDER_CLASS_NAME_ENTITY = FolderImpl.class.getName();
	public static final String FINDER_CLASS_NAME_LIST = FINDER_CLASS_NAME_ENTITY +
		".List";
	public static final FinderPath FINDER_PATH_FIND_BY_ACCOUNTENTRYID = new FinderPath(FolderModelImpl.ENTITY_CACHE_ENABLED,
			FolderModelImpl.FINDER_CACHE_ENABLED, FINDER_CLASS_NAME_LIST,
			"findByAccountEntryId", new String[] { Long.class.getName() });
	public static final FinderPath FINDER_PATH_FIND_BY_OBC_ACCOUNTENTRYID = new FinderPath(FolderModelImpl.ENTITY_CACHE_ENABLED,
			FolderModelImpl.FINDER_CACHE_ENABLED, FINDER_CLASS_NAME_LIST,
			"findByAccountEntryId",
			new String[] {
				Long.class.getName(),
				
			"java.lang.Integer", "java.lang.Integer",
				"com.liferay.portal.kernel.util.OrderByComparator"
			});
	public static final FinderPath FINDER_PATH_COUNT_BY_ACCOUNTENTRYID = new FinderPath(FolderModelImpl.ENTITY_CACHE_ENABLED,
			FolderModelImpl.FINDER_CACHE_ENABLED, FINDER_CLASS_NAME_LIST,
			"countByAccountEntryId", new String[] { Long.class.getName() });
	public static final FinderPath FINDER_PATH_FETCH_BY_A_N = new FinderPath(FolderModelImpl.ENTITY_CACHE_ENABLED,
			FolderModelImpl.FINDER_CACHE_ENABLED, FINDER_CLASS_NAME_ENTITY,
			"fetchByA_N",
			new String[] { Long.class.getName(), String.class.getName() });
	public static final FinderPath FINDER_PATH_COUNT_BY_A_N = new FinderPath(FolderModelImpl.ENTITY_CACHE_ENABLED,
			FolderModelImpl.FINDER_CACHE_ENABLED, FINDER_CLASS_NAME_LIST,
			"countByA_N",
			new String[] { Long.class.getName(), String.class.getName() });
	public static final FinderPath FINDER_PATH_FIND_ALL = new FinderPath(FolderModelImpl.ENTITY_CACHE_ENABLED,
			FolderModelImpl.FINDER_CACHE_ENABLED, FINDER_CLASS_NAME_LIST,
			"findAll", new String[0]);
	public static final FinderPath FINDER_PATH_COUNT_ALL = new FinderPath(FolderModelImpl.ENTITY_CACHE_ENABLED,
			FolderModelImpl.FINDER_CACHE_ENABLED, FINDER_CLASS_NAME_LIST,
			"countAll", new String[0]);

	public void cacheResult(Folder folder) {
		EntityCacheUtil.putResult(FolderModelImpl.ENTITY_CACHE_ENABLED,
			FolderImpl.class, folder.getPrimaryKey(), folder);

		FinderCacheUtil.putResult(FINDER_PATH_FETCH_BY_A_N,
			new Object[] {
				new Long(folder.getAccountEntryId()),
				
			folder.getFullName()
			}, folder);
	}

	public void cacheResult(List<Folder> folders) {
		for (Folder folder : folders) {
			if (EntityCacheUtil.getResult(
						FolderModelImpl.ENTITY_CACHE_ENABLED, FolderImpl.class,
						folder.getPrimaryKey(), this) == null) {
				cacheResult(folder);
			}
		}
	}

	public void clearCache() {
		CacheRegistry.clear(FolderImpl.class.getName());
		EntityCacheUtil.clearCache(FolderImpl.class.getName());
		FinderCacheUtil.clearCache(FINDER_CLASS_NAME_ENTITY);
		FinderCacheUtil.clearCache(FINDER_CLASS_NAME_LIST);
	}

	public Folder create(long folderId) {
		Folder folder = new FolderImpl();

		folder.setNew(true);
		folder.setPrimaryKey(folderId);

		return folder;
	}

	public Folder remove(long folderId)
		throws NoSuchFolderException, SystemException {
		Session session = null;

		try {
			session = openSession();

			Folder folder = (Folder)session.get(FolderImpl.class,
					new Long(folderId));

			if (folder == null) {
				if (_log.isWarnEnabled()) {
					_log.warn("No Folder exists with the primary key " +
						folderId);
				}

				throw new NoSuchFolderException(
					"No Folder exists with the primary key " + folderId);
			}

			return remove(folder);
		}
		catch (NoSuchFolderException nsee) {
			throw nsee;
		}
		catch (Exception e) {
			throw processException(e);
		}
		finally {
			closeSession(session);
		}
	}

	public Folder remove(Folder folder) throws SystemException {
		for (ModelListener<Folder> listener : listeners) {
			listener.onBeforeRemove(folder);
		}

		folder = removeImpl(folder);

		for (ModelListener<Folder> listener : listeners) {
			listener.onAfterRemove(folder);
		}

		return folder;
	}

	protected Folder removeImpl(Folder folder) throws SystemException {
		Session session = null;

		try {
			session = openSession();

			if (folder.isCachedModel() || BatchSessionUtil.isEnabled()) {
				Object staleObject = session.get(FolderImpl.class,
						folder.getPrimaryKeyObj());

				if (staleObject != null) {
					session.evict(staleObject);
				}
			}

			session.delete(folder);

			session.flush();
		}
		catch (Exception e) {
			throw processException(e);
		}
		finally {
			closeSession(session);
		}

		FinderCacheUtil.clearCache(FINDER_CLASS_NAME_LIST);

		FolderModelImpl folderModelImpl = (FolderModelImpl)folder;

		FinderCacheUtil.removeResult(FINDER_PATH_FETCH_BY_A_N,
			new Object[] {
				new Long(folderModelImpl.getOriginalAccountEntryId()),
				
			folderModelImpl.getOriginalFullName()
			});

		EntityCacheUtil.removeResult(FolderModelImpl.ENTITY_CACHE_ENABLED,
			FolderImpl.class, folder.getPrimaryKey());

		return folder;
	}

	public Folder update(Folder folder) throws SystemException {
		if (_log.isWarnEnabled()) {
			_log.warn(
				"Using the deprecated update(Folder folder) method. Use update(Folder folder, boolean merge) instead.");
		}

		return update(folder, false);
	}

	public Folder update(Folder folder, boolean merge)
		throws SystemException {
		boolean isNew = folder.isNew();

		for (ModelListener<Folder> listener : listeners) {
			if (isNew) {
				listener.onBeforeCreate(folder);
			}
			else {
				listener.onBeforeUpdate(folder);
			}
		}

		folder = updateImpl(folder, merge);

		for (ModelListener<Folder> listener : listeners) {
			if (isNew) {
				listener.onAfterCreate(folder);
			}
			else {
				listener.onAfterUpdate(folder);
			}
		}

		return folder;
	}

	public Folder updateImpl(com.liferay.mail.model.Folder folder, boolean merge)
		throws SystemException {
		boolean isNew = folder.isNew();

		FolderModelImpl folderModelImpl = (FolderModelImpl)folder;

		Session session = null;

		try {
			session = openSession();

			BatchSessionUtil.update(session, folder, merge);

			folder.setNew(false);
		}
		catch (Exception e) {
			throw processException(e);
		}
		finally {
			closeSession(session);
		}

		FinderCacheUtil.clearCache(FINDER_CLASS_NAME_LIST);

		EntityCacheUtil.putResult(FolderModelImpl.ENTITY_CACHE_ENABLED,
			FolderImpl.class, folder.getPrimaryKey(), folder);

		if (!isNew &&
				((folder.getAccountEntryId() != folderModelImpl.getOriginalAccountEntryId()) ||
				!Validator.equals(folder.getFullName(),
					folderModelImpl.getOriginalFullName()))) {
			FinderCacheUtil.removeResult(FINDER_PATH_FETCH_BY_A_N,
				new Object[] {
					new Long(folderModelImpl.getOriginalAccountEntryId()),
					
				folderModelImpl.getOriginalFullName()
				});
		}

		if (isNew ||
				((folder.getAccountEntryId() != folderModelImpl.getOriginalAccountEntryId()) ||
				!Validator.equals(folder.getFullName(),
					folderModelImpl.getOriginalFullName()))) {
			FinderCacheUtil.putResult(FINDER_PATH_FETCH_BY_A_N,
				new Object[] {
					new Long(folder.getAccountEntryId()),
					
				folder.getFullName()
				}, folder);
		}

		return folder;
	}

	public Folder findByPrimaryKey(long folderId)
		throws NoSuchFolderException, SystemException {
		Folder folder = fetchByPrimaryKey(folderId);

		if (folder == null) {
			if (_log.isWarnEnabled()) {
				_log.warn("No Folder exists with the primary key " + folderId);
			}

			throw new NoSuchFolderException(
				"No Folder exists with the primary key " + folderId);
		}

		return folder;
	}

	public Folder fetchByPrimaryKey(long folderId) throws SystemException {
		Folder folder = (Folder)EntityCacheUtil.getResult(FolderModelImpl.ENTITY_CACHE_ENABLED,
				FolderImpl.class, folderId, this);

		if (folder == null) {
			Session session = null;

			try {
				session = openSession();

				folder = (Folder)session.get(FolderImpl.class,
						new Long(folderId));
			}
			catch (Exception e) {
				throw processException(e);
			}
			finally {
				if (folder != null) {
					cacheResult(folder);
				}

				closeSession(session);
			}
		}

		return folder;
	}

	public List<Folder> findByAccountEntryId(long accountEntryId)
		throws SystemException {
		Object[] finderArgs = new Object[] { new Long(accountEntryId) };

		List<Folder> list = (List<Folder>)FinderCacheUtil.getResult(FINDER_PATH_FIND_BY_ACCOUNTENTRYID,
				finderArgs, this);

		if (list == null) {
			Session session = null;

			try {
				session = openSession();

				StringBuilder query = new StringBuilder();

				query.append("SELECT folder FROM Folder folder WHERE ");

				query.append("folder.accountEntryId = ?");

				query.append(" ");

				query.append("ORDER BY ");

				query.append("folder.fullName ASC");

				Query q = session.createQuery(query.toString());

				QueryPos qPos = QueryPos.getInstance(q);

				qPos.add(accountEntryId);

				list = q.list();
			}
			catch (Exception e) {
				throw processException(e);
			}
			finally {
				if (list == null) {
					list = new ArrayList<Folder>();
				}

				cacheResult(list);

				FinderCacheUtil.putResult(FINDER_PATH_FIND_BY_ACCOUNTENTRYID,
					finderArgs, list);

				closeSession(session);
			}
		}

		return list;
	}

	public List<Folder> findByAccountEntryId(long accountEntryId, int start,
		int end) throws SystemException {
		return findByAccountEntryId(accountEntryId, start, end, null);
	}

	public List<Folder> findByAccountEntryId(long accountEntryId, int start,
		int end, OrderByComparator obc) throws SystemException {
		Object[] finderArgs = new Object[] {
				new Long(accountEntryId),
				
				String.valueOf(start), String.valueOf(end), String.valueOf(obc)
			};

		List<Folder> list = (List<Folder>)FinderCacheUtil.getResult(FINDER_PATH_FIND_BY_OBC_ACCOUNTENTRYID,
				finderArgs, this);

		if (list == null) {
			Session session = null;

			try {
				session = openSession();

				StringBuilder query = new StringBuilder();

				query.append("SELECT folder FROM Folder folder WHERE ");

				query.append("folder.accountEntryId = ?");

				query.append(" ");

				if (obc != null) {
					query.append("ORDER BY ");

					String[] orderByFields = obc.getOrderByFields();

					for (int i = 0; i < orderByFields.length; i++) {
						query.append("folder.");
						query.append(orderByFields[i]);

						if (obc.isAscending()) {
							query.append(" ASC");
						}
						else {
							query.append(" DESC");
						}

						if ((i + 1) < orderByFields.length) {
							query.append(", ");
						}
					}
				}

				else {
					query.append("ORDER BY ");

					query.append("folder.fullName ASC");
				}

				Query q = session.createQuery(query.toString());

				QueryPos qPos = QueryPos.getInstance(q);

				qPos.add(accountEntryId);

				list = (List<Folder>)QueryUtil.list(q, getDialect(), start, end);
			}
			catch (Exception e) {
				throw processException(e);
			}
			finally {
				if (list == null) {
					list = new ArrayList<Folder>();
				}

				cacheResult(list);

				FinderCacheUtil.putResult(FINDER_PATH_FIND_BY_OBC_ACCOUNTENTRYID,
					finderArgs, list);

				closeSession(session);
			}
		}

		return list;
	}

	public Folder findByAccountEntryId_First(long accountEntryId,
		OrderByComparator obc) throws NoSuchFolderException, SystemException {
		List<Folder> list = findByAccountEntryId(accountEntryId, 0, 1, obc);

		if (list.isEmpty()) {
			StringBuilder msg = new StringBuilder();

			msg.append("No Folder exists with the key {");

			msg.append("accountEntryId=" + accountEntryId);

			msg.append(StringPool.CLOSE_CURLY_BRACE);

			throw new NoSuchFolderException(msg.toString());
		}
		else {
			return list.get(0);
		}
	}

	public Folder findByAccountEntryId_Last(long accountEntryId,
		OrderByComparator obc) throws NoSuchFolderException, SystemException {
		int count = countByAccountEntryId(accountEntryId);

		List<Folder> list = findByAccountEntryId(accountEntryId, count - 1,
				count, obc);

		if (list.isEmpty()) {
			StringBuilder msg = new StringBuilder();

			msg.append("No Folder exists with the key {");

			msg.append("accountEntryId=" + accountEntryId);

			msg.append(StringPool.CLOSE_CURLY_BRACE);

			throw new NoSuchFolderException(msg.toString());
		}
		else {
			return list.get(0);
		}
	}

	public Folder[] findByAccountEntryId_PrevAndNext(long folderId,
		long accountEntryId, OrderByComparator obc)
		throws NoSuchFolderException, SystemException {
		Folder folder = findByPrimaryKey(folderId);

		int count = countByAccountEntryId(accountEntryId);

		Session session = null;

		try {
			session = openSession();

			StringBuilder query = new StringBuilder();

			query.append("SELECT folder FROM Folder folder WHERE ");

			query.append("folder.accountEntryId = ?");

			query.append(" ");

			if (obc != null) {
				query.append("ORDER BY ");

				String[] orderByFields = obc.getOrderByFields();

				for (int i = 0; i < orderByFields.length; i++) {
					query.append("folder.");
					query.append(orderByFields[i]);

					if (obc.isAscending()) {
						query.append(" ASC");
					}
					else {
						query.append(" DESC");
					}

					if ((i + 1) < orderByFields.length) {
						query.append(", ");
					}
				}
			}

			else {
				query.append("ORDER BY ");

				query.append("folder.fullName ASC");
			}

			Query q = session.createQuery(query.toString());

			QueryPos qPos = QueryPos.getInstance(q);

			qPos.add(accountEntryId);

			Object[] objArray = QueryUtil.getPrevAndNext(q, count, obc, folder);

			Folder[] array = new FolderImpl[3];

			array[0] = (Folder)objArray[0];
			array[1] = (Folder)objArray[1];
			array[2] = (Folder)objArray[2];

			return array;
		}
		catch (Exception e) {
			throw processException(e);
		}
		finally {
			closeSession(session);
		}
	}

	public Folder findByA_N(long accountEntryId, String fullName)
		throws NoSuchFolderException, SystemException {
		Folder folder = fetchByA_N(accountEntryId, fullName);

		if (folder == null) {
			StringBuilder msg = new StringBuilder();

			msg.append("No Folder exists with the key {");

			msg.append("accountEntryId=" + accountEntryId);

			msg.append(", ");
			msg.append("fullName=" + fullName);

			msg.append(StringPool.CLOSE_CURLY_BRACE);

			if (_log.isWarnEnabled()) {
				_log.warn(msg.toString());
			}

			throw new NoSuchFolderException(msg.toString());
		}

		return folder;
	}

	public Folder fetchByA_N(long accountEntryId, String fullName)
		throws SystemException {
		return fetchByA_N(accountEntryId, fullName, true);
	}

	public Folder fetchByA_N(long accountEntryId, String fullName,
		boolean retrieveFromCache) throws SystemException {
		Object[] finderArgs = new Object[] { new Long(accountEntryId), fullName };

		Object result = null;

		if (retrieveFromCache) {
			result = FinderCacheUtil.getResult(FINDER_PATH_FETCH_BY_A_N,
					finderArgs, this);
		}

		if (result == null) {
			Session session = null;

			try {
				session = openSession();

				StringBuilder query = new StringBuilder();

				query.append("SELECT folder FROM Folder folder WHERE ");

				query.append("folder.accountEntryId = ?");

				query.append(" AND ");

				if (fullName == null) {
					query.append("folder.fullName IS NULL");
				}
				else {
					query.append("folder.fullName = ?");
				}

				query.append(" ");

				query.append("ORDER BY ");

				query.append("folder.fullName ASC");

				Query q = session.createQuery(query.toString());

				QueryPos qPos = QueryPos.getInstance(q);

				qPos.add(accountEntryId);

				if (fullName != null) {
					qPos.add(fullName);
				}

				List<Folder> list = q.list();

				result = list;

				Folder folder = null;

				if (list.isEmpty()) {
					FinderCacheUtil.putResult(FINDER_PATH_FETCH_BY_A_N,
						finderArgs, list);
				}
				else {
					folder = list.get(0);

					cacheResult(folder);

					if ((folder.getAccountEntryId() != accountEntryId) ||
							(folder.getFullName() == null) ||
							!folder.getFullName().equals(fullName)) {
						FinderCacheUtil.putResult(FINDER_PATH_FETCH_BY_A_N,
							finderArgs, folder);
					}
				}

				return folder;
			}
			catch (Exception e) {
				throw processException(e);
			}
			finally {
				if (result == null) {
					FinderCacheUtil.putResult(FINDER_PATH_FETCH_BY_A_N,
						finderArgs, new ArrayList<Folder>());
				}

				closeSession(session);
			}
		}
		else {
			if (result instanceof List<?>) {
				return null;
			}
			else {
				return (Folder)result;
			}
		}
	}

	public List<Object> findWithDynamicQuery(DynamicQuery dynamicQuery)
		throws SystemException {
		Session session = null;

		try {
			session = openSession();

			dynamicQuery.compile(session);

			return dynamicQuery.list();
		}
		catch (Exception e) {
			throw processException(e);
		}
		finally {
			closeSession(session);
		}
	}

	public List<Object> findWithDynamicQuery(DynamicQuery dynamicQuery,
		int start, int end) throws SystemException {
		Session session = null;

		try {
			session = openSession();

			dynamicQuery.setLimit(start, end);

			dynamicQuery.compile(session);

			return dynamicQuery.list();
		}
		catch (Exception e) {
			throw processException(e);
		}
		finally {
			closeSession(session);
		}
	}

	public List<Folder> findAll() throws SystemException {
		return findAll(QueryUtil.ALL_POS, QueryUtil.ALL_POS, null);
	}

	public List<Folder> findAll(int start, int end) throws SystemException {
		return findAll(start, end, null);
	}

	public List<Folder> findAll(int start, int end, OrderByComparator obc)
		throws SystemException {
		Object[] finderArgs = new Object[] {
				String.valueOf(start), String.valueOf(end), String.valueOf(obc)
			};

		List<Folder> list = (List<Folder>)FinderCacheUtil.getResult(FINDER_PATH_FIND_ALL,
				finderArgs, this);

		if (list == null) {
			Session session = null;

			try {
				session = openSession();

				StringBuilder query = new StringBuilder();

				query.append("SELECT folder FROM Folder folder ");

				if (obc != null) {
					query.append("ORDER BY ");

					String[] orderByFields = obc.getOrderByFields();

					for (int i = 0; i < orderByFields.length; i++) {
						query.append("folder.");
						query.append(orderByFields[i]);

						if (obc.isAscending()) {
							query.append(" ASC");
						}
						else {
							query.append(" DESC");
						}

						if ((i + 1) < orderByFields.length) {
							query.append(", ");
						}
					}
				}

				else {
					query.append("ORDER BY ");

					query.append("folder.fullName ASC");
				}

				Query q = session.createQuery(query.toString());

				if (obc == null) {
					list = (List<Folder>)QueryUtil.list(q, getDialect(), start,
							end, false);

					Collections.sort(list);
				}
				else {
					list = (List<Folder>)QueryUtil.list(q, getDialect(), start,
							end);
				}
			}
			catch (Exception e) {
				throw processException(e);
			}
			finally {
				if (list == null) {
					list = new ArrayList<Folder>();
				}

				cacheResult(list);

				FinderCacheUtil.putResult(FINDER_PATH_FIND_ALL, finderArgs, list);

				closeSession(session);
			}
		}

		return list;
	}

	public void removeByAccountEntryId(long accountEntryId)
		throws SystemException {
		for (Folder folder : findByAccountEntryId(accountEntryId)) {
			remove(folder);
		}
	}

	public void removeByA_N(long accountEntryId, String fullName)
		throws NoSuchFolderException, SystemException {
		Folder folder = findByA_N(accountEntryId, fullName);

		remove(folder);
	}

	public void removeAll() throws SystemException {
		for (Folder folder : findAll()) {
			remove(folder);
		}
	}

	public int countByAccountEntryId(long accountEntryId)
		throws SystemException {
		Object[] finderArgs = new Object[] { new Long(accountEntryId) };

		Long count = (Long)FinderCacheUtil.getResult(FINDER_PATH_COUNT_BY_ACCOUNTENTRYID,
				finderArgs, this);

		if (count == null) {
			Session session = null;

			try {
				session = openSession();

				StringBuilder query = new StringBuilder();

				query.append("SELECT COUNT(folder) ");
				query.append("FROM Folder folder WHERE ");

				query.append("folder.accountEntryId = ?");

				query.append(" ");

				Query q = session.createQuery(query.toString());

				QueryPos qPos = QueryPos.getInstance(q);

				qPos.add(accountEntryId);

				count = (Long)q.uniqueResult();
			}
			catch (Exception e) {
				throw processException(e);
			}
			finally {
				if (count == null) {
					count = Long.valueOf(0);
				}

				FinderCacheUtil.putResult(FINDER_PATH_COUNT_BY_ACCOUNTENTRYID,
					finderArgs, count);

				closeSession(session);
			}
		}

		return count.intValue();
	}

	public int countByA_N(long accountEntryId, String fullName)
		throws SystemException {
		Object[] finderArgs = new Object[] { new Long(accountEntryId), fullName };

		Long count = (Long)FinderCacheUtil.getResult(FINDER_PATH_COUNT_BY_A_N,
				finderArgs, this);

		if (count == null) {
			Session session = null;

			try {
				session = openSession();

				StringBuilder query = new StringBuilder();

				query.append("SELECT COUNT(folder) ");
				query.append("FROM Folder folder WHERE ");

				query.append("folder.accountEntryId = ?");

				query.append(" AND ");

				if (fullName == null) {
					query.append("folder.fullName IS NULL");
				}
				else {
					query.append("folder.fullName = ?");
				}

				query.append(" ");

				Query q = session.createQuery(query.toString());

				QueryPos qPos = QueryPos.getInstance(q);

				qPos.add(accountEntryId);

				if (fullName != null) {
					qPos.add(fullName);
				}

				count = (Long)q.uniqueResult();
			}
			catch (Exception e) {
				throw processException(e);
			}
			finally {
				if (count == null) {
					count = Long.valueOf(0);
				}

				FinderCacheUtil.putResult(FINDER_PATH_COUNT_BY_A_N, finderArgs,
					count);

				closeSession(session);
			}
		}

		return count.intValue();
	}

	public int countAll() throws SystemException {
		Object[] finderArgs = new Object[0];

		Long count = (Long)FinderCacheUtil.getResult(FINDER_PATH_COUNT_ALL,
				finderArgs, this);

		if (count == null) {
			Session session = null;

			try {
				session = openSession();

				Query q = session.createQuery(
						"SELECT COUNT(folder) FROM Folder folder");

				count = (Long)q.uniqueResult();
			}
			catch (Exception e) {
				throw processException(e);
			}
			finally {
				if (count == null) {
					count = Long.valueOf(0);
				}

				FinderCacheUtil.putResult(FINDER_PATH_COUNT_ALL, finderArgs,
					count);

				closeSession(session);
			}
		}

		return count.intValue();
	}

	public void afterPropertiesSet() {
		String[] listenerClassNames = StringUtil.split(GetterUtil.getString(
					com.liferay.util.service.ServiceProps.get(
						"value.object.listener.com.liferay.mail.model.Folder")));

		if (listenerClassNames.length > 0) {
			try {
				List<ModelListener<Folder>> listenersList = new ArrayList<ModelListener<Folder>>();

				for (String listenerClassName : listenerClassNames) {
					listenersList.add((ModelListener<Folder>)Class.forName(
							listenerClassName).newInstance());
				}

				listeners = listenersList.toArray(new ModelListener[listenersList.size()]);
			}
			catch (Exception e) {
				_log.error(e);
			}
		}
	}

	@BeanReference(name = "com.liferay.mail.service.persistence.AccountEntryPersistence.impl")
	protected com.liferay.mail.service.persistence.AccountEntryPersistence accountEntryPersistence;
	@BeanReference(name = "com.liferay.mail.service.persistence.AccountSettingPersistence.impl")
	protected com.liferay.mail.service.persistence.AccountSettingPersistence accountSettingPersistence;
	@BeanReference(name = "com.liferay.mail.service.persistence.AttachmentPersistence.impl")
	protected com.liferay.mail.service.persistence.AttachmentPersistence attachmentPersistence;
	@BeanReference(name = "com.liferay.mail.service.persistence.ContactPersistence.impl")
	protected com.liferay.mail.service.persistence.ContactPersistence contactPersistence;
	@BeanReference(name = "com.liferay.mail.service.persistence.FolderPersistence.impl")
	protected com.liferay.mail.service.persistence.FolderPersistence folderPersistence;
	@BeanReference(name = "com.liferay.mail.service.persistence.MessagePersistence.impl")
	protected com.liferay.mail.service.persistence.MessagePersistence messagePersistence;
	private static Log _log = LogFactoryUtil.getLog(FolderPersistenceImpl.class);
}