/**
 * Copyright (c) 2000-2009 Liferay, Inc. All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package com.liferay.mail.store;

import com.liferay.mail.MailFile;
import com.liferay.mail.mailbox.PasswordRetriever;
import com.liferay.mail.model.Attachment;
import com.liferay.portal.PortalException;
import com.liferay.portal.SystemException;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;

import java.util.List;

/**
 * <a href="AttachmentStore.java.html"><b><i>View Source</i></b></a>
 *
 * @author Michael C. Han
 * @author Scott Lee
 *
 */
public interface AttachmentStore {

	public void addAttachments(long messageId, List<MailFile> mailFiles)
		throws PortalException, SystemException;

	public void addAttachmentsReferenceToMailServer(
			long messageId, List<MailFile> mailFiles)
		throws PortalException, SystemException;

	public AttachmentStore clone();

	public void copyAttachments(
			long accountEntryId, long messageId, long destinationMessageId)
		throws PortalException, SystemException;

	public void deleteAttachment(long messageId, long attachmentId)
		throws PortalException, SystemException;

	public void deleteAttachments(long messageId)
		throws PortalException, SystemException;

	public InputStream getAttachmentFromMailServer(
			long accountEntryId, long attachmentId,
			PasswordRetriever passwordRetriever)
		throws PortalException, SystemException;

	public InputStream getAttachmentFromStoreAsInputStream(
			long accountEntryId, long attachmentId)
		throws PortalException, SystemException;

	public File getAttachmentFromStoreAsFile(
			long accountEntryId, long attachmentId)
		throws IOException, PortalException, SystemException;

	public List<Attachment> getAttachments(long messageId)
		throws SystemException;

	public boolean hasAttachments(long messageId)
		throws PortalException, SystemException;

}