/**
 * Copyright (c) 2000-2009 Liferay, Inc. All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package com.liferay.mail.store;

import com.liferay.mail.Recipient;
import com.liferay.mail.model.AccountEntry;
import com.liferay.mail.model.Message;
import com.liferay.portal.PortalException;
import com.liferay.portal.SystemException;
import com.liferay.portal.model.User;

import java.util.List;

/**
 * <a href="MessageStore.java.html"><b><i>View Source</i></b></a>
 *
 * @author Michael C. Han
 * @author Scott Lee
 *
 */
public interface MessageStore {

	public MessageStore clone(User user);

	public Message copyMessage(
			long messageId, long destinationFolderId, boolean updateSentDate)
		throws PortalException, SystemException;

	public void deleteMessage(long messageId, boolean useTrashFolder)
		throws PortalException, SystemException;

	public List<Recipient> getAllRecipients(long messageId)
		throws PortalException, SystemException;

	public Message getMessage(long messageId)
		throws PortalException, SystemException;

	public Message getMessage(long folderId, long mailMessageUid)
		throws PortalException, SystemException;

	public Message getMessageNumber(
			long accountEntryId, long folderId, int messageNumber, int offset,
			String orderByField, String orderByType, String keywords)
		throws PortalException, SystemException;

	public int getMessages(
			List<Message> messages, long accountEntryId, long folderId,
			int pageNumber, int messagesPerPage, String orderByField,
			String orderByType, String keywords)
		throws PortalException, SystemException;

	public List<Message> getMessages(long folderId)
		throws PortalException, SystemException;

	public int getMessagesStoredCount(long accountEntryId, long folderId)
		throws PortalException, SystemException;

	public Message getNewestMailMessage(long folderId) throws SystemException;

	public Message getOldestMailMessage(long folderId) throws SystemException;

	public int getPagesStoredCount(
			long accountEntryId, long folderId, int messagesPerPage)
		throws PortalException, SystemException;

	public int getPercentageDownloaded(long folderId)
		throws PortalException, SystemException;

	public int getStartingMessageNumber(
			long accountEntryId, long folderId, int pageNumber,
			int messagesPerPage)
		throws PortalException, SystemException;

	public long getUnreadMessageCount(AccountEntry accountEntry)
		throws PortalException, SystemException;

	public void moveMessage(
			long[] messageIds, long destinationFolderId, boolean updateSentDate)
		throws PortalException, SystemException;

	public Message saveDraft(
			long accountEntryId, Recipient[] to, Recipient[] cc,
			Recipient[] bcc, String subject, String body, long messageId)
		throws PortalException, SystemException;

	public void updateMessageFlag(long messageId, int flag, boolean value)
		throws PortalException, SystemException;

	public void updateMessageFlags(
			long[] messageIds, int flag, boolean value)
		throws PortalException, SystemException;

	public void updateMessageSize(long messageId, long size)
		throws PortalException, SystemException;

}