/**
 * Copyright (c) 2000-2009 Liferay, Inc. All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package com.liferay.mail.util;

import com.liferay.mail.MailboxConstants;
import com.liferay.mail.model.AccountEntry;
import com.liferay.mail.model.AccountSetting;
import com.liferay.mail.service.AccountSettingLocalServiceUtil;
import com.liferay.mail.store.MailStore;
import com.liferay.portal.PortalException;
import com.liferay.portal.SystemException;
import com.liferay.portal.kernel.json.JSONArray;
import com.liferay.portal.kernel.json.JSONFactoryUtil;
import com.liferay.portal.kernel.json.JSONObject;

import java.util.List;

/**
 * <a href="AccountConverterImpl.java.html"><b><i>View Source</i></b></a>
 *
 * @author Scott Lee
 *
 */
public class AccountConverterImpl implements AccountConverter {

	public AccountConverterImpl(MailStore mailStore) {
		_mailStore = mailStore;
	}

	public JSONObject convert(AccountEntry accountEntry)
		throws PortalException, SystemException {

		JSONObject jsonObj = JSONFactoryUtil.createJSONObject();

		jsonObj.put("type", accountEntry.getType());
		jsonObj.put("accountEntryId", accountEntry.getAccountEntryId());
		jsonObj.put("messageAddress", accountEntry.getMessageAddress());
		jsonObj.put("inboxFolderId", accountEntry.getInboxFolderId());
		jsonObj.put("draftFolderId", accountEntry.getDraftFolderId());
		jsonObj.put("sentFolderId", accountEntry.getSentFolderId());
		jsonObj.put("trashFolderId", accountEntry.getTrashFolderId());
		jsonObj.put("username",accountEntry.getUsername());
		jsonObj.put("passwordSaved", accountEntry.isPasswordSaved());
		jsonObj.put("initialized", accountEntry.isInitialized());
		jsonObj.put(
			"newMessages", _mailStore.getUnreadMessageCount(accountEntry));
		jsonObj.put("senderName", accountEntry.getSenderName());
		jsonObj.put("signature", accountEntry.getSignature());
		jsonObj.put("useSignature", accountEntry.isUseSignature());

		if (accountEntry.getType().equals(MailboxConstants.ACCOUNT_TYPE_IMAP)) {
			AccountSetting accountSetting =
				AccountSettingLocalServiceUtil.getAEAccountSetting(
					accountEntry.getAccountEntryId());

			jsonObj.put("mailInHostName", accountSetting.getMailInHostName());
			jsonObj.put("mailInPort", accountSetting.getMailInPort());
			jsonObj.put("mailInSecure", accountSetting.isMailInSecure());
			jsonObj.put("mailOutHostName", accountSetting.getMailOutHostName());
			jsonObj.put("mailOutPort", accountSetting.getMailOutPort());
			jsonObj.put("mailOutSecure", accountSetting.isMailOutSecure());
			jsonObj.put("defaultFolderPrefix",
				accountSetting.getDefaultFolderPrefix());
			jsonObj.put("requiresSynchronization", true);
		}
		else {
			jsonObj.put("requiresSynchronization", false);
		}

		return jsonObj;
	}

	public JSONObject convertList(List<AccountEntry> accountEntries)
		throws PortalException, SystemException {

		JSONObject jsonObj = JSONFactoryUtil.createJSONObject();

		JSONArray jsonArray = JSONFactoryUtil.createJSONArray();

		jsonObj.put("accountEntries", jsonArray);

		for (AccountEntry accountEntry : accountEntries) {
			jsonArray.put(convert(accountEntry));
		}

		return jsonObj;
	}

	private MailStore _mailStore = null;

}