/**
 * Copyright (c) 2000-2009 Liferay, Inc. All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package com.liferay.mail.util;

import com.liferay.mail.MailboxConstants;
import com.liferay.mail.model.AccountEntry;
import com.liferay.mail.model.Folder;
import com.liferay.mail.store.MailStore;
import com.liferay.portal.PortalException;
import com.liferay.portal.SystemException;
import com.liferay.portal.kernel.json.JSONArray;
import com.liferay.portal.kernel.json.JSONFactoryUtil;
import com.liferay.portal.kernel.json.JSONObject;

import java.util.Date;
import java.util.List;

/**
 * <a href="FolderConverterImpl.java.html"><b><i>View Source</i></b></a>
 *
 * @author Scott Lee
 *
 */
public class FolderConverterImpl implements FolderConverter {

	public FolderConverterImpl(MailStore mailStore) {
		_mailStore = mailStore;
	}

	public JSONObject convert(Folder folder)
		throws PortalException, SystemException {

		JSONObject jsonObj = JSONFactoryUtil.createJSONObject();

		AccountEntry accountEntry = _mailStore.getAccountEntry(
			folder.getAccountEntryId());

		int messageCount = 0;
		int storedMessageCount = 0;

		if (accountEntry.getType().equals(
				MailboxConstants.ACCOUNT_TYPE_PRIVATE)) {

			messageCount = _mailStore.getMessagesStoredCount(
				accountEntry.getAccountEntryId(), folder.getFolderId());

			storedMessageCount = messageCount;
		}
		else if (accountEntry.getType().equals(
				MailboxConstants.ACCOUNT_TYPE_IMAP)) {

			messageCount = folder.getRemoteMessageCount();
			storedMessageCount = _mailStore.getMessagesStoredCount(
				accountEntry.getAccountEntryId(), folder.getFolderId());
		}

		jsonObj.put("folderId", folder.getFolderId());
		jsonObj.put("initialized", folder.isInitialized());
		jsonObj.put("lastUpdated", new Date());
		jsonObj.put("messageCount", messageCount);
		jsonObj.put("fullName", folder.getFullName());
		jsonObj.put("name", folder.getName());
		jsonObj.put(
			"newMessageCount", _mailStore.getUnreadMessageCount(folder));
		jsonObj.put(
			"percentageDownloaded",
			_mailStore.getPercentageDownloaded(folder.getFolderId()));
		jsonObj.put("storedMessageCount", storedMessageCount);

		return jsonObj;
	}

	public JSONObject convertList(List<Folder> folders)
		throws PortalException, SystemException {

		JSONObject jsonObj = JSONFactoryUtil.createJSONObject();

		JSONArray jsonArray = JSONFactoryUtil.createJSONArray();

		jsonObj.put("folders", jsonArray);

		for (Folder folder : folders) {
			jsonArray.put(convert(folder));
		}

		return jsonObj;
	}

	private MailStore _mailStore = null;

}