/**
 * Copyright (c) 2000-2009 Liferay, Inc. All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package com.liferay.mail.util;

import com.liferay.portal.kernel.util.DateUtil;
import com.liferay.portal.kernel.util.Validator;

import java.text.DateFormat;
import java.text.NumberFormat;
import java.text.SimpleDateFormat;

import java.util.Calendar;
import java.util.Date;
import java.util.Locale;
import java.util.TimeZone;

/**
 * <a href="FormatUtil.java.html"><b><i>View Source</i></b></a>
 *
 * @author Scott Lee
 *
 */
public class FormatUtil {

	public static String formatCompleteDate(Date date, Locale locale) {
		return formatCompleteDate(date, null, locale);
	}

	public static String formatCompleteDate(
		Date date, TimeZone timeZone, Locale locale) {

		DateFormat df = DateFormat.getDateTimeInstance(
			DateFormat.MEDIUM, DateFormat.SHORT, locale
		);

		if (timeZone != null) {
			df.setTimeZone(timeZone);
		}

		if (Validator.isNotNull(date)) {
			return df.format(date);
		}
		else {
			return DATE_UNAVAILABLE;
		}
	}

	public static String formatDate(Date date, Locale locale) {
		return formatDate(date, null, locale);
	}

	public static String formatDate(
		Date date, TimeZone timeZone, Locale locale) {

		if (Validator.isNotNull(date)) {
			Calendar now = Calendar.getInstance();
			Calendar date2 = Calendar.getInstance();

			date2.setTime(date);

			DateFormat df = null;

			if (now.get(Calendar.YEAR) == date2.get(Calendar.YEAR)) {
				if ((now.get(Calendar.MONTH) == date2.get(Calendar.MONTH)) &&
						(now.get(Calendar.DATE) == date2.get(Calendar.DATE))) {

					df = DateFormat.getTimeInstance(DateFormat.SHORT, locale);

					if (timeZone != null) {
						df.setTimeZone(timeZone);
					}

					return df.format(date);
				}
				else {
					df = DateFormat.getDateInstance(DateFormat.MEDIUM, locale);

					if (timeZone != null) {
						df.setTimeZone(timeZone);
					}

					try {
						SimpleDateFormat sdf = (SimpleDateFormat) df;

						sdf.applyPattern(sdf.toPattern().replaceAll("y", ""));
						sdf.applyPattern(sdf.toPattern().replaceAll(", ", ""));

						return sdf.format(date);
					} catch (ClassCastException e) {
						return df.format(date);
					}
				}
			}
			else {
				df = DateFormat.getDateInstance(DateFormat.MEDIUM, locale);

				if (timeZone != null) {
					df.setTimeZone(timeZone);
				}

				return df.format(date);
			}
		}

		return DATE_UNAVAILABLE;
	}

	public static String formatDateNormalized(Date date) {
		if (Validator.isNull(date)) {
			return "0";
		}

		return DateUtil.getDate(date, "yyyyMMddHHmmss", Locale.getDefault());
	}

	public static String formatSize(double size, Locale locale) {
		NumberFormat numberFormat = NumberFormat.getInstance(locale);

		return numberFormat.format((int)Math.ceil(size / 1000)) + " KB";
	}

	private static final String DATE_UNAVAILABLE = "----";

}