/**
 * Copyright (c) 2000-2009 Liferay, Inc. All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package com.liferay.mail.util;

import com.liferay.mail.AddressException;
import com.liferay.mail.MailboxConstants;
import com.liferay.mail.Recipient;
import com.liferay.mail.model.AccountEntry;
import com.liferay.mail.portlet.MailPortlet;
import com.liferay.mail.service.AccountEntryLocalServiceUtil;
import com.liferay.mail.service.MailServiceUtil;
import com.liferay.portal.PortalException;
import com.liferay.portal.SystemException;
import com.liferay.portal.kernel.log.Log;
import com.liferay.portal.kernel.log.LogFactoryUtil;
import com.liferay.portal.kernel.mail.MailMessage;
import com.liferay.portal.kernel.util.StringUtil;
import com.liferay.portal.model.Company;
import com.liferay.portal.model.User;
import com.liferay.portal.service.CompanyLocalServiceUtil;
import com.liferay.portal.service.UserLocalServiceUtil;

import java.io.IOException;

import java.util.List;

import javax.mail.internet.InternetAddress;

/**
 * <a href="MessageNotificationUtil.java.html"><b><i>View Source</i></b></a>
 *
 * @author Scott Lee
 *
 */
public class MessageNotificationUtil {

	public static void sendNotifications(
			long sendersAccountEntryId, List<Recipient> recipients,
			String subject, String body)
		throws PortalException, SystemException {

		AccountEntry sendersAccountEntry =
			AccountEntryLocalServiceUtil.getAccountEntry(
				sendersAccountEntryId);

		User user = UserLocalServiceUtil.getUser(
			sendersAccountEntry.getUserId());

		Company company = CompanyLocalServiceUtil.getCompany(
			user.getCompanyId());

		try {
			InternetAddress notificationFrom = new InternetAddress(
				company.getEmailAddress());
			String notificationSubject = getNotificationMessageSubject();
			String notificationBody = getNotificationMessageBody();

			notificationSubject = StringUtil.replace(
				notificationSubject,
				new String[] {
					"[$COMPANY_NAME$]",
					"[$FROM_ADDRESS$]",
					"[$FROM_NAME$]"
				},
				new String[] {
					company.getName(),
					sendersAccountEntry.getMessageAddress(),
					user.getFullName()
				});

			notificationBody = StringUtil.replace(
				notificationBody,
				new String[] {
					"[$COMPANY_NAME$]",
					"[$SUBJECT$]",
					"[$BODY$]",
					"[$FROM_ADDRESS$]",
					"[$FROM_NAME$]"
				},
				new String[] {
					company.getName(),
					subject,
					body,
					sendersAccountEntry.getMessageAddress(),
					user.getFullName()
				});

			boolean htmlFormat = true;

			for (Recipient recipient : recipients) {
				AccountEntry recipientAccountEntry =
					AccountEntryLocalServiceUtil.getCompanyAccountEntry(
						company.getCompanyId(),
						MailboxConstants.ACCOUNT_TYPE_PRIVATE,
						recipient.getMessageAddress());

				User recipientUser = UserLocalServiceUtil.getUser(
					recipientAccountEntry.getUserId());

				try {
					InternetAddress notificationTo = new InternetAddress(
						recipientUser.getEmailAddress());

					MailMessage notificationMessage = new MailMessage(
						notificationFrom, notificationTo, notificationSubject,
						notificationBody, htmlFormat);

					MailServiceUtil.sendEmail(notificationMessage);
				}
				catch (javax.mail.internet.AddressException ae) {
					throw new AddressException(
						ae, recipientUser.getEmailAddress());
				}
			}
		}
		catch (javax.mail.internet.AddressException ae) {
			throw new AddressException(ae, company.getEmailAddress());
		}
		catch (IOException ioe) {
			_log.error(ioe);
		}
	}

	protected static String getNotificationMessageBody()
		throws IOException {

		return StringUtil.read(
			MailPortlet.class.getResourceAsStream(
				"dependencies/notification_message_body.tmpl"));
	}

	protected static String getNotificationMessageSubject()
		throws IOException {

		return StringUtil.read(
			MailPortlet.class.getResourceAsStream(
				"dependencies/notification_message_subject.tmpl"));
	}

	private static Log _log = LogFactoryUtil.getLog(
		MessageNotificationUtil.class);

}