/**
 * Copyright (c) 2000-2009 Liferay, Inc. All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package com.liferay.mail.util;

import com.liferay.portal.kernel.util.StringPool;

import java.util.regex.Pattern;

/**
 * <a href="SecurityUtil.java.html"><b><i>View Source</i></b></a>
 *
 * @author Scott Lee
 * @author Minhchau Dang
 * @author Michael C. Han
 *
 */
public class SecurityUtil {

	public static String stripHtml(String html) {
		html = _TAG_PATTERN.matcher(html).replaceAll(StringPool.BLANK);
		html = _LINE_BREAK_PATTERN.matcher(html).replaceAll(StringPool.BLANK);

		return html;
	}

	public static String stripUnsafeHtmlAndCss(String html) {

		// Remove lines (external stylesheets, html, body)

		html = _LINK_TAG_PATTERN.matcher(html).replaceAll(StringPool.BLANK);
		html = _HTML_TAG_PATTERN.matcher(html).replaceAll(StringPool.BLANK);
		html = _BODY_TAG_PATTERN.matcher(html).replaceAll(StringPool.BLANK);
		html = _DOCTYPE_TAG_PATTERN.matcher(html).replaceAll(StringPool.BLANK);

		// Remove blocks (script, style, head)

		html = _SCRIPT_TAG_PATTERN.matcher(html).replaceAll(StringPool.BLANK);
		html = _STYLE_TAG_PATTERN.matcher(html).replaceAll(StringPool.BLANK);
		html = _HEAD_TAG_PATTERN.matcher(html).replaceAll(StringPool.BLANK);

		return html;
	}

	private static final Pattern _BODY_TAG_PATTERN =
		Pattern.compile("</?body[^>]+>", Pattern.CASE_INSENSITIVE);

	private static final Pattern _DOCTYPE_TAG_PATTERN =
		Pattern.compile("<!doctype[^>]+>", Pattern.CASE_INSENSITIVE);

	private static final Pattern _HEAD_TAG_PATTERN =
		Pattern.compile("<head.*?</head>", Pattern.CASE_INSENSITIVE +
			Pattern.DOTALL);

	private static final Pattern _HTML_TAG_PATTERN =
		Pattern.compile("</?html[^>]+>", Pattern.CASE_INSENSITIVE);

	private static final Pattern _LINE_BREAK_PATTERN =
		Pattern.compile("[\r\n]+");

	private static final Pattern _LINK_TAG_PATTERN =
		Pattern.compile("</?link[^>]+>", Pattern.CASE_INSENSITIVE);

	private static final Pattern _SCRIPT_TAG_PATTERN =
		Pattern.compile("<script.*?</script>", Pattern.CASE_INSENSITIVE +
			Pattern.DOTALL);

	private static final Pattern _STYLE_TAG_PATTERN =
		Pattern.compile("<style.*?</style>", Pattern.CASE_INSENSITIVE +
			Pattern.DOTALL);

	private static final Pattern _TAG_PATTERN = Pattern.compile("<[^>]+>");

}