
package com.vaadin.liferay.mail;

import java.io.IOException;

import javax.portlet.ActionRequest;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;

import com.liferay.mail.MessageManager;
import com.liferay.mail.mailbox.PasswordRetriever;
import com.liferay.mail.model.AccountEntry;
import com.liferay.mail.model.AccountSetting;
import com.liferay.mail.service.AccountEntryLocalServiceUtil;
import com.liferay.portal.PortalException;
import com.liferay.portal.SystemException;
import com.liferay.portal.kernel.json.JSONObject;
import com.liferay.portal.kernel.util.ParamUtil;
import com.liferay.portal.kernel.util.StringPool;
import com.liferay.portal.kernel.util.Validator;
import com.liferay.portal.util.PortalUtil;

/**
 * Manager for e-mail accounts. This is based on code from MailPortlet, but uses
 * objects such as AccountEntry instead of parameters for individual fields in
 * requests.
 *
 * @author Henri Sara
 */
// TODO password management: normal, saved, remote - interface & extensibility
public class AccountManager {

	public void deleteAccount(AccountEntry account, Controller controller)
		throws IOException, PortalException, SystemException {

		long accountEntryId = account.getAccountEntryId();

		if (MessageManager.STORE_PASSWORDS_REMOTELY) {
			deleteRemotePassword(account, controller);
		}

		controller.getMessageManager().deleteAccountEntry(accountEntryId);
	}

	public String getPassword(long accountEntryId, Controller controller)
		throws PortalException, SystemException {

		MessageManager messageManager = controller.getMessageManager();

		if (Validator.isNotNull(messageManager) &&
			messageManager.isPasswordSaved(accountEntryId)) {

			AccountEntry accountEntry =
				AccountEntryLocalServiceUtil.getAccountEntry(accountEntryId);
			if (MessageManager.STORE_PASSWORDS_REMOTELY) {
				return getRemotePassword(accountEntry, controller);
			}
			else {
				return accountEntry.getPasswordDecrypted();
			}
		}
		else {
			// TODO implement
			// HttpSession session = request.getSession();
			//
			// return (String) session.getAttribute(messageManager
			// .getPasswordKey(accountEntryId));
			return "";
		}
	}

	public void saveAccount(
		AccountEntry account, AccountSetting accountSetting,
		Controller controller)
		throws PortalException, SystemException, IOException {

		// decrypted password used here
		saveAccount(
			account.getAccountEntryId(), account.getMessageAddress(),
			account.getUsername(), account.getPasswordDecrypted(),
			account.isPasswordSaved(), accountSetting.getMailInHostName(),
			accountSetting.getMailInPort(), accountSetting.isMailInSecure(),
			accountSetting.getMailOutHostName(),
			accountSetting.getMailOutPort(), accountSetting.isMailOutSecure(),
			controller);
	}

	// following the model of MailPortlet (and not modifying the backend),
	// accountEntryId is not used in the saving part but only for password
	// management, and saving identifies old account by messageAddress
	// for a new account, accountEntryId should be 0
	public void saveAccount(
		long accountEntryId, String messageAddress, String username,
		String password, boolean passwordSaved, String mailInHostName,
		int mailInPort, boolean mailInSecure, String mailOutHostName,
		int mailOutPort, boolean mailOutSecure, Controller controller)
		throws PortalException, SystemException, IOException {

		// copy stored password or password from session to the account to store
		if ((password == null || "".equals(password)) && (accountEntryId != 0)) {
			password = getPassword(accountEntryId, controller);
		}

		boolean hasRemotePasswordError = false;

		if (MessageManager.STORE_PASSWORDS_REMOTELY) {
			hasRemotePasswordError = !testRemotePasswordLocker(controller);
		}

		MessageManager messageManager = controller.getMessageManager();

		// TODO implement default folder prefix, sender name, signature, ...
		String defaultFolderPrefix = "";
		String senderName = "";
		String signature = "";
		boolean useSignature = false;

		// TODO disabled testing for now
		// JSONObject jsonObj = messageManager.testAccount(messageAddress,
		// mailInHostName, mailInPort, mailInSecure, mailOutHostName,
		// mailOutPort, mailOutSecure, hasRemotePasswordError, password,
		// username, senderName, signature, useSignature);
		//
		// if (jsonObj.getBoolean("success")) {
		AccountEntry accountEntry =
			messageManager.addOrUpdateAccount(
				messageAddress, mailInHostName, mailInPort, mailInSecure,
				mailOutHostName, mailOutPort, mailOutSecure,
				defaultFolderPrefix, password, passwordSaved, username,
				senderName, signature, useSignature);

		if (MessageManager.STORE_PASSWORDS_REMOTELY) {
			if (passwordSaved) {
				storeRemotePassword(accountEntry, password, controller);
			}
			else {
				deleteRemotePassword(accountEntry, controller);
			}
		}
		// }
	}

	public void storePasswordInSession(ActionRequest actionRequest)
		throws IOException, PortalException, SystemException {

		long accountEntryId =
			ParamUtil.getLong(actionRequest, "accountEntryId");
		String password = ParamUtil.getString(actionRequest, "password");

		AccountEntry accountEntry =
			AccountEntryLocalServiceUtil.getAccountEntry(accountEntryId);

		if (!accountEntry.isPasswordSaved()) {

			// Check to see if password is valid

			MessageManager messageManager =
				MessageManager.getInstance(PortalUtil.getHttpServletRequest(actionRequest));

			JSONObject jsonObj =
				messageManager.testPassword(accountEntryId, password);

			if (jsonObj.getBoolean("success")) {

				// Save password in session

				HttpServletRequest request =
					PortalUtil.getHttpServletRequest(actionRequest);

				PasswordRetriever passwordRetriever =
					MessageManager.getPasswordRetriever();

				HttpSession session = request.getSession();

				session.setAttribute(
					passwordRetriever.getPasswordKey(accountEntryId), password);
			}
		}
	}

	protected void deleteRemotePassword(
		AccountEntry account, Controller controller)
		throws PortalException, SystemException {

	}

	protected String getRemotePassword(
		AccountEntry account, Controller controller)
		throws PortalException, SystemException {

		return StringPool.BLANK;
	}

	protected void storeRemotePassword(
		AccountEntry account, String password, Controller controller)
		throws PortalException, SystemException {

	}

	protected boolean testRemotePasswordLocker(Controller controller)
		throws PortalException, SystemException {

		return false;
	}

}
