
package com.vaadin.liferay.mail;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.liferay.mail.model.AccountEntry;
import com.liferay.mail.model.Folder;
import com.liferay.portal.SystemException;
import com.vaadin.data.Item;
import com.vaadin.data.Property;
import com.vaadin.data.Container.Hierarchical;
import com.vaadin.data.util.MethodProperty;
import com.vaadin.data.util.ObjectProperty;
import com.vaadin.terminal.Resource;
import com.vaadin.terminal.ThemeResource;

public class FolderContainer implements Hierarchical {

	private static final String CAPTION = "caption";

	private static final String ICON = "icon";

	private final Resource ICON_INBOX = new ThemeResource("icons/home.png");

	private final Resource ICON_GENERIC_FOLDER =
		new ThemeResource("icons/folder.png");

	private final Resource ICON_DRAFT_FOLDER =
		new ThemeResource("icons/draft.png");

	private final Resource ICON_TRASH_FOLDER =
		new ThemeResource("icons/trash.png");

	private final Resource ICON_SENT_FOLDER =
		new ThemeResource("icons/sentfolder.png");

	private final Resource ICON_ACCOUNT =
		new ThemeResource("icons/account.png");

	List<ItemId> rootIds = new ArrayList<ItemId>();

	private Map<Long, List<ItemId>> accountFolders =
		new HashMap<Long, List<ItemId>>();
	private Map<Long, AccountEntry> accountIdToAccount =
		new HashMap<Long, AccountEntry>();

	private static List<String> propertyIds;

	static {
		propertyIds = new ArrayList<String>();
		propertyIds.add(CAPTION);
	}

	private class ItemId {

		private AccountEntry account;
		private Folder folder;

		public ItemId(AccountEntry account) {

			this.account = account;
		}

		public AccountEntry getAccount() {

			return account;
		}

		public Folder getFolder() {

			return folder;
		}

		public ItemId(Folder folder) {

			this.folder = folder;
		}

		public boolean isAccount() {

			return (account != null);
		}
	}

	public FolderContainer(List<AccountEntry> accounts) {

		for (AccountEntry account : accounts) {
			rootIds.add(new ItemId(account));
			accountIdToAccount.put(account.getAccountEntryId(), account);
		}
	}

	public Collection rootItemIds() {

		return rootIds;
	}

	public boolean hasChildren(Object itemId) {

		if (rootIds.contains(itemId)) {
			return true;
		}

		return false;
	}

	public Collection getChildren(Object itemId) {

		ItemId id = (ItemId) itemId;
		if (id.isAccount()) {
			return getFolders(id.getAccount().getAccountEntryId());
		}
		else {
			return null;
		}
	}

	private List<ItemId> getFolders(Long accountEntryId) {

		List<ItemId> itemIds = accountFolders.get(accountEntryId);
		if (itemIds == null) {
			itemIds = new ArrayList<ItemId>();
			try {
				List<Folder> folders =
					Controller.get().getMailStore().getFolders(accountEntryId);
				for (Folder folder : folders) {
					itemIds.add(new ItemId(folder));
				}
			}
			catch (SystemException e) {
				e.printStackTrace();
			}

			accountFolders.put(accountEntryId, itemIds);
		}

		return itemIds;
	}

	public Object getParent(Object itemId) {

		if (rootIds.contains(itemId)) {
			return null;
		}
		else {
			return ((ItemId) itemId).getFolder().getAccountEntryId();
		}
	}

	public boolean areChildrenAllowed(Object itemId) {

		boolean rootItem = isRoot(itemId);
		return rootItem;
	}

	public boolean isRoot(Object itemId) {

		return getParent(itemId) == null;
	}

	public boolean setChildrenAllowed(Object itemId, boolean areChildrenAllowed)
		throws UnsupportedOperationException {

		throw new UnsupportedOperationException();
	}

	public boolean setParent(Object itemId, Object newParentId)
		throws UnsupportedOperationException {

		throw new UnsupportedOperationException();
	}

	public Property getContainerProperty(Object itemId, Object propertyId) {

		ItemId id = (ItemId) itemId;
		if (CAPTION.equals(propertyId)) {
			if (id.isAccount()) {
				return new MethodProperty(id.getAccount(), "messageAddress");
			}
			else {
				Folder f = id.getFolder();
				if (accountIdToAccount.get(f.getAccountEntryId()).getInboxFolderId() == f.getFolderId()) {
					String text = id.getFolder().getName();
					try {
						long unread =
							Controller.get().getMailStore().getUnreadMessageCount(
								f);
						text += " (" + unread + ")";
					}
					catch (SystemException e) {
						text += " (?)";
						e.printStackTrace();
					}

					return new ObjectProperty(text);
				}
				else {
					return new MethodProperty(id.getFolder(), "name");
				}
			}
		}
		else if (ICON.equals(propertyId)) {
			Resource icon = getIconForItem(id);
			if (icon == null) {
				return null;
			}
			else {
				return new ObjectProperty(icon);
			}

		}
		else {
			return null;
		}
	}

	private Resource getIconForItem(ItemId id) {

		if (id.isAccount()) {
			return ICON_ACCOUNT;
		}
		else {
			Folder f = id.getFolder();
			AccountEntry account =
				accountIdToAccount.get(f.getAccountEntryId());
			if (account.getInboxFolderId() == f.getFolderId()) {
				return ICON_INBOX;
			}
			else if (account.getTrashFolderId() == f.getFolderId()) {
				return ICON_TRASH_FOLDER;
			}
			else if (account.getDraftFolderId() == f.getFolderId()) {
				return ICON_DRAFT_FOLDER;
			}
			else if (account.getSentFolderId() == f.getFolderId()) {
				return ICON_SENT_FOLDER;
			}
			else {
				return ICON_GENERIC_FOLDER;
			}
		}

	}

	public static Object getCaptionPropertyId() {

		return CAPTION;
	}

	public static Object getIconPropertyId() {

		return ICON;
	}

	public Folder getFolder(Object itemId) {

		ItemId id = (ItemId) itemId;
		if (id.isAccount()) {
			return null;
		}

		return id.getFolder();
	}

	public boolean addContainerProperty(
		Object propertyId, Class<?> type, Object defaultValue)
		throws UnsupportedOperationException {

		throw new UnsupportedOperationException();
	}

	public Object addItem()
		throws UnsupportedOperationException {

		throw new UnsupportedOperationException();
	}

	public Item addItem(Object itemId)
		throws UnsupportedOperationException {

		throw new UnsupportedOperationException();
	}

	public boolean containsId(Object itemId) {

		if (itemId instanceof ItemId) {
			return true;
		}

		return false;
	}

	public Collection<?> getContainerPropertyIds() {

		return propertyIds;
	}

	public Item getItem(Object itemId) {

		return null;
	}

	public Collection<?> getItemIds() {

		return rootItemIds();
	}

	public Class<?> getType(Object propertyId) {

		if (ICON.equals(propertyId)) {
			return Resource.class;
		}
		return String.class;
	}

	public boolean removeAllItems()
		throws UnsupportedOperationException {

		throw new UnsupportedOperationException();
	}

	public boolean removeContainerProperty(Object propertyId)
		throws UnsupportedOperationException {

		throw new UnsupportedOperationException();
	}

	public boolean removeItem(Object itemId)
		throws UnsupportedOperationException {

		throw new UnsupportedOperationException();
	}

	public int size() {

		return 0;
	}

	public AccountEntry getAccountEntry(Object value) {

		if (value == null || !(value instanceof ItemId)) {
			return null;
		}
		ItemId id = (ItemId) value;
		if (id.isAccount()) {
			return id.getAccount();
		}
		else {
			return accountIdToAccount.get(id.getFolder().getAccountEntryId());
		}
	}

	@SuppressWarnings("unchecked")
	public Object getAccountItemId(Long accountEntryId) {

		if (accountEntryId == null) {
			return null;
		}

		for (ItemId itemId : (Collection<ItemId>) rootItemIds()) {
			if (itemId.getAccount().getAccountEntryId() == accountEntryId) {
				return itemId;
			}
		}

		return null;
	}

	public Object getFolderItemId(Long accountEntryId, Long folderId) {

		if (accountEntryId == null || folderId == null) {
			return null;
		}

		List<ItemId> folders = getFolders(accountEntryId);
		if (folders == null) {
			return null;
		}

		for (ItemId folderItemId : folders) {
			if (folderItemId.getFolder().getFolderId() == folderId) {
				return folderItemId;
			}
		}

		return null;
	}

}
