
package com.vaadin.liferay.mail;

import com.vaadin.data.Container;
import com.vaadin.data.Property;
import com.vaadin.ui.Tree;

public class FolderTree extends Tree {

	FolderChangeListener folderChangedListener;

	public FolderTree(FolderChangeListener listener) {

		super("");
		this.folderChangedListener = listener;

		setSizeFull();
		setSelectable(true);
		setNullSelectionAllowed(false);
		setImmediate(true);
		setMultiSelect(false);

		addListener(new Property.ValueChangeListener() {

			public void valueChange(Property.ValueChangeEvent event) {

				if (folderChangedListener != null) {
					folderChangedListener.selectedFolderChanged(((FolderContainer) getContainerDataSource()).getFolder(event.getProperty().getValue()));
				}
			}

		});
	}

	@Override
	public void setContainerDataSource(Container newDataSource) {

		super.setContainerDataSource(newDataSource);

		if (newDataSource instanceof FolderContainer) {
			setItemCaptionPropertyId(FolderContainer.getCaptionPropertyId());
			setItemIconPropertyId(FolderContainer.getIconPropertyId());
		}
	}

	private FolderContainer getFolderContainer() {

		return (FolderContainer) getContainerDataSource();
	}

	public void expand(Long accountEntryId, Long folderId) {

		if (accountEntryId == null) {
			return;
		}
		expandItemsRecursively(getFolderContainer().getAccountItemId(
			accountEntryId));

	}

	public void selectFolder(Long accountEntryId, Long folderId) {

		if (accountEntryId == null || folderId == null) {
			return;
		}
		select(getFolderContainer().getFolderItemId(accountEntryId, folderId));
	}
}
