
package com.vaadin.liferay.mail;

import java.util.List;

import com.liferay.mail.model.AccountEntry;
import com.liferay.mail.model.Folder;
import com.vaadin.liferay.mail.Controller.MessageModifiedListener;
import com.vaadin.liferay.mail.util.Lang;
import com.vaadin.ui.Accordion;
import com.vaadin.ui.Button;
import com.vaadin.ui.CustomComponent;
import com.vaadin.ui.NativeButton;
import com.vaadin.ui.VerticalLayout;
import com.vaadin.ui.Button.ClickEvent;
import com.vaadin.ui.Button.ClickListener;

@SuppressWarnings("serial")
public class Folders extends CustomComponent
	implements FolderChangeListener, MessageModifiedListener {

	private VerticalLayout mainLayout;
	private VerticalLayout accountLayout;
	private Accordion mainPanel;

	private MessageList messageList;
	private FolderTree tree;
	private FolderContainer folderContainer;

	/**
	 * The constructor should first build the main layout, set the composition
	 * root and then do any custom initialization. The constructor will not be
	 * automatically regenerated by the visual editor.
	 */
	public Folders(MessageList messageList) {

		this.messageList = messageList;

		mainLayout = new VerticalLayout();
		mainLayout.setSizeFull();

		mainPanel = new Accordion();
		mainPanel.setSizeFull();
		mainPanel.setStyleName("borderless");

		accountLayout = new VerticalLayout();
		accountLayout.setCaption(Lang.get("accounts"));

		mainLayout.addComponent(createComposeButton());
		mainLayout.addComponent(mainPanel);
		mainLayout.setExpandRatio(mainPanel, 1);

		mainPanel.addComponent(accountLayout);

		tree = new FolderTree(this);
		accountLayout.addComponent(tree);

		// top-level component properties
		setSizeFull();
		setCompositionRoot(mainLayout);

		Controller.get().addListener(this);
	}

	private Button createComposeButton() {

		Button compose = new NativeButton("Compose new message");
		compose.addListener(new Button.ClickListener() {

			public void buttonClick(ClickEvent event) {

				ComposeWindow cw = new ComposeWindow(new Composer());
				Controller.get().getApplication().getMainWindow().addWindow(cw);
			}
		});

		compose.setStyleName("compose");
		compose.setWidth("100%");

		return compose;
	}

	public void setAccounts(List<AccountEntry> accountEntries) {

		folderContainer = new FolderContainer(accountEntries);

		tree.setContainerDataSource(folderContainer);
	}

	public void setFolders(List<Folder> folders) {

		for (Folder folder : folders) {
			Button b = new Button(folder.getName());
			b.setStyleName(Button.STYLE_LINK);
			b.setData(folder);
			b.addListener(new ClickListener() {

				public void buttonClick(ClickEvent event) {

					showFolder(((Folder) event.getButton().getData()));
				}
			});

		}
	}

	private void showFolder(Folder folder) {

		if (folder == null) {
			showFolder(null, null);
		}
		else {
			showFolder(folder.getAccountEntryId(), folder.getFolderId());
		}
	}

	private void showFolder(Long accountEntryId, Long folderId) {

		// Make sure the folder is expanded
		tree.expand(accountEntryId, folderId);

		messageList.showMessages(accountEntryId, folderId);
	}

	public void showInbox(AccountEntry accountEntry) {

		tree.selectFolder(
			accountEntry.getAccountEntryId(), accountEntry.getInboxFolderId());

	}

	public AccountEntry getCurrentAccount() {

		return folderContainer.getAccountEntry(tree.getValue());
	}

	public void selectedFolderChanged(Folder folder) {

		showFolder(folder);
	}

	public void messageModified(long messageId) {

		tree.requestRepaint();
	}

	public void messagesAddedOrRemoved() {

		// Not interested in these events
	}

}

interface FolderChangeListener {

	public void selectedFolderChanged(Folder folder);
}
