
package com.vaadin.liferay.mail;

import java.util.List;

import javax.portlet.ActionRequest;
import javax.portlet.ActionResponse;
import javax.portlet.EventRequest;
import javax.portlet.EventResponse;
import javax.portlet.PortletMode;
import javax.portlet.PortletModeException;
import javax.portlet.PortletURL;
import javax.portlet.RenderRequest;
import javax.portlet.RenderResponse;
import javax.portlet.ResourceRequest;
import javax.portlet.ResourceResponse;
import javax.portlet.WindowState;
import javax.portlet.WindowStateException;

import com.liferay.mail.model.AccountEntry;
import com.liferay.portal.PortalException;
import com.liferay.portal.SystemException;
import com.liferay.portal.model.User;
import com.liferay.portal.util.PortalUtil;
import com.vaadin.Application;
import com.vaadin.liferay.mail.util.Lang;
import com.vaadin.terminal.ExternalResource;
import com.vaadin.terminal.gwt.server.PortletApplicationContext2;
import com.vaadin.terminal.gwt.server.PortletApplicationContext2.PortletListener;
import com.vaadin.ui.Button;
import com.vaadin.ui.HorizontalLayout;
import com.vaadin.ui.Label;
import com.vaadin.ui.NativeButton;
import com.vaadin.ui.VerticalLayout;
import com.vaadin.ui.Window;
import com.vaadin.ui.Button.ClickEvent;
import com.vaadin.ui.Button.ClickListener;
import com.vaadin.ui.Window.CloseEvent;
import com.vaadin.ui.Window.Notification;

@SuppressWarnings("serial")
public class MailApplication extends Application {

	Window mainWindow;

	MainPopupWindow mainPopupWindow;
	private Controller controller;
	private String portletTitle;

	private VerticalLayout main;
	private HorizontalLayout top;
	private Button unreadCount;

	private String preferencesURL;

	@Override
	public void init() {

		controller = new Controller(this);
		mainWindow = new Window();
		setMainWindow(mainWindow);
		setTheme("mailportlet");

		if (getContext() instanceof PortletApplicationContext2) {
			PortletApplicationContext2 ctx =
				(PortletApplicationContext2) getContext();
			ctx.addPortletListener(this, new MailPortletListener());
		}
		else {
			// TODO The servlet has been initialized directly so we need to
			// restart
			// it.
			getMainWindow().showNotification(
				"Not inited via Portal!", Notification.TYPE_ERROR_MESSAGE);
		}

	}

	public Controller getController() {

		return controller;
	}

	private class MailPortletListener implements PortletListener {

		public void handleActionRequest(
			ActionRequest request, ActionResponse response) {
			// nothing to do
		}

		public void handleRenderRequest(
			RenderRequest request, RenderResponse response) {

			if (preferencesURL == null) {
				PortletURL url = response.createActionURL();
				try {
					url.setPortletMode(PortletMode.EDIT);
					url.setWindowState(WindowState.MAXIMIZED);
					preferencesURL = url.toString();
				}
				catch (PortletModeException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
				catch (WindowStateException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}

			mainWindow.removeAllComponents();

			// check mode in request and act accordingly
			// TODO this could also be done with separate windows for each mode
			if (PortletMode.VIEW.equals(request.getPortletMode())) {
				try {
					User user = PortalUtil.getUser(request);
					controller.setUser(user);
					if (user == null) {
						Label msg = new Label("Login to use the mail portlet");
						msg.setStyleName("portlet-msg-error");
						mainWindow.addComponent(msg);

						return;
					}
					else {
						portletTitle =
							PortalUtil.getPortletTitle(
								PortalUtil.getPortletId(request),
								controller.getUser());
						controller.setPreferences(request.getPreferences());
					}
				}
				catch (PortalException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
				catch (SystemException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}

				main = new VerticalLayout();
				main.setStyleName("v-main");
				mainWindow.setContent(main);

				createTopbar();
				updateUnreadCount();

			}
			else if (PortletMode.EDIT.equals(request.getPortletMode())) {
				createPreferencesView();
			}
		}

		public void handleEventRequest(EventRequest request,
				EventResponse response) {
			// nothing to do
		}

		public void handleResourceRequest(ResourceRequest request,
				ResourceResponse response) {
			// nothing to do
		}
	}

	private void createTopbar() {

		top = new HorizontalLayout();
		top.setStyleName("v-top");
		top.setSpacing(true);
		top.setWidth("100%");

		final Button prefsButton =
			new NativeButton(
				Lang.get("preferences"), new Button.ClickListener() {

					public void buttonClick(ClickEvent event) {

						mainWindow.open(new ExternalResource(
							preferencesURL.toString()));
					}

				});
		prefsButton.setDescription(Lang.get("preferences"));
		prefsButton.setStyleName("icon");
		prefsButton.addStyleName("prefs");

		final Button reloadButton =
			new NativeButton(Lang.get("refresh"), new Button.ClickListener() {

				public void buttonClick(ClickEvent event) {

					updateUnreadCount();

				}

			});
		reloadButton.setDescription(Lang.get("refresh"));
		reloadButton.setStyleName("icon");
		reloadButton.addStyleName("reload");

		final Button composeButton =
			new NativeButton("Compose Message", new Button.ClickListener() {

				public void buttonClick(ClickEvent event) {

					ComposeWindow c = new ComposeWindow(new Composer());
					c.center();
					mainWindow.addWindow(c);
				}

			});
		composeButton.setStyleName("compose");

		unreadCount = new NativeButton("", new Button.ClickListener() {

			public void buttonClick(ClickEvent event) {

				getMainPopupWindow().showFirstAccountWithUnreadMessages();

			}
		});
		unreadCount.setWidth("100%");
		unreadCount.setStyleName("unread");
		top.addComponent(unreadCount);
		top.setExpandRatio(unreadCount, 1);
		top.addComponent(prefsButton);
		top.addComponent(reloadButton);
		top.addComponent(composeButton);

		main.addComponent(top);
	}

	private void createAccountEntriesLinks() {

		int unread = 0;
		try {
			List<AccountEntry> accountEntries =
				controller.getMessageManager().getAccountEntries(
					false, true);
			for (final AccountEntry accountEntry : accountEntries) {
				final long messages =
					controller.getMessageManager().getUnreadMessageCountByAccountEntryId(
						accountEntry.getAccountEntryId());
				unread += messages;
				Button b =
					new NativeButton("Private messages (" + messages + ")");
				b.setStyleName("account");
				b.setWidth("100%");

				b.addListener(new ClickListener() {

					public void buttonClick(ClickEvent event) {

						getMainPopupWindow().show(accountEntry);
					}
				});

				main.addComponent(b);
			}
		}
		catch (SystemException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		try {
			List<AccountEntry> accountEntries =
				controller.getMessageManager().getAccountEntries(
					true, false);
			for (final AccountEntry accountEntry : accountEntries) {
				final long messages =
					controller.getMessageManager().getUnreadMessageCountByAccountEntryId(
						accountEntry.getAccountEntryId());
				unread += messages;
				Button b =
					new NativeButton(accountEntry.getMessageAddress() + " (" +
						messages + ")");
				b.setStyleName("account");
				b.setWidth("100%");
				main.addComponent(b);

				b.addListener(new ClickListener() {

					public void buttonClick(ClickEvent event) {

						getMainPopupWindow().show(accountEntry);
					}
				});
			}
		}
		catch (SystemException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		if (unread > 0) {
			unreadCount.setCaption(unread + " unread message(s)");
			unreadCount.getParent().addStyleName("v-unread");
		}
		else {
			unreadCount.setCaption("No unread messages");
			unreadCount.getParent().removeStyleName("v-unread");
		}
	}

	private void createPreferencesView() {

		// preferences: own e-mail accounts, add new account
		mainWindow.setContent(new PreferencesView(controller));
	}

	private void updateUnreadCount() {

		main.removeAllComponents();
		main.addComponent(top);
		createAccountEntriesLinks();
	}

	public MainPopupWindow getMainPopupWindow() {

		if (mainPopupWindow == null) {
			mainPopupWindow = new MainPopupWindow(portletTitle);
			mainPopupWindow.addListener(new Window.CloseListener() {

				public void windowClose(CloseEvent e) {

					updateUnreadCount();

				}

			});
		}

		return mainPopupWindow;
	}
}
