
package com.vaadin.liferay.mail;

import java.util.ArrayList;
import java.util.List;

import com.liferay.mail.model.AccountEntry;
import com.liferay.mail.model.Message;
import com.liferay.portal.SystemException;
import com.vaadin.ui.SplitPanel;
import com.vaadin.ui.Window;

@SuppressWarnings("serial")
public class MainPopupWindow extends Window {

	private MessageList messageList;
	private Folders folders;
	private MessageView messageView;

	public MainPopupWindow(String caption) {

		setCaption("Mail");
		messageView = new MessageView();
		messageList = new MessageList(this, messageView);
		folders = new Folders(messageList);

		SplitPanel messageListViewSplit =
			new SplitPanel(SplitPanel.ORIENTATION_VERTICAL);
		messageListViewSplit.setSizeFull();
		messageListViewSplit.setSplitPosition(250, SplitPanel.UNITS_PIXELS);
		messageListViewSplit.setFirstComponent(messageList);
		messageListViewSplit.setSecondComponent(messageView);

		SplitPanel split = new SplitPanel(SplitPanel.ORIENTATION_HORIZONTAL);
		split.setSplitPosition(170, SplitPanel.UNITS_PIXELS);
		setContent(split);
		split.setStyleName("small");

		split.addComponent(folders);
		split.addComponent(messageListViewSplit);

		setWidth("80%");
		setHeight("80%");
		center();
		setStyleName("mainpopup");

	}

	public void show(AccountEntry accountEntry) {

		try {
			List<AccountEntry> accountEntries =
				Controller.get().getMessageManager().getAccountEntries(
					true, true);

			folders.setAccounts(accountEntries);
		}
		catch (SystemException e) {
			Controller.get().showUnexpectedError(e);
		}

		folders.showInbox(accountEntry);

		// Display window to user
		Controller.get().getApplication().getMainWindow().addWindow(this);

	}

	public void showFirstAccountWithUnreadMessages() {

		List<AccountEntry> accountEntries = new ArrayList<AccountEntry>();
		try {
			accountEntries =
				Controller.get().getMessageManager().getAccountEntries(
					true, true);

			folders.setAccounts(accountEntries);
		}
		catch (SystemException e) {
			Controller.get().showUnexpectedError(e);
		}

		for (final AccountEntry accountEntry : accountEntries) {
			final long messages =
				Controller.get().getMessageManager().getUnreadMessageCountByAccountEntryId(
					accountEntry.getAccountEntryId());
			if (messages > 0) {
				show(accountEntry);
				return;
			}
		}
		show(accountEntries.get(0));
	}

	public AccountEntry getCurrentAccount() {

		return folders.getCurrentAccount();
	}

	public Message getSelectedMessage() {

		return messageList.getSelectedMessage();
	}

	public List<Message> getSelectedMessages() {

		return messageList.getSelectedMessages();
	}

}
