
package com.vaadin.liferay.mail;

import java.util.List;

import com.liferay.mail.model.Message;
import com.liferay.portal.PortalException;
import com.liferay.portal.SystemException;
import com.vaadin.liferay.mail.util.Lang;
import com.vaadin.ui.CustomComponent;
import com.vaadin.ui.VerticalLayout;

public class MessageList extends CustomComponent {

	public static final String STYLE_IMPORTANT = "message-important";
	public static final String STYLE_NOT_SEEN = "message-not-seen";

	private VerticalLayout mainLayout;
	private MessageListTable table;
	private MessageToolbar top;

	private MessageView messageView;
	private MainPopupWindow mainPopupWindow;

	/**
	 * The constructor should first build the main layout, set the composition
	 * root and then do any custom initialization. The constructor will not be
	 * automatically regenerated by the visual editor.
	 *
	 * @param mainPopupWindow
	 */
	public MessageList(MainPopupWindow mainPopupWindow, MessageView messageView) {

		this.mainPopupWindow = mainPopupWindow;
		this.messageView = messageView;
		buildMainLayout();
		setCompositionRoot(mainLayout);

		mainLayout.setSizeFull();
		mainLayout.setExpandRatio(table, 1.0f);

	}

	protected void showMessage(Message msg) {

		messageView.showMessage(msg);
		// Mark message as seen
		if (msg != null && !MessageUtil.isSeen(msg)) {
			try {
				MessageUtil.markMessageRead(msg, true);
			}
			catch (PortalException e) {
				Controller.get().showError(
					Lang.get("unable-to-flag-messages"), e);
			}
			catch (SystemException e) {
				Controller.get().showError(
					Lang.get("unable-to-flag-messages"), e);
			}

		}
	}

	private VerticalLayout buildMainLayout() {

		// common part: create layout
		mainLayout = new VerticalLayout();

		// top-level component properties
		setHeight("100.0%");
		setWidth("100.0%");

		// top
		top = new MessageToolbar(mainPopupWindow);
		top.setImmediate(false);
		top.setHeight(null);
		top.setWidth("100.0%");
		mainLayout.addComponent(top);

		// table
		table = new MessageListTable(this);
		mainLayout.addComponent(table);

		return mainLayout;
	}

	/**
	 * Returns the currently selected messages. If one more more message is
	 * checked, these are returned. If no messages are checked, the selected
	 * message is returned.
	 *
	 * @return
	 */
	public List<Message> getSelectedMessages() {

		return table.getSelectedMessages();
	}

	public Message getSelectedMessage() {

		return table.getSelectedMessage();
	}

	public void showMessages(Long accountEntryId, Long folderId) {

		table.showMessages(accountEntryId, folderId);
		top.updateToolbar(accountEntryId, folderId);

	}

}
