
package com.vaadin.liferay.mail;

import java.util.ArrayList;
import java.util.List;

import com.liferay.mail.model.AccountEntry;
import com.liferay.mail.model.Folder;
import com.liferay.mail.model.Message;
import com.liferay.portal.PortalException;
import com.liferay.portal.SystemException;
import com.vaadin.liferay.mail.Composer.ComposerListener;
import com.vaadin.liferay.mail.util.Lang;
import com.vaadin.terminal.Resource;
import com.vaadin.terminal.ThemeResource;
import com.vaadin.ui.Button;
import com.vaadin.ui.HorizontalLayout;
import com.vaadin.ui.Label;
import com.vaadin.ui.NativeButton;
import com.vaadin.ui.Panel;
import com.vaadin.ui.PopupView;
import com.vaadin.ui.VerticalLayout;
import com.vaadin.ui.Button.ClickEvent;
import com.vaadin.ui.Button.ClickListener;
import com.vaadin.ui.PopupView.PopupVisibilityEvent;
import com.vaadin.ui.PopupView.PopupVisibilityListener;

@SuppressWarnings("serial")
public class MessageToolbar extends HorizontalLayout {

	private static final Resource ICON_EDIT_DRAFT =
		new ThemeResource("icons/edit_draft.png");
	private static final Resource ICON_REPLY_ALL =
		new ThemeResource("icons/reply_all.png");
	private static final Resource ICON_REPLY =
		new ThemeResource("icons/reply.png");
	private static final Resource ICON_FORWARD =
		new ThemeResource("icons/forward.png");
	private static final Resource ICON_DELETE =
		new ThemeResource("icons/delete.png");
	private static final Resource ICON_MOVE_TO =
		new ThemeResource("icons/move.png");

	private int moveToIndex = 4;

	private Button openDraft;
	private Button reply;
	private Button replyAll;
	private Button forward;
	private Button delete;
	private Button moveTo;

	private MainPopupWindow mainPopupWindow;

	private String TOOLBAR_BUTTON_STYLE = "toolbar";

	public MessageToolbar(MainPopupWindow mainPopupWindow) {

		this.mainPopupWindow = mainPopupWindow;

		openDraft = new NativeButton(Lang.get("edit-draft"), new ClickListener() {

			public void buttonClick(ClickEvent event) {

				openDraft();
			}
		});
		openDraft.setIcon(ICON_EDIT_DRAFT);
		openDraft.setStyleName(TOOLBAR_BUTTON_STYLE);
		reply = new NativeButton(Lang.get("reply"), new ClickListener() {

			public void buttonClick(ClickEvent event) {

				reply(false);
			}
		});
		reply.setIcon(ICON_REPLY);
		reply.setStyleName(TOOLBAR_BUTTON_STYLE);
		replyAll = new NativeButton(Lang.get("reply-all"), new ClickListener() {

			public void buttonClick(ClickEvent event) {

				reply(true);
			}
		});
		replyAll.setIcon(ICON_REPLY_ALL);
		replyAll.setStyleName(TOOLBAR_BUTTON_STYLE);
		forward = new NativeButton(Lang.get("forward"), new ClickListener() {

			public void buttonClick(ClickEvent event) {

				forward();
			}
		});
		forward.setIcon(ICON_FORWARD);
		forward.setStyleName(TOOLBAR_BUTTON_STYLE);
		delete = new NativeButton(Lang.get("delete"), new ClickListener() {

			public void buttonClick(ClickEvent event) {

				delete();
			}
		});
		delete.setIcon(ICON_DELETE);
		delete.setStyleName(TOOLBAR_BUTTON_STYLE);
		moveTo = new NativeButton(Lang.get("move-to"), new ClickListener() {

			public void buttonClick(ClickEvent event) {

				selectMoveTarget();
			}
		});
		moveTo.setIcon(ICON_MOVE_TO);
		moveTo.setStyleName(TOOLBAR_BUTTON_STYLE);

		updateToolbar(null, null);

		setStyleName("toolbar");
	}

	public void updateToolbar(Long accountEntryId, Long folderId) {

		boolean drafts = isDraftFolder(accountEntryId, folderId);

		removeAllComponents();

		if (drafts) {
			addComponent(openDraft);
		}
		else {
			addComponent(reply);
			addComponent(replyAll);
			addComponent(forward);
			moveToIndex = 4;
			addComponent(moveTo);
		}
		addComponent(delete);

		Label spacer = new Label("");
		addComponent(spacer);

		setExpandRatio(spacer, 1);
	}

	private boolean isDraftFolder(Long accountEntryId, Long folderId) {

		boolean drafts = false;

		if (accountEntryId != null && folderId != null) {
			try {
				AccountEntry account =
					MessageUtil.getAccountEntry(accountEntryId);
				drafts = (account.getDraftFolderId() == folderId);
			}
			catch (SystemException e) {
				// show normal mode and not draft mode if there was a problem
				e.printStackTrace();
			}
			catch (PortalException e) {
				// show normal mode and not draft mode if there was a problem
				e.printStackTrace();
			}
		}
		return drafts;
	}

	protected void delete() {

		List<Message> messages = mainPopupWindow.getSelectedMessages();
		if (messages.isEmpty()) {
			Controller.get().showError(Lang.get("no-messages-selected"));
			return;
		}

		try {
			MessageUtil.deleteMessage(messages);

			Controller.get().showInfo(Lang.get("messages-have-been-deleted"));
		}
		catch (Exception e) {
			Controller.get().showError(Lang.get("unable-to-delete-messages"), e);
		}

	}

	protected void selectMoveTarget() {

		// Ensure a mail is selected
		List<Message> message = mainPopupWindow.getSelectedMessages();
		if (message.isEmpty()) {
			Controller.get().showInfo(Lang.get("no-messages-selected"));
			return;
		}

		AccountEntry account;
		try {
			account = MessageUtil.getAccountForMessage(message.get(0));
		}
		catch (Exception e) {
			Controller.get().showError(Lang.get("unable-to-move-messages"), e);
			return;
		}

		VerticalLayout panelLayout = new VerticalLayout();
		panelLayout.setMargin(false, true, true, false);
		panelLayout.setSizeUndefined();

		Panel p = new Panel(panelLayout);
		p.setSizeUndefined();

		final PopupView popupView = new PopupView("", p);

		FolderTree destinationTree = new FolderTree(new FolderChangeListener() {

			public void selectedFolderChanged(Folder folder) {

				if (folder != null) {
					popupView.setPopupVisible(false);
					try {
						MessageUtil.moveMessagesTo(
							mainPopupWindow.getSelectedMessages(), folder);

						Controller.get().showInfo(
							Lang.get("messages-have-been-moved"));
					}
					catch (PortalException e) {
						Controller.get().showError(
							Lang.get("unable-to-move-messages"), e);
					}
					catch (SystemException e) {
						Controller.get().showError(
							Lang.get("unable-to-move-messages"), e);
					}

				}
			}
		});

		p.addComponent(destinationTree);

		popupView.setPopupVisible(true);
		popupView.addListener(new PopupVisibilityListener() {

			public void popupVisibilityChange(PopupVisibilityEvent event) {

				// Remove popupview from layout when it has been closed
				if (!event.isPopupVisible()) {
					removeComponent(event.getPopupView());

				}
			}
		});

		// Set tree properties
		List<AccountEntry> accounts = new ArrayList<AccountEntry>();
		accounts.add(account);
		final FolderContainer folderContainer = new FolderContainer(accounts);

		destinationTree.setContainerDataSource(folderContainer);
		destinationTree.expandItemsRecursively(destinationTree.rootItemIds().iterator().next());

		addComponent(popupView, moveToIndex);

	}

	protected void forward() {

		// Ensure a mail is selected
		Message message = mainPopupWindow.getSelectedMessage();
		if (message == null) {
			Controller.get().showInfo(Lang.get("no-messages-selected"));
			return;
		}

		Controller.get().forwardInComposer(message);

	}

	protected void reply(boolean toAll) {

		// Ensure a mail is selected
		final Message message = mainPopupWindow.getSelectedMessage();
		if (message == null) {
			Controller.get().showInfo(Lang.get("no-messages-selected"));
			return;
		}

		Composer composer = Controller.get().replyInComposer(message, toAll);
		composer.addListener(new ComposerListener() {

			public void messageDiscarded(Composer composer) {

				// TODO Auto-generated method stub

			}

			public void messageSaved(Composer composer) {

				// TODO Auto-generated method stub

			}

			public void messageSent(Composer composer) {

				// Set replied flag on mail
				try {
					MessageUtil.setAnswered(message, true);
				}
				catch (PortalException e) {
					Controller.get().showError(
						Lang.get("unable-to-flag-messages"), e);
				}
				catch (SystemException e) {
					Controller.get().showError(
						Lang.get("unable-to-flag-messages"), e);
				}

			}

		});

	}

	protected void openDraft() {

		// Ensure a mail is selected
		final Message message = mainPopupWindow.getSelectedMessage();
		if (message == null) {
			Controller.get().showInfo(Lang.get("no-messages-selected"));
			return;
		}

		Controller.get().openDraftInComposer(message);

	}

}
