<%
/**
 * Copyright (c) 2000-2009 Liferay, Inc. All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
%>

<%@ include file="/init.jsp" %>

<%
MessageManager messageManager = MessageManager.getInstance(request);
%>

<div class="mail-portlet">
	<h3><liferay-ui:message key="your-email-accounts" /></h3>

	<fieldset class="block-labels">
		<div class="ctrl-holder" id="<portlet:namespace />savedAccountsList" />
	</fieldset>

	<h3><liferay-ui:message key="add-a-new-email-account" /></h3>

	<fieldset class="block-labels">
		<div class="ctrl-holder" id="<portlet:namespace />preconfiguredAccountsList" />
	</fieldset>
</div>

<script type="text/javascript">
jQuery(
	function(){
		var preconfiguredMailAccounts = <%= preferences.getValue("preconfiguredMailAccounts", PortletProps.get("preconfigured.mail.accounts")) %>;
		var savedMailAccounts = <%= messageManager.getJSONAccounts(true, false) %>;
		var preconfiguredAccountsList = jQuery('#<portlet:namespace />preconfiguredAccountsList');
		var savedAccountsList = jQuery('#<portlet:namespace />savedAccountsList');

		function getPreconfiguredMailAccount(titleLanguageKey) {
			for (i = 0; i < preconfiguredMailAccounts.accountEntries.length; i++) {
				var tempAccountEntry = preconfiguredMailAccounts.accountEntries[i];

				if (tempAccountEntry.titleLanguageKey == titleLanguageKey) {
					return tempAccountEntry;
				}
			}

			return null;
		}

		function getSavedMailAccount(accountEntryId) {
			for (i = 0; i < savedMailAccounts.accountEntries.length; i++) {
				var tempAccountEntry = savedMailAccounts.accountEntries[i];

				if (tempAccountEntry.accountEntryId == accountEntryId) {
					return tempAccountEntry;
				}
			}

			return null;
		}

		jQuery('.add-account').livequery('click', function () {
			var titleLanguageKey = jQuery(this).attr('titleLanguageKey');

			editAccountEntry(getPreconfiguredMailAccount(titleLanguageKey), true);
		});

		jQuery('.cancel-setup').livequery('click', function() {
			Liferay.Popup.close('#<portlet:namespace />accountEntryFm');
		});

		jQuery('.delete-account').livequery('click', function () {
			var answer = confirm('<liferay-ui:message key="are-you-sure-you-want-to-delete-this-account" />');

			if (answer) {
				jQuery(this).parents('form').submit();

				jQuery(this).attr('disabled', true);
			}
		});

		jQuery('.edit-account').livequery('click', function () {
			var accountEntryId = jQuery(this).attr('accountEntryId');

			editAccountEntry(getSavedMailAccount(accountEntryId), false);
		});

		jQuery('.save-account').livequery('click', function () {

			// Client side validation

			var accountEntryFm = jQuery('#<portlet:namespace />accountEntryFm');
			var passwordRequired = jQuery(this).attr('passwordRequired');
			var passwordSaved = jQuery('#<portlet:namespace />passwordSaved');

			if (Liferay.Mail.sessionExpirationCheck()) {
				sendMessage('error', 'your-session-has-expired');

				return false;
			}
			else if ((accountEntryFm.find('.password:first').val() === '') && (passwordSaved.is(':checked') || (passwordRequired == '<%= true %>'))) {
				sendMessage('error', 'please-enter-a-valid-password');

				return false;
			}
			else if (accountEntryFm.find('.mail-in-host-name:first').val() === '') {
				sendMessage('error', 'please-enter-a-valid-incoming-server-name');

				return false;
			}
			else if (accountEntryFm.find('.mail-out-host-name:first').val() === '') {
				sendMessage('error', 'please-enter-a-valid-outgoing-server-name');

				return false;
			}

			// Server side validation

	 		var actionURL = Liferay.PortletURL.createActionURL();

	 		actionURL.setPortletId('1_WAR_mailportlet');
	 		actionURL.setName('saveAccount');
	 		actionURL.setWindowState('MAXIMIZED');

	 		accountEntryFm.ajaxSubmit(
				{
					url: actionURL,
					beforeSubmit: function (formData, jqForm, options) {
						sendMessage('info', 'saving-account-settings');
					},
					error: function (XMLHttpRequest, textStatus, errorThrown) {
						sendMessage('error', 'unable-to-connect-with-mail-server');
					},
					success: function(responseText, statusText) {
						var jsonResult = eval('(' + responseText + ')');

						if (jsonResult.success) {
							sendMessage('success', jsonResult.message);

							var renderURL = Liferay.PortletURL.createRenderURL();

					 		renderURL.setPortletId('1_WAR_mailportlet');
					 		renderURL.setPortletMode('<%= PortletMode.EDIT %>');
					 		renderURL.setWindowState('NORMAL');

							window.location = renderURL;
						}
						else {
							sendMessage('error', jsonResult.message);
						}
					}
				}
			);

			return false;
		});

		function sendMessage(type, key) {
			var messageType = 'portlet-msg-error';

			if (type == 'success') {
				messageType = 'portlet-msg-success';
			}
			else if (type == 'info') {
				messageType = 'portlet-msg-info';
			}

			var message = Liferay.Language.get(key);

			var currentMsg = jQuery('.lfr-message-response');

			if (message === '') {
				currentMsg.fadeOut('slow');
			}
			else if (currentMsg.length) {
				currentMsg.html(message);
				currentMsg.removeClass('portlet-msg-success').removeClass('portlet-msg-info').removeClass('portlet-msg-error');
				currentMsg.addClass(messageType);
				currentMsg.fadeIn('fast');
			}
			else {
				currentMsg = jQuery('<div class="' + messageType + ' lfr-message-response">' + message + '</div>');

				jQuery('#<portlet:namespace />accountEntryPopupStatus').prepend(currentMsg);
			}
		}

		function editAccountEntry(accountEntry, preconfigured) {
			var accountEntryId = 0;
			var titleLanguageKey = '';
			var descriptionLanguageKey = '';
			var messageAddress = '';
			var username = '';
			var passwordSaved = false;
			var useLocalPartOfEmailAddressAsUsername = false;
			var hideSettings = false;
			var mailInHostName = '';
			var mailInPort = '';
			var mailInSecure = '';
			var mailOutHostName = '';
			var mailOutPort = '';
			var mailOutSecure = '';
			var defaultFolderPrefix = '';
			var senderName = '';
			var signature = '';
			var useSignature = '';

			if (accountEntry != null) {
				if (preconfigured) {
					titleLanguageKey = accountEntry.titleLanguageKey;
					descriptionLanguageKey = accountEntry.descriptionLanguageKey;
					useLocalPartOfEmailAddressAsUsername = accountEntry.useLocalPartOfEmailAddressAsUsername;
					hideSettings = accountEntry.hideSettings;
				}
				else {
					passwordSaved = accountEntry.passwordSaved;
					username = accountEntry.username;
					senderName = accountEntry.senderName;
					signature = accountEntry.signature;
					useSignature = accountEntry.useSignature;
				}	

				accountEntryId = accountEntry.accountEntryId;
				messageAddress = accountEntry.messageAddress;
				mailInHostName = accountEntry.mailInHostName;
				mailInPort = accountEntry.mailInPort;
				mailInSecure = accountEntry.mailInSecure;
				mailOutHostName = accountEntry.mailOutHostName;
				mailOutPort = accountEntry.mailOutPort;
				mailOutSecure = accountEntry.mailOutSecure;
				defaultFolderPrefix = accountEntry.defaultFolderPrefix;
			}

			var popup = Liferay.Popup({
				title: '<liferay-ui:message key="setup-mail-account" />',
				handles: '',
				url: themeDisplay.getLayoutURL() + '/-/mail/account_entry_popup',
				urlData: {
					accountEntryId: accountEntryId,
					admin: false,
					preconfigured: preconfigured,
					titleLanguageKey: titleLanguageKey,
					descriptionLanguageKey: descriptionLanguageKey,
					messageAddress: messageAddress,
					username: username,
					passwordSaved: passwordSaved,
					useLocalPartOfEmailAddressAsUsername: useLocalPartOfEmailAddressAsUsername,
					hideSettings: hideSettings,
					mailInHostName: mailInHostName,
					mailInPort: mailInPort,
					mailInSecure: mailInSecure,
					mailOutHostName: mailOutHostName,
					mailOutPort: mailOutPort,
					mailOutSecure: mailOutSecure,
					defaultFolderPrefix: defaultFolderPrefix,
					senderName:senderName,
					signature: signature,
					useSignature: useSignature				
				},
				modal: true,
				position: ['center', 100],
				width: 550
			});
		}

		function refreshAccounts() {
			var accountEntryHtml = [];

			// Preconfigured accounts

			accountEntryHtml.push('<table class="lfr-table">');

			for (i = 0; i < preconfiguredMailAccounts.accountEntries.length; i++) {
				var accountEntry = preconfiguredMailAccounts.accountEntries[i];

				accountEntryHtml.push('<tr>');
				accountEntryHtml.push('<td>' + Liferay.Language.get(accountEntry.titleLanguageKey) + '</td>');
				accountEntryHtml.push('<td><input class="add-account" titleLanguageKey="' + accountEntry.titleLanguageKey + '" type="button" value="<liferay-ui:message key="add" />" /></td>');
				accountEntryHtml.push('</tr>');
			}

			accountEntryHtml.push('</table>');

			preconfiguredAccountsList.html(accountEntryHtml.join(''));

			// Saved accounts

			accountEntryHtml = [];

			accountEntryHtml.push('<table class="lfr-table">');

			for (i = 0; i < savedMailAccounts.accountEntries.length; i++) {
				var accountEntry = savedMailAccounts.accountEntries[i];

				accountEntryHtml.push('<tr>');
				accountEntryHtml.push('<td>' + accountEntry.messageAddress + '</td>');
				accountEntryHtml.push('<td><input class="edit-account" accountEntryId="' + accountEntry.accountEntryId + '" type="button" value="<liferay-ui:message key="edit" />" /></td>');
				accountEntryHtml.push('<td>');
				accountEntryHtml.push('<form action="<portlet:actionURL name="deleteAccount" />" class="uni-form" id="<portlet:namespace />fm" method="post" name="<portlet:namespace />deleteAccountEntryFm' + accountEntry.accountEntryId + '">');
				accountEntryHtml.push('<input id="<portlet:namespace />accountEntryId" name="<portlet:namespace />accountEntryId" type="hidden" value="' + accountEntry.accountEntryId + '" />');
				accountEntryHtml.push('<input class="delete-account" accountEntryId="' + accountEntry.accountEntryId + '" type="button" value="<liferay-ui:message key="delete" />" />');
				accountEntryHtml.push('</form>');
				accountEntryHtml.push('</td>');
				accountEntryHtml.push('</tr>');
			}

			accountEntryHtml.push('</table>');

			savedAccountsList.html(accountEntryHtml.join(''));
		}

		refreshAccounts();
	}
);
</script>